<?php
/**
* 
* ******* DON'T MODIFY THIS FILE DIRECTLY ******
* 
* This file sets up the defaults. To override
* simply create a
**********************
* Config_local.inc.php
**********************
* file.
* Copy / paste the variables you want to get a different
* value from the default one.
* 
* Example: you want to define another database name.
* Add the following line into your Config_local.inc.php file:
*
* defineOnce("XP_DB_DATABASE", "your database name");
*
* This way every time you upgrade you don't have to worry about
* resetting your configuration file - and you get all the new
* defaults.
* Just remember to always keep your local file safe and
* to put it back into the upgraded XPWeb folder.
* Give a look at the new config file to see if new params
* are available...
*/

/** **************************************
* 	Version 3.3.2
************************************** */

/**
 * If set to true, story completion is calculated as a weighed average
 *
 * DEFAULT:
 * defineOnce("XP_WEIGHED_STORY_COMPLETION", false);
 */

defineOnce("XP_WEIGHED_STORY_COMPLETION", false);

/** **************************************
* 	Version 3.2
************************************** */
/**
 * What order fields should be available for stories and in which order (order IS important)
 * The value is made of a concatenation of these available field names:
 * - ORDERBY_ID
 * - ORDERBY_TITLE
 * - ORDERBY_RISK
 * - ORDERBY_WEIGHT
 * - ORDERBY_PRIORITY
 *
 * ***** NO OTHER VALUE IS POSSIBLE *****
 *
 * The values must be separated with "."
 * Example: ORDERBY_RISK.ORDERBY_WEIGHT.ORDERBY_PRIORITY.ORDERBY_TITLE
 * is different from:
 * ORDERBY_TITLE.ORDERBY_WEIGHT.ORDERBY_RISK.ORDERBY_PRIORITY
 * ...because in the second case, if user chooses to sort stories with all
 * ascending fields in both cases,
 * if story "aaaaa" has higher weight than story "zzz" it will still be on top...
 *
 * DEFAULT:
 * defineOnce("ORDERBY_STORIES_FIELDS", ORDERBY_ID.ORDERBY_TITLE.ORDERBY_PRIORITY.ORDERBY_RISK.ORDERBY_WEIGHT);
 */
defineOnce("ORDERBY_STORIES_FIELDS", ORDERBY_ID.ORDERBY_TITLE.ORDERBY_PRIORITY.ORDERBY_RISK.ORDERBY_WEIGHT);

/** **************************************
* 	Version 3.1
************************************** */
/**
* Patterns for "hyper description"
* Describe here the patterns you want to be applied
* to all description fields (story, task, project, etc.).
*
*    Don't forget to check help files (XPWeb/Help/index.php#patternsHelp)
*
* Parameters to the "addPattern" method are the following:
* - pattern name. Will be used as a CSS class too.
*   Example "metaphorLink"
*   You can add your own CSS corresponding class in the View/XPWeb_local.css file
* - pattern "long name". Name shown to users to describe the pattern.
*   Example "Metaphor word"
* - the pattern itself. The pattern to look for.
*   Example "{_phrase_}"
* - the replacement url or pattern. The URL to use to replace the found pattern.
*   Example "Metaphor_quickWord.php?word=_phrase_"
*	if last param == XP_PATTERN_SPECIFIC, any pattern can be used
* - the replacement pattern for simple output (ie. xml)
*	Example "_phrase_"
* - the URL method. Possible values are:
*   * XP_PATTERN_LINK_TO_POPUP: a "popup" window similar to the old "metaphor" window will show up.
*   * XP_PATTERN_LINK_TO_SELF: the link will be opened in the current window
*   * XP_PATTERN_LINK_TO_NEW_WINDOW: the link will be opened in a new full sized window
*   * XP_PATTERN_SPECIFIC: no modification is made to your replacement pattern
*
* There are two ways to describe a pattern:
*
* - either by using the special word "_phrase_" => that will
*   look for a list of word. example: "{_phrase_}" will find "{user story}"
*   and "Metaphor_quickWord.php?word=_phrase_" will be replaced by
*   "Metaphor_quickWord.php?word=user story"
*
* - or by using your own perl-compatible regular expression
*   example: "\b([A-Z][a-z]+[A-Z][a-z]+([A-Z][a-z]+)*)\b" means
*   "Uppercase followed by one or more lowercase/number,
*    repeated at least twice", and
*   using "\${1}" will return the matching word.
*
* Other examples:
*
* addPattern("user",
*		   "User link",
* 		   "user{_phrase_}",
* 		   "Team.php#member__phrase_",
* 		   "user _phrase_",
* 		   XP_PATTERN_LINK_TO_NEW_WINDOW);
* 
* addPattern("url",
* 		   "A basic url",
* 		   "url:(\b[^\s]+\b)\[(\b[\w\s]+\b)\]",
* 		   "<a class=\"external\" href=\"\${1}\" title=\"\${2}\">\${2}</a>",
* 		   "\${2} (url:\${1})",
* 		   XP_PATTERN_SPECIFIC);
* 
* addPattern("emphasize",
* 		   "Emphasize text",
*		   "\*_phrase_\*",
* 		   "<span class=\"important\">_phrase_</span>",
* 		   "<em>_phrase_</em>",
*		   XP_PATTERN_SPECIFIC);
*
* See http://fr2.php.net/manual/fr/function.preg-replace.php for more patterns
*
*/

// The wiki pattern
addPattern("wiki", // Don't change this - used by CSS
		   XP_WIKI_NAME, // Don't change this - special value for localized
		   XP_WIKI_PATTERN, // The wiki pattern = \b([A-Z][a-z]+[A-Z][a-z]+([A-Z][a-z]+)*)\b

		   // -----------------------------------------------------------
		   "/cgi-bin/view/TWiki/_phrase_", // Set your wiki address here!
		   // -----------------------------------------------------------

		   "\${1}", // Don't show anything special in XML output
		   XP_PATTERN_LINK_TO_SELF // Open in current window
		   );

addPattern("metaphorLink", // Don't change this - used by CSS
		   XP_METAPHORWORD_NAME, // Don't change this - special value for localized

		   "{_phrase_}",
		   "Metaphor.php?search&amp;word=_phrase_",
		   "_phrase_", // Don't show anything special in XML output
		   XP_PATTERN_LINK_TO_POPUP // Open in special popup
		   );

addPattern("google", // Don't change this - used by CSS
		   XP_GOOGLE_NAME, // Don't change this - special value for localized

		   "google{_phrase_}",
		   "http://www.google.fr/search?q=_phrase_",
		   "_phrase_", // Don't show anything special in XML output
		   XP_PATTERN_LINK_TO_NEW_WINDOW // Open to new window
		   );

/**
* Default iteration lenght, in days.
* DEFAULT:
* defineOnce("XP_DEFAULT_ITERATION_LENGTH", 10);
*/
defineOnce("XP_DEFAULT_ITERATION_LENGTH", 10);

/** **************************************
* 	Version 3
************************************** */


/**
* Default iteration lenght, in days.
* DEFAULT:
* defineOnce("XP_DEFAULT_ITERATION_LENGTH", 10);
*/
defineOnce("XP_DEFAULT_ITERATION_LENGTH", 10);

/** **************************************
* 	Version 2.5.2
************************************** */

/**
* Number of days to be incremented by +/- signs
* for "todo" and "done" fields.
* Will be used when moving something from todo to done
* (blue arrow).
* Note: can be less than a day (ex: 0.25)
* DEFAULT:
* defineOnce("XP_DAY_INCREMENT", 1);
*/
defineOnce("XP_DAY_INCREMENT", 1);

/** **************************************
* 	Version 2.3.1
************************************** */

/**
* Submits forms over ssl... for lib_login
* and for other links
* DEFAULT:
* defineOnce("XP_SECURE_SUBMIT",false);
* ** For SSL access... just access XPWeb with
* the https:// protocol!
*/
defineOnce("XP_SECURE_SUBMIT", false);

/** **************************************
* 	Version 2.3
************************************** */

/**
* Choose between a drop down list of users or
* a standard text input for username when logging in
* DEFAULT:
* defineOnce("XP_LOGIN_LIST_USERS",true);
*/
defineOnce("XP_LOGIN_LIST_USERS",true);

/**
* Define if you want to check for newer versions when
* an admin logs in
* DEFAULT:
* defineOnce("XP_AUTOMATIC_ADMIN_VERSION_CHECK",true);
*/
defineOnce("XP_AUTOMATIC_ADMIN_VERSION_CHECK",true);

/**
* The path to the directory where images
* are temporarily created for reports section
* Current directory is XPWeb/
* DEFAULT:
* defineOnce("XP_REPORT_TEMP_PATH","View/images");
*/
defineOnce("XP_REPORT_TEMP_PATH","View/images");

/**
* Set this to true to allow users to move a task
* or story to another project
* DEFAULT:
* defineOnce("XP_ALLOW_MOVE_OTHER_PROJECTS",true);
*/
defineOnce("XP_ALLOW_MOVE_OTHER_PROJECTS",true);

/**
* Set this to true if you want not to SEE projects
* that you cannot SELECT on Projects section...
* DEFAULT:
* defineOnce("XP_CANT_SELECT_CANT_SEE_PROJECT",false);
*/
defineOnce("XP_CANT_SELECT_CANT_SEE_PROJECT",false);

/**
* If you want to use your own libraries...
* (You shouldn't need more explanation if you're
* insterested in these params ;-)
* DEFAULT:
* defineOnce("XP_JSCALENDAR_BASEDIR",	"Lib/jscalendar-1.0");
* defineOnce("XP_JPGRAPH_BASEDIR",	"Lib/jpgraph-2.0alpha/src");
* defineOnce("XP_LIB_LOGIN_BASEDIR",	"Lib/php_lib_login_includes");
* defineOnce("XP_ADODB_BASEDIR",	XP_LIB_LOGIN_BASEDIR."/adodb/");
*/
defineOnce("XP_JSCALENDAR_BASEDIR",	"Lib/jscalendar-1.0");

// Copy paste this line to your Config_local.inc.php file,
// and change it to another version of jpgraph (< 2) if you wish to 
// use PHP version less than 5 and get reports in XPWeb.
// You'll need to visit http://www.aditus.nu/jpgraph/ and download it,
// decompress it,
// and move it to the right directory (like Lib/)
// EXAMPLE :
// defineOnce("XP_JPGRAPH_BASEDIR",		"Lib/jpgraph-1.17/src");

defineOnce("XP_JPGRAPH_BASEDIR",	"Lib/jpgraph-2.0alpha2");

defineOnce("XP_LIB_LOGIN_BASEDIR",	"Lib/php_lib_login_includes");
defineOnce("XP_ADODB_BASEDIR",		XP_LIB_LOGIN_BASEDIR."/adodb/");

/** **************************************
* 	Version 2.2
************************************** */
/**
* Set XP_DISPLAY_LIGHT_PLANNING to true if you want
* the stories and tasks to be displayed in shrinked mode
* on the planning page. Set it to false to display it in
* expanded mode when you enter the planning section.
* This only affects the "default setting": tasks and stories
* can always be enlarged again...
*/
defineOnce("XP_DISPLAY_LIGHT_PLANNING",false);

/**
* Set $XP_DO_PASS_HUNDRED to true if you want
* theory completions to pass 100%
* If set to false, theory completion will always be displayed
* 100 when more than 100
*/
$XP_COMPLETION_DO_PASS_HUNDRED = true;

/**
* The hours in between you actually work...
* ** WARNING: 24H time **
* Format : hour:minutes
* DEFAULT:
* defineOnce("XP_START_HOUR","9:00");
* defineOnce("XP_END_HOUR","18:00");
*/ 
defineOnce("XP_START_HOUR","9:00");
defineOnce("XP_END_HOUR","18:00");

/** **************************************
* 	Version 2.1
************************************** */

/**
* The default language that XPWeb will use.
* Users can still choose their prefered language later.
* Use a "xx" that has a corresponding file
* xx.inc.php in the View/languages/ directory...
* Exemple:
* - en => english
* - fr => franais
* - it => italiano
* ...
*/
defineOnce("XP_DEFAULT_LANGUAGE", "en");

/** **************************************
* 	Version 2
************************************** */


/** **************************************
* 	General config
*/

/**
* The machine name on which XPWeb is running.
* Something like "localhost", "127.0.0.1", or the
* IP address of the machine.
*/
defineOnce("XP_THIS_SITE", "127.0.0.1");


/**
* Specify a path you prefer to use instead of the default one
* for keeping your PHP session info.
* A few notes:
* - On Windows/PHP < 4.0.1, use the same drive as where PHP
* is running
* - For unlucky Wintel owners, again ;-) use "/", not "\" for
* your paths (ex: C:/temp, NOT C:\temp, that would mean "C:	emp"!!)
* You'll have to check access rights to the directory you want
* to use... please refer to PHP help for details... 
* DEFAULT: ""
* (will use the default path of your php config)
*/
defineOnce("XP_SESSION_PATH", "");


/** **************************************
* 	Database config
*/

/**
* Database type
* _In theory_, can be any of:
* - access 
* - ado 
* - ado_access 
* - vfp 
* - ibase 
* - mssql 
* - mysql 
* - mysqlt 
* - maxsql 
* - oci8 
* - odbc 
* - oracle 
* - postgres 
* - postgres7 
* - sybase
*
* _WARNING_
* at this stage,
* XPWeb has been tested with only:
* - mysql
* - postgres (experimental)
*
* Please help us and give us feedback about any other database
* you use with XPWeb...
*
* DEFAULT: "mysql"
*/
defineOnce("XP_DB_TYPE", "mysql");

/**
* The host name or IP where the database is running
* Often "localhost", or "127.0.0.1".
* Often the same machine as the one running
* XPWeb.
* DEFAULT: XP_THIS_SITE
*/
defineOnce("XP_DB_HOST", XP_THIS_SITE);

/**
* Your database name
*/
defineOnce("XP_DB_DATABASE", "xpweb");

/**
* The user login you use to connect to the mysql database
*/
defineOnce("XP_DB_USER", "root");

/**
* The password that is required to connect
* to the <XP_DB_DATABASE> database,
* on the <XP_DB_HOST> machine,
* with the <XP_DB_USER> login
*/
defineOnce("XP_DB_PASS", "xpweb");

/** **************************************
* 	Dates config
*/

/**
* Your holidays that come every year.
* (comma separated)
* Format:
* 
* ** WARNING: US FORMAT **
* 
* 	"MONTH-DAY,OTHER_MONTH-OTHER_DAY"
* 	etc.
* 	
* (don't forget the comma)
* 	
* Exmple:
* defineOnce("XP_EVERY_YEAR_HOLIDAYS", "01-01,07-14");
* 
*/
defineOnce("XP_EVERY_YEAR_HOLIDAYS", "01-01,07-14");


/**
* Your specific holidays at very specific dates.
* (comma separated)
* Format:
* 
* ** WARNING: US FORMAT **
* 
* 	"YEAR-MONTH-DAY,OTHER_YEAR-OTHER_MONTH-OTHER_DAY"
* 	etc.
* 	
* Example:
* defineOnce("XP_SPECIFIC_HOLIDAYS", "2003-01-15,2004-10-10");
* 
*/
defineOnce("XP_SPECIFIC_HOLIDAYS", "");

/** **************************************
* 	Planning config
*/

/**
* The factor by which the weight of a task is multiplied
* to make it more realistic.
* This default load factor is the one that will be
* proposed when creating an iteration.
* Any custom one can be used afterall.
* (real engineering weight =
* 	ideal engineering weight * <XP_DEFAULT_LOAD_FACTOR>)
* DEFAULT: 1.6
*/
defineOnce("XP_DEFAULT_LOAD_FACTOR", 1.6);

/**
* The default working pace.
* This default working pace is the one that will be
* proposed when creating an iteration.
* Any custom one can be used afterall.
* Use 0 or 1 depending on if you want the day to be a 
* "working day". (false = holiday, true = working day)
* defineOnce("XP_MONDAY_WORK",		true);
* defineOnce("XP_TUESDAY_WORK",		true);
* defineOnce("XP_WEDNESDAY_WORK",		true);
* defineOnce("XP_THURSDAY_WORK",		true);
* defineOnce("XP_FRIDAY_WORK",		true);
* defineOnce("XP_SATURDAY_WORK",		false);
* defineOnce("XP_SUNDAY_WORK",		false);
*/
defineOnce("XP_MONDAY_WORK",		true);
defineOnce("XP_TUESDAY_WORK",		true);
defineOnce("XP_WEDNESDAY_WORK",	true);
defineOnce("XP_THURSDAY_WORK",	true);
defineOnce("XP_FRIDAY_WORK",		true);
defineOnce("XP_SATURDAY_WORK",	false);
defineOnce("XP_SUNDAY_WORK",		false);

/**
* Maximum weight in ideal days that a task can
* take (actually only constraints display)
* DEFAULT: 10
*/
defineOnce("XP_TASK_MAX_WEIGHT", 10);

/**
* Maximum priority to affect to a story
* (actually only constraints display)
* DEFAULT: 10
*/
defineOnce("XP_STORY_MAX_PRIORITY", 10);

/**
* Maximum story risk
* (actually only constraints display)
* DEFAULT: 10
*/
defineOnce("XP_STORY_MAX_RISK", 10);

/**
* Maximum weight in any unit that a story can
* take (actually only constraints display)
* DEFAULT: 10
*/
defineOnce("XP_STORY_MAX_WEIGHT", 10);

/** **************************************
*	Authentication config
*/

/**
* Don't bother if this is your first install of XPWeb.
* Password that will be used for existing team members
* from version 1.x
* DEFAULT: "oldschool" (don't try to find an explanation :o)
*
* ** ** ** WARNING ** ** **
* DON'T set this to empty string...
*
*/
defineOnce("XP_DEFAULT_PASSWORD", "oldschool");

/**
* Admin user of XPWeb: all rights on any projects
* DEFAULT: XP_DB_USER
* So you've got the same database / admin user.
* You might want to have different for safety reasons.
*/
defineOnce("XP_UBER_USER", XP_DB_USER);

/**
* Admin password
* DEFAULT: XP_DB_PASS
* So you've got the same database / admin user.
* You might want to have different for safety reasons.
*/
defineOnce("XP_UBER_PASS", XP_DB_PASS);

/**
* Timeout in seconds after what a user is
* automatically logged out
* DEFAULT: 1200 (20 minutes)
* Note: needs to be superior to 15!!
*/
defineOnce("XP_TIMEOUT_IN_SECONDS", 1200);

/**
* If you want the users to be notified before they are
* timed out...
* 
* DEFAULT: true
*/
defineOnce("XP_DO_NOTIFY_BEFORE_TIMEOUT", true);

/**
* Time before you want to be notified
* that the timeout is close
* DEFAULT: 60 seconds
* NOTE: only has sense if XP_DO_NOTIFY_BEFORE_TIMEOUT
* is set to true...
*/

defineOnce("XP_NOTIFY_TIMEOUT_TIME_IN_SECONDS", 60);

/**
* Minimum password length (number of
* letters)
* DEFAULT: 5
*/
defineOnce("XP_MIN_PASSWORD_LENGTH", 5);

/**
* Do you want to punish bad attemps to login with
* a time while the account is
* blocked? (true or false)
* DEFAULT: false
*/
defineOnce("XP_PUNISH_BAD_ATTEMPTS", false);

/**
* If XP_PUNISH_BAD_ATTEMPTS == true, set
* how many  bad attempts are accepted before
* the account is blocked for an amount of time
* DEFAULT: 3
*/
defineOnce("XP_BAD_ATTEMPTS_MAX", 3);

/**
* The amount of time discussed just before...
* in seconds (only if XP_PUNISH_BAD_ATTEMPTS == true)
* DEFAULT: 120 (2 minutes)
*/
defineOnce("XP_BAD_ATTEMPTS_WAIT", 120);
?>
