<?php

// *********************************************************************
// Class MetaphorControler
// *********************************************************************
class MetaphorControler extends XPItemListControler {
	
	function dbPopulate() {
		if ($this->item->isSearch()) {
			$this->searchPopulate($this->item->searchWord);
		} else {
			$this->_private_dbPopulate();
		}
	}
	
	function searchPopulate($search) {
		/**
		* Try to find the EXACT word
		*/
		$wordsFound = $this->findWords($search);
		
		if (empty($wordsFound)) {
			/**
			* Try to find the SIMILAR words
			*/
			$wordsFound = $this->findSimilarWords($search);
		}
		
		if (empty($wordsFound)) {
			/**
			* Try to find the words TALKING ABOUT this word
			*/
			$wordsFound = $this->findLinkedWords($search);
		}
		
		if (empty($wordsFound)) {
			return false;
		}
	}
	
	function findWords($word, $firstTry = true) {
		

		$word = urldecode($word);
		$customConstraints = "LOWER(name) = " . $GLOBALS["gDb"]->quote($word);
		$results = $this->_private_dbPopulate($customConstraints);

		if ($firstTry && empty($results)) {
			/**
			* Try without the "s" at the end
			*/
			$word = substr($word, 0, -1);
			$results = $this->findWords($word, false);
		}
		return $results;
	}
	
	function findSimilarWords($word, $firstTry = true) {
		

		$customConstraints = "LOWER(name) LIKE " . $GLOBALS["gDb"]->quote("%$word%");
		
		$results = $this->_private_dbPopulate($customConstraints);

		if ($firstTry && empty($results)) {
			/**
			* Try without the "s" at the end
			*/
			$word = substr($word, 0, -1);
			$results = $this->findSimilarWords($word, false);
		}
		return $results;
	}
	
	function findLinkedWords($word, $firstTry = true) {
		
		$customConstraints = "LOWER(description) LIKE " . $GLOBALS["gDb"]->quote("%$word%");
		
		$results = $this->_private_dbPopulate($customConstraints);
		if ($firstTry && empty($results)) {
			/**
			* Try without the "s" at the end
			*/
			$word = substr($word, 0, -1);
			$results = $this->findLinkedWords($word, false);
		}
		return $results;
	}
	
	function _private_dbPopulate($customConstraints = "") {
		
		
		$selectFields = "*";
		
		$selectTableNames = array(
			$this->dbIncludedTableName()
		);
				
		$selectConstraints = array(
			"project_id" => $this->item->projectId
		);
		
		$selectGroupBy = "";/*array(
			"id"
		);*/
		
		$selectOrderBy = array(
			"name asc"
		);
		
		$result = $GLOBALS["gDb"]->selectEntries(
			$selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy,
			$customConstraints);
		
		while (!$result->EOF) {
			
			$factory = & MetaphorFactory::GetFactory();
			
			$fields = $this->dbIncludedFields($result);

			$metaphorWord = & $factory->newMetaphorWord($fields);
			
			$this->item->addIncluded($metaphorWord);
			
			unSet($metaphorWord);
			
			$result->MoveNext();
		}
		
		return $this->item->includedItems;
	}
		
}

// *********************************************************************
// Class MetaphorWordControler
// *********************************************************************
class MetaphorWordControler extends XPItemControler {

	// /////////////////////////////////////////////////////////////
	// DB METHODS
	// /////////////////////////////////////////////////////////////

	/// DB table name
	function dbGetTableName() {
		return "metaphor_words";
	}

	/// Correspondance <db column name> <=> <class property name>
	function dbProperties() {

		// db column name => class property name
		$fields = array(
			"id"			=> "id",
			"name"			=> "title",
			"description"	=> "description",
			"project_id"	=> "projectId"
		);

		return $fields;
	}


}
?>
