<?php

// *********************************************************************
// Class TaskControler
// *********************************************************************

class TaskControler extends XPItemControler {

	// /////////////////////////////////////////////////////////////
	// DB METHODS
	// /////////////////////////////////////////////////////////////

	/// DB table name
	function dbGetTableName() {
		return "tasks";
	}

	/// Correspondance <db column name> <=> <class property name>
	function dbProperties() {

		// db column name => class property name
		$fields = array(
			"responsible_team_member_id"	=> "memberId",
			"pair_team_member_id"			=> "pairId",
			"name"							=> "title",
			"description"					=> "description",
			"start_date"					=> "startDate",
			"end_date"						=> "endDate",
			"auto"							=> "autoUpdate",
			"story_id"						=> "storyId",
			"iteration_id"					=> "iterationId",
			"project_id"					=> "projectId",
			"done"							=> "done",
			"todo"							=> "todo",
			"working_days_name"				=> "workingDaysName"
		);

		return $fields;
	}



	/**
	* DB update functions
	*/

	function dbUpdateInDb() {
		
		
		// Check if has been moved
		$hasMovedFields = array(
			"id"
		);
		$hasMovedTableNames = array(
			$this->dbGetTableName()
		);
		$hasMovedConstraints = array(
			"id" => $this->item->id,
			"story_id" => $this->item->storyId,
			"iteration_id" => $this->item->iterationId
		);
		
		$result = & $GLOBALS["gDb"]->selectOneEntry($hasMovedTableNames, $hasMovedConstraints, $hasMovedFields);
		
		$hasBeenMoved = $result->EOF;
		
		if ($this->item->isAllowedToUpdate()) {
		
	 		$this->dbUpdateDescription();
	 		
	 		if ($hasBeenMoved) {
	 			// Load factor and working days might have changed
	 			// after the task has been moved: need to recalculate
	 			// end date
				echoDebug("HAS BEEN MOVED");
				
				echoDebug("BEFORE: ".$this->item->endDate);

	 			$this->dbPopulateForCalculation();
	 			
	 			$this->dbUpdateAutoEndDate();
	 			
	 			echoDebug("AFTER:  ".$this->item->endDate);
	 		}
	 		$this->dbUpdateTimingFields();
	 		
	 		$this->item->setItemModified();

		} else {
			echoDebug("***NOT ALLOWED TO UPDATE TASK***");
		}
	}
	
	/**
	 * Updates the end date in db if necessary...
	 */
	function dbUpdateAutoEndDate() {
		
		if ($this->item->autoUpdate && $this->item->theoryEndCompletion == XP_IMPOSSIBLE_ID) {
			// The end date is automatically calculated,
			// and the tasks is not terminated...
			// => recalculate it!
			$this->item->calculateEndDate();
			
			$tableName = $this->dbGetTableName();
    		
    		$fields = array();
    		$fields["end_date"] = $this->item->endDate;
    		
    		$constraints = array(
    			"id" => $this->item->id
    		);
    		
    		$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
		}
	}
	
	/**
	* This method only updates description fields
	* such as title description, done, todo etc.
	*/
	function dbUpdateDescription() {
		
		
		$tableName = $this->dbGetTableName();
		$constraints =  array(
			"id" => $this->item->id
		);
		
		$fields = $this->dbGetFields();

		if (isForbidden($fields["story_id"])) {
			echoDebug("isForbidden story: ".$fields["story_id"]);
			unSet($fields["story_id"]);
		}
		
		if (isset($fields["iteration_id"])
			&& isForbidden($fields["iteration_id"])) {
			echoDebug("isForbidden iteration: ".$fields["iteration_id"]);
			unSet($fields["iteration_id"]);
		}
		
		// Check if allready exists
		$selectFields = array(
			"setup_name"
		);
		$selectTableNames = array(
			"working_days"
		);
		$selectConstraints = array(
			"setup_name" => $this->item->workingDaysName
		);
		
		$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields);

		$customExists = ($result->RecordCount() != 0);
		
		if (!$this->item->isDefaultWorkingDays) {
			// Custom working days
			
			$tableNameDays = "working_days";
			
			$fieldsDays = $this->item->workingDays;
			
			if ($customExists) {
				// UPDATE custom entry
				
				$constraintsDays = array(
					"setup_name" => $this->item->workingDaysName
				);
				
				$GLOBALS["gDb"]->updateTable($tableNameDays, $fieldsDays, $constraintsDays);
				
			} else {
				// INSERT custom entry
				
				$fieldsDays["setup_name"] = $this->item->workingDaysName;
				
				$GLOBALS["gDb"]->insertIntoTable($tableNameDays, $fieldsDays);
			}
		} else {
			// Default working type
			// Iteration: default working days from iteration
			if ($customExists) {
				// No more custom: need to delete custom entry (if any)
				$selectConstraints = array(
					"setup_name" => "task_".$this->item->id
				);
				$GLOBALS["gDb"]->deleteFromTable("working_days", $selectConstraints);
			}
		}
		
		$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
	}
	
	/**
	* This method only updates calculated fields
	* start date (which is set by user but might be updated
	* because of working days), theory end completion
	*/
	function dbUpdateTimingFields() {
		
		
		$tableName = $this->dbGetTableName();
		
		// *********** Weight update
		
		$fields = array();
	
		$constraints = array(
			"id" => $this->item->id
		);
		
		if ($this->item->isStarted()) {
			// The task is started (done != 0)
			// => it's time to update weight in db if it's not updated yet...
			$fields["weight"] = $this->item->getCalculatedWeight();
			//echoDebug("dbUpdateTimingFields: task started, will try to set db weight to ".$this->item->weight);
			$constraints["weight"] = XP_IMPOSSIBLE_ID;
		} else {
			// The task is not started: whatever happens the weight is
			// forced in db to -1
			$fields["weight"] = XP_IMPOSSIBLE_ID;
			//echoDebug("dbUpdateTimingFields: task not started, set db weight to ".XP_IMPOSSIBLE_ID);
		}
		
		// Time to update weight with
		// 	- if started:
		//		- if weigth = -1
		//		=> the calculated value
		//		- else
		//		=> no change
		//	- if not started:
		//		set to weigth = -1
		$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
		
		// *********** End date and end theory completion update
		
		// Update fields...
		$this->item->calculateActualCompletion();
		$this->item->calculateTheoryCompletion();
		
		// By default we don't set the end date and let the theory completion to -1
		//$lbEndDate = false;
		$this->item->theoryEndCompletion = XP_IMPOSSIBLE_ID;
		
		$constraints = array(
			"id" => $this->item->id
		);
		
		if ($this->item->isCompleted()) {
			// if actualCompletion >= 100, task is now completed
			// => that means theoryEndCompletion needs to be updated
			$this->item->theoryEndCompletion = $this->item->theoryCompletion;
			// It's also time to register end date to db

			$fields = $this->getDbCalculatedFields();
			//echoDebug("dbUpdateTimingFields: task completed, will try update end date and theoryEndCompletion in DB");
			
			// if theoryEndCompletion == -1, task was "never completed"
			// so we'll update it only if that's the case...
			$constraints["theory_end_comp"] = XP_IMPOSSIBLE_ID;
			//$constraints["end_date"] = XP_FORBIDDEN_DATE;
		} else {
			// We will update with init values...
			$fields = $this->getDbCalculatedFields(false);
		}
		
		// Time to update end date and end theory completion with
		// 	- if completed:
		//		- if end date = "0000-00-00" AND theory end comp = -1
		//		=> the calculated values
		//		- else
		//		=> no change
		//	- if not completed:
		//		set to end date = "0000-00-00" AND theory end comp = -1
		$tableName = $this->dbGetTableName();

		$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
	}
	
	function dbAddToDb() {
		
		$this->item->id = 0;
		
		$tableName = $this->dbGetTableName();
		
		$allowed = $this->item->isAllowedToInsertIfResponsible($GLOBALS["gConnectedUser"]->id, $GLOBALS["gConnectedUser"]->name);

		if ($allowed) {
			
			//echoDebug("allowed to insert task");
			
			// Forced to calculated values so we're sure
	 		// they're updated if necessary
	 		$this->item->setWeightFromDb($this->item->getCalculatedWeight());
	 		$this->item->theoryEndCompletion = XP_IMPOSSIBLE_ID;
	 		
	 		//$this->item->_getDBTheoryCompletion();
	 		
	 		$fields = $this->dbGetFields();

			echoDebug("task end date : ".$this->item->endDate);
			
			if (!$this->item->isDefaultWorkingDays) {
				// Custom
				
				// First: insert
				// But without working days name !!
				unset($fields["working_days_name"]);
				
				$this->dbInsertItem($tableName, $fields);
				
				//echoDebug("task added: ".$this->item->id);
				
				$this->item->workingDaysName = "task_".$this->item->id;
				
				// Second: insert entry in working_days
				
				// INSERT custom entry
				$tableNameDays = "working_days";
				$fieldsDays = $this->item->workingDays;

				$fieldsDays["setup_name"] = $this->item->workingDaysName;
				
				$GLOBALS["gDb"]->insertIntoTable($tableNameDays, $fieldsDays);
				
				// Third: now update Task with working_day foreign key
				
				unset($fields);
				$fields["working_days_name"] = $this->item->workingDaysName;;
				
				$constraints =  array(
					"id" => $this->item->id
				);
				
				$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
			} else {
				// Iteration: default working days from iteration
				$fields["working_days_name"] = "ite_".$this->item->iterationId;
				// Just need to insert: entry in working_days allready
				// exists for iteration
				$this->dbInsertItem($tableName, $fields);
			}
			
			$this->dbUpdateTimingFields();
			
			//echoDebug("TASK added: ".$this->item->id);
		}
	}
	
	function dbMove($projectId, $iterationId, $storyId) {
		
		// No verification: called from Iteration or Story class,
		// where the verification is done...

		$tableName = $this->dbGetTableName();
		$constraints =  array(
			"id" => $this->item->id
		);
		$fields = array(
			"project_id" => $projectId,
			"iteration_id" => $iterationId,
			"story_id" => $storyId
		);
		
		if (isset($this->item->memberId)) {
			// In case we moved project,
			// user might have been reseted...
			$fields["responsible_team_member_id"] = $this->item->memberId;
		}

		if (isset($this->item->pairId)) {
			// In case we moved project,
			// user might have been reseted...
			$fields["pair_team_member_id"] = $this->item->pairId;
		}

		$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
		
		
		/**
		* Update working days if default
		* (if working days name is DIFFERENT from custom)
		*/
		$query = "UPDATE tasks SET working_days_name = " . $GLOBALS["gDb"]->quote("ite_".$iterationId) . " WHERE (id = " . $GLOBALS["gDb"]->quote($this->item->id) ." AND working_days_name <> " . $GLOBALS["gDb"]->quote("task_".$this->item->id) .");";
		
		$GLOBALS["gDb"]->executeQuery($query);
		
		// Load factor and working days might have changed
		// after the task has been moved: need to recalculate
		// end date
		//$this->dbUpdateEndDateAfterMoved();
		// Actually we don't need to update anything. Things are re-calculated
		// when displayed or they are registered for good (completed task)
	}
	
	function dbDeleteFromDb() {

		if ($this->item->isAllowedToDelete()) {
			$this->dbDoDeleteFromDb();
		
			if ($this->item->storyId != 0) {
				XPItem::class_itemModified("story", $this->item->storyId);
			} else {
				XPItem::class_itemModified("iteration", $this->item->iterationId);
			}
		}
	}
	
	function dbDoDeleteFromDb() {
		
		$selectFields = array(
			"iteration_id",
			"story_id",
			"working_days_name"
		);
		$selectTableNames = array(
			$this->dbGetTableName()
		);
		$selectConstraints = array(
			"id" => $this->item->id
		);
		
		$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields);

		$deleteTableName = $this->dbGetTableName();
		$deleteConstraints = array(
			"id" => $this->item->id
		);
		$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $deleteConstraints);
		
		$workingDaysEntry = $result->Fields("working_days_name");
		
		
		$customWorkingId = "task_".$this->item->id;
		if ($workingDaysEntry == $customWorkingId) {
			// custom entry: needs to be deleted

			$deleteTableName = "working_days";
			$deleteConstraints = array(
				"setup_name" => $customWorkingId
			);
			$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $deleteConstraints);
		}
	}
	
	/**
	* This method populates fields that will be used
	* to calculate "timing" fields
	*/
	function dbPopulateForCalculation() {
		
		$selectFields = "*";

		$selectTableNames = array(
			$this->dbGetTableName(),
			"working_days"
		);
		$selectConstraints = array(
			"id" => $this->item->id,
			"working_days_name" => "working_days.setup_name"
		);
		
		$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields);
		
		$this->item->workingDays = array (
			"Mon" => $result->Fields("Mon"),
			"Tue" => $result->Fields("Tue"),
			"Wed" => $result->Fields("Wed"),
			"Thu" => $result->Fields("Thu"),
			"Fri" => $result->Fields("Fri"),
			"Sat" => $result->Fields("Sat"),
			"Sun" => $result->Fields("Sun")
		);
		//$this->item->loadFactor = $result->Fields("load_factor");
		$this->dbUpdateLFFromDb();

		$this->item->done = $result->Fields("done");
		$this->item->todo = $result->Fields("todo");
		$this->item->weight = $result->Fields("weight");
		
		$this->item->setStartDate($result->Fields("start_date"));
		$this->item->setEndDate($result->Fields("end_date"));
		$this->item->theoryEndCompletion = $result->Fields("theory_end_comp");
	}
	
	function getDbCalculatedFields($calculated = true) {
		if ($calculated) {	
	 		$fields =  array(
	 			"theory_end_comp" => $this->item->theoryCompletion
	 			//"end_date" => $this->item->endDate
	 		);
		} else {
			$fields =  array(
	 			"theory_end_comp" => XP_IMPOSSIBLE_ID
	 			//"end_date" => XP_FORBIDDEN_DATE
	 		);
		}
		return $fields;
	}
	
	function dbUpdateLFFromDb() {
		
		// If isolated
		if ($this->item->iterationId == 0) {
			$this->item->loadFactor = XP_DEFAULT_LOAD_FACTOR;
		} else {
			$selectFields = array(
	 			"load_factor"
	 		);
	 		$selectTableNames = array(
	 			"iterations"
	 		);
	 		$selectConstraints = array(
	 			"id" => $this->item->iterationId
	 		);
	 		
	 		$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields);
	 		$this->item->loadFactor = $result->Fields("load_factor");
		}
	}

	function getPropertiesForDelete() {
		
		return array("memberId", "storyId", "iterationId");
	}

	/**
	* STATIC functions
	*/

	// /////////////////////////////////////////////////////////////
	// HTTP METHODS
	// /////////////////////////////////////////////////////////////
	
	function & httpGetPostItem($val) {
		global $XP_DATE_FORMAT;
		
		$lbDoChangeProject = false;
		
		if ($val["id"] < 0) {
			$val["id"] = - $val["id"];
		}
		if (isset($val["iterationId"]) && isset($val["storyId"])) {
			$iterationId = $val["iterationId"];
			$storyId = $val["storyId"];
		} else {
			echoDebug("========>".$val["iterationStoryId"]);
			if (isForbidden($val["iterationStoryId"])) {
				// isolate
				$iterationId = 0;
				$storyId = 0;
			} else {
	 			list($iterationId,$storyId) = @explode("_", $val["iterationStoryId"]);
	 			if ($iterationId == "proj") {
	 				echoDebug("changeProj!");
	 				$lbDoChangeProject = true;
	 				// That was "proj_id", ie. moving to another project
	 				$projectId = $storyId;
	 				// It'll be isolated
	 				$iterationId = 0;
	 				$storyId = 0;
	 			}
			}
		}
		
		if ($val["wkType"] == "default") {
			// Default working type
			// Iteration: default working days from iteration
			$workingDaysName = "ite_".$iterationId;
		} else {
			$workingDaysName = "task_".$val["id"];
		}
		$newWorkingPace = explode("|",$val["wp"]);
		
		$mon = (int)in_array("Mon", $newWorkingPace);
		$tue = (int)in_array("Tue", $newWorkingPace);
		$wed = (int)in_array("Wed", $newWorkingPace);
		$thu = (int)in_array("Thu", $newWorkingPace);
		$fri = (int)in_array("Fri", $newWorkingPace);
		$sat = (int)in_array("Sat", $newWorkingPace);
		$sun = (int)in_array("Sun", $newWorkingPace);
			
			
		$workingDays = array(
			"Mon" => $mon,
			"Tue" => $tue,
			"Wed" => $wed,
			"Thu" => $thu,
			"Fri" => $fri,
			"Sat" => $sat,
			"Sun" => $sun
		);
		
		$factory = & PlanningFactory::GetFactory();
			
		$theTask = & $factory->newTask(
			array(
				"id" 					=> $val["id"],
				"projectId" 				=> getCurrentProjectId(),
				"loadFactor" 			=> $val["loadFactor"],
				"title" 					=> $val["title"],
				"description" 			=> $val["description"],
				"workingDays" 			=> $workingDays,
				"workingDaysName" 		=> $workingDaysName,
				"storyId" 				=> $storyId,
				"iterationId" 			=> $iterationId,
				"startDate" 				=> @dateFromHtmlToSql($val["startDate"]),
				"endDate" 				=> @dateFromHtmlToSql($val["endDate"]),
				"theoryEndCompletion" 	=> $val["theoryEndCompletion"],
				"weight" 				=> -1,
				"done" 					=> $val["done"],
				"todo" 					=> $val["todo"],
				"memberId" 				=> $val["memberId"],
				//"memberName" 			=> "",
				"pairId" 				=> $val["pairId"],
				//"pairName"				=> "",
				"iterationDeadlineTime"	=> "",
				"autoUpdate"				=> (@$val["auto"]==1)
			)
		);
		
		if ($lbDoChangeProject) {
			$theTask->setNewProject($projectId);
		}
		
		return $theTask;
	}
	
	function getObjectFromDb($id) {
		

		// @todo use DBFIELDS
		
		//echoDebug("getObjectFromDb id $id");
		
		$selectFields = "*";

		$selectTableNames = array(
			TaskControler::dbGetTableName(),
			"working_days"
		);
		$selectConstraints = array(
			"id" => $id,
			"working_days_name" => "working_days.setup_name"
		);
		
		$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields);
		
		if ($result->EOF) {
			//echoDebug("getObjectFromDb no task found in db with id $id");
			return null;
		} else {
	 		
	 		// Get iteration info
			$iterationId = $result->Fields("iteration_id");
			if ($iterationId == 0) {
	 			$iterationDeadLine = 
		  			mktime(0,0,0,date("m",XP_FORBIDDEN_DATE),
		  			date("d",XP_FORBIDDEN_DATE),date("Y",XP_FORBIDDEN_DATE));
			} else {
	 			$selectTableNames = array(
	 				"iterations"
	 			);
	 			$selectConstraints = array(
	 				"id" => $iterationId
	 			);
	 			
	 			$iterationResult = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints);
	 			$iterationDeadLine = $iterationResult->Fields("date");
			}
	 		
	 		// Call to internal function that returns the Task object
	 		// SHIT! Why the hell doesn't it work????
			$factory = & new PlanningFactory();//PlanningFactory::GetFactory();
						
			$theTask = & $factory->newTask(
				array(
					"id" 					=> $result->Fields("id"),
					"projectId" 			=> $result->Fields("project_id"),
					"loadFactor" 			=> $result->Fields("load_factor"),
					"title" 				=> $result->Fields("name"),
					"description" 			=> $result->Fields("description"),
					"workingDays" 			=> array (
											   	"Mon" => $result->Fields("Mon"),
											   	"Tue" => $result->Fields("Tue"),
											   	"Wed" => $result->Fields("Wed"),
											   	"Thu" => $result->Fields("Thu"),
											   	"Fri" => $result->Fields("Fri"),
											   	"Sat" => $result->Fields("Sat"),
											   	"Sun" => $result->Fields("Sun")
											   ),
					"workingDaysName" 		=> $result->Fields("working_days_name"),
					"storyId" 				=> $result->Fields("story_id"),
					"iterationId" 			=> $iterationId,
					"startDate" 			=> $result->Fields("start_date"),
					"endDate" 				=> $result->Fields("end_date"),
					"theoryEndCompletion" 	=> $result->Fields("theory_end_comp"),
					"weight" 				=> $result->Fields("weight"),
					"done" 					=> $result->Fields("done"),
					"todo" 					=> $result->Fields("todo"),
					"memberId" 				=> $result->Fields("responsible_team_member_id"),
					"pairId" 				=> $result->Fields("pair_team_member_id"),
					"iterationDeadlineTime" => $iterationDeadLine,
					"autoUpdate"			=> ($result->Fields("auto")==1)
				)
			);
			
			$theTask->controler->dbUpdateLFFromDb();
			
			return $theTask;
		}	
	}

}


// *********************************************************************
// Class TaskContainerControler
// *********************************************************************
class TaskContainerControler extends XPItemControler {

	function dbPopulateTasks($iterationId, $storyId, $deadlineTime) {
		
		

		$selectFields = "*";

		$selectTableNames = array(
			"tasks",
			"working_days"
		);

		$selectConstraints = array(
			"project_id"		=> $this->item->projectId,
			"working_days_name"	=> "working_days.setup_name"
		);

		if ($iterationId != XP_IMPOSSIBLE_ID) {
			$selectConstraints["iteration_id"] = $iterationId;
		}

		if ($storyId != XP_IMPOSSIBLE_ID) {
			$selectConstraints["story_id"] = $storyId;
		}

		$selectGroupBy = "";/*array(
			"tasks.id"
		);*/
		$selectOrderBy = array(
			"start_date, end_date asc"
		);
		
		$customConstraints = $this->dbCustomPopulateConstraintsTasks();
		
		$result = $GLOBALS["gDb"]->selectEntries(
			$selectTableNames, $selectConstraints, $selectFields,
			$selectGroupBy, $selectOrderBy, $customConstraints);
		
		while (!$result->EOF) {
			 
	 		// Call to internal function that returns the Task object
			$factory = & PlanningFactory::GetFactory();
		
			$taskToAdd = & $factory->newTask(
				array(
					"id" 					=> $result->Fields("id"),
					"projectId" 			=> $this->item->projectId,
					"loadFactor" 			=> $this->item->loadFactor,
					"title" 				=> $result->Fields("name"),
					"description" 			=> $result->Fields("description"),
					"workingDays" 			=> array (
											   	"Mon" => $result->Fields("Mon"),
											   	"Tue" => $result->Fields("Tue"),
											   	"Wed" => $result->Fields("Wed"),
											   	"Thu" => $result->Fields("Thu"),
											   	"Fri" => $result->Fields("Fri"),
											   	"Sat" => $result->Fields("Sat"),
											   	"Sun" => $result->Fields("Sun")
											   ),
					"workingDaysName" 		=> $result->Fields("working_days_name"),
					"storyId" 				=> $result->Fields("story_id"),
					"iterationId" 			=> $result->Fields("iteration_id"),
					"startDate" 			=> $result->Fields("start_date"),
					"endDate" 				=> $result->Fields("end_date"),
					"theoryEndCompletion" 	=> $result->Fields("theory_end_comp"),
					"weight" 				=> $result->Fields("weight"),
					"done" 					=> $result->Fields("done"),
					"todo" 					=> $result->Fields("todo"),
					"memberId" 				=> $result->Fields("responsible_team_member_id"),
					"pairId" 				=> $result->Fields("pair_team_member_id"),
					"iterationDeadlineTime" => $deadlineTime,
					"autoUpdate"			=> ($result->Fields("auto")==1)
				)
			);

						
			$this->item->addTask($taskToAdd);
			unSet($taskToAdd);
			$result->MoveNext();
		}

	}

	function dbCustomPopulateConstraintsTasks() {
		
		$customConstraints = "";

		if (!$this->item->isAllResponsible()) {
			$customConstraints =
				"tasks.responsible_team_member_id = ".$GLOBALS["gDb"]->quote($this->item->nTasksResponsibleId);
		}

		if (!$this->item->isAllPair()) {
			$customConstraints =
				"tasks.pair_team_member_id = ".$GLOBALS["gDb"]->quote($this->item->nTasksPairId);
		}

		return $customConstraints;
	}
}

// *********************************************************************
// Class StoryControler
// *********************************************************************

class StoryControler extends TaskContainerControler {

	/**
	* DB update functions
	*/
	
	function dbUpdateInDb() {
		
		
		if ($this->item->isAllowedToUpdate()) {
			// Check if has been moved
	 		$hasMovedFields = array(
	 			"id"
	 		);
	 		$hasMovedTableNames = array(
	 			"stories"
	 		);
	 		$hasMovedConstraints = array(
	 			"id" => $this->item->id,
	 			"project_id" => $this->item->projectId,
	 			"iteration_id" => $this->item->iterationId
	 		);

	 		$result = & $GLOBALS["gDb"]->selectOneEntry($hasMovedTableNames, $hasMovedConstraints, $hasMovedFields);
	 		
	 		$hasBeenMoved = $result->EOF;
		  		
			$tableName = "stories";
			$constraints =  array(
				"id" => $this->item->id
			);
			$fields = $this->dbGetFields();
			$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
			
			if ($fields["iteration_id"] == XP_IMPOSSIBLE_ID) {
				unSet($fields["iteration_id"]);
			}
			
			if ($hasBeenMoved) {
				// Move included tasks as well...
				$this->dbMoveIncludedTasks($this->item->iterationId, $this->item->id, $this->item->projectId);
			}
			
			$this->item->setItemModified();
		}
	}
	
	function dbMove($newProject, $newIteration) {
		
		// No verification: called from Iteration class,
		// where the verification is done...

		$tableName = "stories";
		$constraints =  array(
			"id" => $this->item->id
		);
		$fields = array(
			"project_id" => $newProject,
			"iteration_id" => $newIteration
		);
		$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
		
		// "Move" included tasks
		$this->dbMoveIncludedTasks($newIteration, $this->item->id, $newProject);
	}
	
	function dbAddToDb() {
		
		if ($this->item->isAllowedToInsert()) {	
			$tableName = "stories";
			
			$this->dbInsertItem($tableName, $this->dbGetFields());
		}
	}
	
	function dbMoveIncludedTasks($newIteration, $newStory, $projectId) {
		
		$selectFields = array(
			"id",
			"responsible_team_member_id"
		);
		$selectTableNames = array(
			"tasks",
		);
		$selectConstraints = array(
			"story_id" => $this->item->id
		);
		
		$result2 = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields);
		
		while (!$result2->EOF) {
			$val2 = array(
				"id"		=> $result2->Fields("id"),
				"memberId"	=> $result2->Fields("responsible_team_member_id"),
				"pairId"	=> $result2->Fields("pair_team_member_id")
			);
			
			@($task = TaskControler::httpGetPostItem($val2));
			$task->controler->dbMove($projectId, $newIteration, $newStory);
			
			$result2->MoveNext();
		}
	}
	
	function dbDeleteIncludedTasks() {
		
		$selectFields = array(
			"id"
		);
		$selectTableNames = array(
			"tasks"
		);
		$selectConstraints = array(
			"story_id" => $this->item->id
		);
		
		$result2 = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields);

		while (!$result2->EOF) {
			$val2 = array(
				"id" => $result2->Fields("id")
			);
			@($task = & TaskControler::httpGetPostItem($val2));
			$task->controler->dbDoDeleteFromDb();
			$result2->MoveNext();
		}
	}
	
	function dbDeleteFromDb($post) {		
		if ($this->item->isAllowedToDelete()) {
			$this->dbDoDeleteFromDb($post);

			XPItem::class_itemModified("iteration", $this->item->iterationId);
		}
	}
	
	function dbDoDeleteFromDb($post) {
		
		
		$selectFields = array(
			"iteration_id"
		);
		$selectTableNames = array(
			"stories"
		);
		$selectConstraints = array(
			"id" => $this->item->id
		);
		
		$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields);
		
		$oldIteration = $result->Fields("iteration_id");
		
		if ($post["whatTasks"] == "delete") {
		// Delete all tasks included in this story
			$this->dbDeleteIncludedTasks();
			$iterationId = $oldIteration;
			$storyId = XP_IMPOSSIBLE_ID;
			$taskId = 0;
		} else if ($post["whatTasks"] == "move") {
			
			echoDebug("POST : ".$post["iterationStoryId"]);
			
			// Move all tasks included in this story
			list($newIteration, $newStory) = explode("_",$post["iterationStoryId"]);
			
			$newProject = $this->item->projectId;
			
			if ($newIteration == "proj") {
				$newIteration = 0;
				$newProject = $newStory;
				$newStory = 0;
			}
			
			$this->dbMoveIncludedTasks($newIteration, $newStory, $newProject);
			$iterationId = $newIteration;
			$storyId = $newStory;
		}
		
		$deleteTableName = "stories";
		$deleteConstraints = array(
			"id" => $this->item->id
		);
		$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $deleteConstraints);
	}
	
	function dbProperties() {
		// db column name => class property name
		$fields = array(
			"iteration_id"			=> "iterationId",
			"name"					=> "title",
			"description"			=> "description",
			"validation_criteria"	=> "validationCriteria",
			"weight"				=> "weight",
			"risk"					=> "risk",
			"priority"				=> "priority",
			"project_id"			=> "projectId"
		);

		return $fields;
	}

	
	function dbCustomPopulateConstraintsTasks() {
		return TaskContainerControler::dbCustomPopulateConstraintsTasks();
	}
	
	function dbPopulate() {
		$this->dbPopulateTasks(XP_IMPOSSIBLE_ID, $this->item->id, $this->item->iterationDeadlineTime);
	}
	
	function dbPopulateForDelete() {
		
		// We need:
		// - iteration id for select
		// - nb of tasks to know if a select has to be displayed...
		
		$selectFields = "stories.iteration_id, count( * ) AS nb_tasks";

		$selectTableNames = array(
			"stories",
			"tasks"
		);
		$selectConstraints = array(
			"stories.id" => $this->item->id,
			"tasks.story_id" => "stories.id"
		);
		
		$selectGroupBy = array(
			"stories.iteration_id"
		);
				
		$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy);
		
		if (!$result->EOF) {
			$this->item->iterationId = $result->Fields("iteration_id");
			for ($i = 0; $i < $result->Fields("nb_tasks"); $i ++) {
				// Just put anything so that sizeof($this->item->includedItems) == $result->Fields("nbTasks")
				$this->item->includedItems[] = "fake";
			}
		}
	}
	
	/**
	* STATIC functions
	*/

	function & httpGetPostItem($val) {
		if ($val["id"] < 0) {
			$val["id"] = - $val["id"];
		}
		$projectId = getCurrentProjectId();
		if (@(list($nothing, $newProjectId)
			= explode("_", $val["iterationId"]))
			&& ($nothing == "proj")) {
			// We changed project
			
			$projectId = $newProjectId;
			
			// reset iteration
			$val["iterationId"] = 0;
		}
		
		$factory = & PlanningFactory::GetFactory();
		
		return $factory->newStory(
			array(
				"id"					=> $val["id"],
				"projectId"				=> $projectId,
				"loadFactor"			=> NULL,
				"title"					=> $val["title"],
				"description"			=> $val["description"],
				"workingDays"			=> NULL,
				"workingDaysName"		=> NULL,
				"iterationId"			=> $val["iterationId"],
				"validationCriteria"	=> $val["validationCriteria"],
				"weight"				=> $val["weight"],
				"risk"					=> $val["risk"],
				"priority"				=> $val["priority"],
				"iterationDeadlineTime" => NULL
			)
		);

		
	}

	

}

// *********************************************************************
// Class TaskAndStoryContainerControler
// *********************************************************************
class TaskAndStoryContainerControler extends TaskContainerControler {

	function dbGetStoriesSelect($iterationId) {

		$storySelect = & new QueryConstraints();

		$storySelect->fields = array(
			"id",
			"name",
		);

		$storySelect->tableNames = array(
			"stories"		
		);

		$storySelect->constraints = array(
			"iteration_id"	=> $iterationId,
			"project_id"	=> $this->item->projectId
		);

		$storySelect->groupBy = "";/*array(
			"stories.id"
		);*/

		$storySelect->orderBy = "";

		reset($this->item->storiesOrderBy);

		if (sizeof($this->item->storiesOrderBy) != 0) {
			$storySelect->orderBy = array();
		}
		
		foreach($this->item->storiesOrderBy as $field => $ascOrDesc) {
			$storySelect->orderBy[] = "$field $ascOrDesc";
		}
		
		$storySelect->customConstraints = "";

		return $storySelect;
	}



	function dbPopulateStories($iterationId, $deadlineTime) {
		
		
		$storySelect = $this->dbGetStoriesSelect($iterationId);

		$storySelect->fields = "*";

		$result = $GLOBALS["gDb"]->selectEntriesObj($storySelect);

		while (!$result->EOF) {
	 		// Call to internal function that returns the Story object
	 		
	 		$factory = & PlanningFactory::GetFactory();

			$storyToAdd = & $factory->newStory(
				array(
					"id"					=> $result->Fields("id"),
					"projectId"				=> $this->item->projectId,
					"loadFactor"			=> $this->item->loadFactor,
					"title"					=> $result->Fields("name"),
					"description"			=> $result->Fields("description"),
					"workingDays"			=> $this->item->workingDays,
					"workingDaysName"		=> $this->item->workingDaysName,
					"iterationId"			=> $result->Fields("iteration_id"),
					"validationCriteria"	=> $result->Fields("validation_criteria"),
					"weight"				=> $result->Fields("weight"),
					"risk"					=> $result->Fields("risk"),
					"priority"				=> $result->Fields("priority"),
					"iterationDeadlineTime" => $deadlineTime
				)
			);

			
			$storyToAdd->setOptions(
				$this->item->nTasksResponsibleId,
				$this->item->nTasksPairId);

			$storyToAdd->controler->dbPopulate();
			
			echoDebug("adding story $storyToAdd->id to item ".$this->item->type."[".$this->item->id."]");

			$this->item->addStory($storyToAdd);
			
			unSet($storyToAdd);
			$result->MoveNext();
		}

	}

	function dbPopulateBasicStories($iterationId) {
		
		
		$storySelect = $this->dbGetStoriesSelect($iterationId);
		
		$result = $GLOBALS["gDb"]->selectEntriesObj($storySelect);

		while (!$result->EOF) {
	 			 		
			$basicStory = & new BasicStory(
				array(
					"id"		=> $result->Fields("id"),
					"projectId"	=> $this->item->projectId,
					"title"		=> $result->Fields("name"),
					"iterationId"	=> $iterationId
				)
			);

			$this->item->basicStories[] = $basicStory;

			$result->MoveNext();
		}
	}
}
// *********************************************************************
// Class IterationControler
// *********************************************************************
class IterationControler extends TaskAndStoryContainerControler {

	/**
	* DB update functions
	*/
	
	function dbUpdateInDb() {
		
		if ($this->item->isAllowedToUpdate()) {
			
			// Update iteration entry
			
			$tableName = "iterations";
			$constraints =  array(
				"id" => $this->item->id
			);
			$fields = $this->dbGetFields();
			$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
			
			// Update working_days entry
	
			// UPDATE custom entry
			$tableNameDays = "working_days";
			
			$fieldsDays = $this->item->workingDays;
					
			$constraintsDays = array(
				"setup_name" => $this->item->workingDaysName
			);
			
			$GLOBALS["gDb"]->updateTable($tableNameDays, $fieldsDays, $constraintsDays);
			
			//if ($changedLFOrWD) {
				// Not optimized: we update the tasks even if we didn't
				// change the LF or WD...
				$this->dbPopulateTasks($this->item->id, XP_IMPOSSIBLE_ID, $this->item->deadlineTime);
				
				while(list($key, $task) = each($this->item->multiIncludedItems["task"])) {
					
					$task->controler->dbUpdateAutoEndDate();
				}
			//}
			
			$this->item->setItemModified();
		}
	}
	
	function dbAddToDb() {
		
		if ($this->item->isAllowedToInsert()) {
			
			// First: Insert iteration entry
			
			$tableName = "iterations";
			
			$this->dbInsertItem($tableName, $this->dbGetFields());
			
			// Second: Insert working_days entry
			
			$this->item->workingDaysName = "ite_".$this->item->id;
			
			$tableNameDays = "working_days";
			
			$fieldsDays = $this->item->workingDays;

			$fieldsDays["setup_name"] = $this->item->workingDaysName;
			
			$GLOBALS["gDb"]->insertIntoTable($tableNameDays, $fieldsDays);
			
			// Third: now update iteration with working_day foreign key
				
			$fields = array();
			$fields["working_days_name"] = $this->item->workingDaysName;;
			
			$constraints =  array(
				"id" => $this->item->id
			);
			
			$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
		}
	}
	
	function dbDeleteFromDb($post) {
		
		if ($this->item->isAllowedToDelete()) {
			
			$selectFields = array(
				"id"
			);
			$selectTableNames = array(
				"stories"
			);
			$selectConstraints = array(
				"iteration_id" => $this->item->id
			);
			
			$resultStories = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields);
			

			$selectTableNames = array(
				"tasks"
			);
			$selectConstraints = array(
				"iteration_id" => $this->item->id,
				"story_id" => 0
			);

			$resultTasks = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields);
			
			if ($post["whatStories"] == "delete") {
			// Delete all stories and tasks included in this iteration
				while (!$resultStories->EOF) {
					$valStories = array(
						"id" => $resultStories->Fields("id")
					);
					@($story = & StoryControler::httpGetPostItem($valStories));
					$newPost["whatTasks"] = "delete";
		 			$story->controler->dbDoDeleteFromDb($newPost);
		 			$resultStories->MoveNext();
				}
				while (!$resultTasks->EOF) {
					$valTasks = array(
						"id" => $resultTasks->Fields("id")
					);
					@($task = & TaskControler::httpGetPostItem($valTasks));
			 		$task->controler->dbDoDeleteFromDb();
						$resultTasks->MoveNext();
				}
				$id = 0;
			} else if ($post["whatStories"] == "move") {
			// Move all stories and tasks included in this iteration
				$newIteration = $post["iterationId"];
				while (!$resultStories->EOF) {
					$valStories = array(
						"id" => $resultStories->Fields("id"),
						"iterationId" => $newIteration
					);
					@($story = & StoryControler::httpGetPostItem($valStories));
					$story->controler->dbMove($story->projectId, $story->iterationId);
					$resultStories->MoveNext();
				}
				while (!$resultTasks->EOF) {
					$valTasks = array(
						"id" => $resultTasks->Fields("id"),
						"iterationStoryId" => $newIteration."_0"
					);
					@($task = & TaskControler::httpGetPostItem($valTasks));
					$task->controler->dbMove($task->projectId, $task->iterationId, 0);
					$resultTasks->MoveNext();
				}
				$id = $newIteration;
			}
			
			// Delete iteration entry
			$deleteTableName = "iterations";
			$deleteConstraints = array(
				"id" => $this->item->id
			);
			$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $deleteConstraints);
			
			// Delete working_days entry
			
			$deleteTableName = "working_days";
			$deleteConstraints = array(
				"setup_name" => $this->item->workingDaysName
			);
			$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $deleteConstraints);
			
			//setPostModified("");
		}
	}
	
	function dbProperties() {
		// db column name => class property name
		$fields = array(
			"name"							=> "title",
			"description"					=> "description",
			"date"							=> "deadlineDate",
			"project_id"					=> "projectId",
			"load_factor"					=> "loadFactor"
		);

		return $fields;
	}
	
	function dbPopulate() {
		$this->dbPopulateStories($this->item->id, $this->item->deadlineTime);
		$this->dbPopulateTasks($this->item->id, 0, $this->item->deadlineTime);
	}
	
	function dbPopulateForDelete() {
		
		// We need:
		// - nb of tasks and stories to know if a select has to be displayed...
		
		// nb of stories
		$selectFields = "count(*) AS nb_stories";

		$selectTableNames = array(
			"stories"
		);
		$selectConstraints = array(
			"iteration_id" => $this->item->id
		);
		$selectGroupBy = array(
			"iteration_id"
		);
				
		$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy);
				
		if (!$result->EOF) {
			for ($i = 0; $i < $result->Fields("nb_stories"); $i ++) {
				// Just put anything so that sizeof($this->item->multiIncludedItems["story"]) == $result->Fields("nb_stories")
				$this->item->multiIncludedItems["story"][] = "fake";
			}
		}
		
		// nb of tasks
		$selectFields = "count( * ) AS nb_tasks";

		$selectTableNames = array(
			"tasks"
		);
		$selectConstraints = array(
			"iteration_id" => $this->item->id
		);
		$selectGroupBy = array(
			"iteration_id"
		);
		
		$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy);
				
		if (!$result->EOF) {
			for ($i = 0; $i < $result->Fields("nb_tasks"); $i ++) {
				// Just put anything so that sizeof($this->item->multiIncludedItems["task"]) == $result->Fields("nbTasks")
				$this->item->multiIncludedItems["task"][] = "fake";
			}
		}
	}
	
	function dbCustomPopulateConstraintsTasks() {
		return TaskContainerControler::dbCustomPopulateConstraintsTasks();
	}

	/**
	* STATIC functions
	*/
	
	function & httpGetPostItem($val) {
		global $XP_DATE_FORMAT;
		
		if ($val["id"] < 0) {
			$val["id"] = - $val["id"];
		}
		
		$workingDaysName = "ite_".$val["id"];
	
		$newWorkingPace = explode("|",$val["wp"]);
			
		$mon = (int)in_array("Mon", $newWorkingPace);
		$tue = (int)in_array("Tue", $newWorkingPace);
		$wed = (int)in_array("Wed", $newWorkingPace);
		$thu = (int)in_array("Thu", $newWorkingPace);
		$fri = (int)in_array("Fri", $newWorkingPace);
		$sat = (int)in_array("Sat", $newWorkingPace);
		$sun = (int)in_array("Sun", $newWorkingPace);
		
		$workingDays = array(
			"Mon" => $mon,
			"Tue" => $tue,
			"Wed" => $wed,
			"Thu" => $thu,
			"Fri" => $fri,
			"Sat" => $sat,
			"Sun" => $sun
		);
		
		$factory = & PlanningFactory::GetFactory();
		
		return $factory->newIteration(
			array(
				"id"					=> $val["id"],
				"projectId"				=> getCurrentProjectId(),
				"loadFactor"			=> $val["loadFactor"],
				"title"					=> $val["title"],
				"description"			=> $val["description"],
				"workingDays"			=> $workingDays,
				"workingDaysName"		=> $workingDaysName,
				"deadline"				=> dateFromHtmlToSql($val["startDate"])
			)
		);
	}
}


// *********************************************************************
// Class Planning
// *********************************************************************
class PlanningControler extends TaskAndStoryContainerControler {

	function setAllGetOptions() {

		$values = array();

		$values[XP_PLANNING_POP_RESPONSIBLE]		= $_GET[XP_PLANNING_POP_RESPONSIBLE];
		$values[XP_PLANNING_POP_PAIR]				= $_GET[XP_PLANNING_POP_PAIR];

		// We want that get variables are set to "0" if not set (checkboxes)
		$values[XP_PLANNING_POP_ITERATION_CURRENT]	= isSet($_GET[XP_PLANNING_POP_ITERATION_CURRENT])?$_GET[XP_PLANNING_POP_ITERATION_CURRENT]:0;
		$values[XP_PLANNING_POP_ITERATION_OTHER]	= isSet($_GET[XP_PLANNING_POP_ITERATION_OTHER])?$_GET[XP_PLANNING_POP_ITERATION_OTHER]:0;
	 	$values[XP_PLANNING_POP_STORY]				= isSet($_GET[XP_PLANNING_POP_STORY])?$_GET[XP_PLANNING_POP_STORY]:0;
	 	$values[XP_PLANNING_POP_TASK]				= isSet($_GET[XP_PLANNING_POP_TASK])?$_GET[XP_PLANNING_POP_TASK]:0;
		$values[XP_PLANNING_POP_STORIES_ORDERBY]	= isSet($_GET[XP_PLANNING_POP_STORIES_ORDERBY])?$_GET[XP_PLANNING_POP_STORIES_ORDERBY]:"";

		$this->item->setOptions($values);
	}

	function dbPopulateForHeader() {
		
		$this->dbPopulateBasicMembers();
		$this->dbPopulateBasicIterations();
		$this->dbPopulateBasicStories(0);
	}

	function dbPopulateBasicMembers() {
		
		$GLOBALS["gBasicMembers"] = array();

		$selectFields = "*";

		$selectTableNames = array(
			"team_members",
			"projects_affectation"
		);
		$selectConstraints = array(
			"project_id"=> $this->item->projectId,
			"team_member_id" => "team_members.id"
		);
		$selectGroupBy = "";/*array(
			"team_members.id",
			"team_members.profile_id"
		);*/
		
		$selectOrderBy = array(
			"name asc"
		);
		
		$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
		
		while (!$result->EOF) {

			$basicMember = & new BasicPlanningMember(
				array(
					"id"					=> $result->Fields("id"),
					"projectId"				=> $this->item->projectId,
					"title"					=> $result->Fields("name")
				)
			);

			$GLOBALS["gBasicMembers"][$basicMember->id] = $basicMember;

			$result->MoveNext();
		}
	}
	
	function dbPopulate() {

		$this->dbPopulateIterations();

		if ($this->item->populate[XP_PLANNING_POP_STORY]) {
			$this->dbPopulateStories(0, null);
		}
		
		echoDebug("pop1:".$this->item->toString()."<br/>pop2:".$this->item->controler->item->toString());

		if ($this->item->populate[XP_PLANNING_POP_TASK]) {
			$this->dbPopulateTasks(0, 0, null);
		}
		
		echoDebug("pop1:".$this->item->toString()."<br/>pop2:".$this->item->controler->item->toString());
	}

	function getCurrentIterationId() {
		

		$today = date("Y-m-d");
		$selectFields = array(
					"id"
		);
		$selectTableNames = array(
					"iterations"
		);
		$selectConstraints = array(
					"project_id" => $this->item->projectId,
					"date" => array(">=", $GLOBALS["gDb"]->quote($today))
		);
		$selectOrderBy = array(
			"date ASC"
		);

		$result= & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields, "", $selectOrderBy);

		return $result->Fields("id");
	}

	function dbGetIterationsSelect() {

		$iterationSelect = & new QueryConstraints();

		$iterationSelect->fields = array(
			"id",
			"name",
			"Mon",
			"Tue",
			"Wed",
			"Thu",
			"Fri",
			"Sat",
			"Sun"
		);

		$iterationSelect->tableNames = array(
			"iterations",
			"working_days"
		);

		$iterationSelect->constraints = array(
			"project_id"		=> $this->item->projectId,
			"working_days_name"	=> "working_days.setup_name"
		);

		$iterationSelect->groupBy = "";/*array(
			"iterations.id"
		);*/

		$iterationSelect->orderBy = array(
			"date desc"
		);
		
		$iterationSelect->customConstraints = "";

		return $iterationSelect;
	}

	function dbPopulateIterations($bDeep = true) {
		
		
		$iterationSelect = $this->dbGetIterationsSelect();

		$this->item->currentIterationId = $this->getCurrentIterationId();

		$iterationSelect->fields = "*";

        		
		if ($this->item->populate[XP_PLANNING_POP_ITERATION_CURRENT] && $this->item->populate[XP_PLANNING_POP_ITERATION_OTHER]) {
			// Both: no constraint
		} else {
	 		if ($this->item->populate[XP_PLANNING_POP_ITERATION_CURRENT]) {

	 			if ($this->item->currentIterationId) {
					$iterationSelect->customConstraints = "iterations.id = " . $GLOBALS["gDb"]->quote($this->item->currentIterationId);
				} else {
					return;
				}

			} else {
				$iterationSelect->customConstraints = "iterations.id <> " . $GLOBALS["gDb"]->quote($this->item->currentIterationId);
			}
		}
		
		$result = $GLOBALS["gDb"]->selectEntriesObj($iterationSelect);
		
		while (!$result->EOF) {
	 		
			// Call to internal function that returns the Iteration object
	 		$factory = & PlanningFactory::GetFactory();
	 		
			$iterationToAdd = & $factory->newIteration(
				array(
					"id"					=> $result->Fields("id"),
					"projectId"				=> $this->item->projectId,
					"loadFactor"			=> $result->Fields("load_factor"),
					"title"					=> $result->Fields("name"),
					"description"			=> $result->Fields("description"),
					"workingDays"			=> array (
											   	"Mon" => $result->Fields("Mon"),
											   	"Tue" => $result->Fields("Tue"),
											   	"Wed" => $result->Fields("Wed"),
											   	"Thu" => $result->Fields("Thu"),
											   	"Fri" => $result->Fields("Fri"),
											   	"Sat" => $result->Fields("Sat"),
											   	"Sun" => $result->Fields("Sun")
											   ),
					"workingDaysName"		=> $result->Fields("working_days_name"),
					"deadline"				=> $result->Fields("date")
				)
			);
			
			
			// Options for member's tasks
			$iterationToAdd->setOptions(
				$this->item->nTasksResponsibleId,
				$this->item->nTasksPairId,
				$this->item->storiesOrderBy);

			if ($bDeep) {

				$iterationToAdd->controler->dbPopulate();

			}
			
			if ($this->item->currentIterationId == $iterationToAdd->id) {
				$iterationToAdd->isCurrent = true;
			}
			
			$this->item->addIteration($iterationToAdd);
			unSet($iterationToAdd);
			$result->MoveNext();
		}
	}

	function dbPopulateBasicIterations($bDeep = true) {
		
			
		$iterationSelect = $this->dbGetIterationsSelect();
	
		$currentIterationId = $this->getCurrentIterationId();
		
		$result = $GLOBALS["gDb"]->selectEntriesObj($iterationSelect);
		
		while (!$result->EOF) {

			$factory = & PlanningFactory::GetFactory();
			
			$basicIteration = & $factory->newBasicIteration(
				array(
					"id"					=> $result->Fields("id"),
					"projectId"				=> $this->item->projectId,
					"title"					=> $result->Fields("name"),
					"workingDays"			=> array (
						"Mon" => $result->Fields("Mon"),
						"Tue" => $result->Fields("Tue"),
						"Wed" => $result->Fields("Wed"),
						"Thu" => $result->Fields("Thu"),
						"Fri" => $result->Fields("Fri"),
						"Sat" => $result->Fields("Sat"),
						"Sun" => $result->Fields("Sun")
					),
				)
			);

			if ($basicIteration->id == $currentIterationId) {
				$basicIteration->isCurrent = true;
			}
			
			if ($bDeep) {

				$basicIteration->controler->dbPopulateBasicStories($basicIteration->id);
				
			}
			
			$this->item->basicIterations[] = $basicIteration;

			$result->MoveNext();
		}
	}

	function dbPopulateAllTasksFromProject($bWithCompletion = false) {

		echoDebug("allTasks : start size : ".sizeof($this->item->multiIncludedItems["task"]));

		$this->dbPopulate();

		if ($bWithCompletion) {

			$this->item->calculateCompletion();

		}
		
		// All isolated tasks allready in $this->item->multiIncludedItems["task"]

		echoDebug("allTasks : size after populate : ".sizeof($this->item->multiIncludedItems["task"]));

		// For all isolated stories
		reset($this->item->multiIncludedItems["story"]);
		while(list($key, $story) = each($this->item->multiIncludedItems["story"])) {

			echoDebug("allTasks : story ".$story->id." / ".sizeof($story->multiIncludedItems["task"])." tasks");

			// Add all tasks from story
			$this->item->multiIncludedItems["task"] = array_merge(
				$this->item->multiIncludedItems["task"],
				$story->multiIncludedItems["task"]);
		}

		echoDebug("allTasks : size after stories : ".sizeof($this->item->multiIncludedItems["task"]));
		
		// For all iterations
		reset($this->item->multiIncludedItems["iteration"]);
		while(list($key, $iteration) = each($this->item->multiIncludedItems["iteration"])) {


			// Add all isolated tasks from iterations
			$this->item->multiIncludedItems["task"] = array_merge(
				$this->item->multiIncludedItems["task"],
				$iteration->multiIncludedItems["task"]);

			// For all stories in iteration
			reset($iteration->multiIncludedItems["story"]);
			while(list($key, $story) = each($iteration->multiIncludedItems["story"])) {
	 			// Add all tasks from story
	 			$this->item->multiIncludedItems["task"] = array_merge(
					$this->item->multiIncludedItems["task"],
					$story->multiIncludedItems["task"]);
	 		}
		}

	}
	
	// _________ Private functions
	
	// STATIC
	function cookiesSaveGETOptions() {
		// Affect cookies
		// Only the ones that have been changed...

		XPItemControler::setCookieIfGet(XP_PLANNING_POP_RESPONSIBLE);
		
		XPItemControler::setCookieIfGet(XP_PLANNING_POP_PAIR);
		
		XPItemControler::setCookieIfGet(XP_PLANNING_POP_ITERATION_CURRENT);
		
		XPItemControler::setCookieIfGet(XP_PLANNING_POP_ITERATION_OTHER);
		
		XPItemControler::setCookieIfGet(XP_PLANNING_POP_STORY);
		
		XPItemControler::setCookieIfGet(XP_PLANNING_POP_TASK);

		XPItemControler::setCookieIfGet(XP_PLANNING_POP_STORIES_ORDERBY);
		
	}

}

	
// *********************************************************************
// Others
// *********************************************************************
function processPost() {

	if ($_POST["id"] < 0) {
	 	// Deletion
		  if ($_POST["switch"] == "task") {
				@($task = TaskControler::httpGetPostItem($_POST));
				$task->controler->dbDeleteFromDb();
		  } else if ($_POST["switch"] == "story") {
				@($story = StoryControler::httpGetPostItem($_POST));
				// Notice the param to dbDeleteFromDb() function
				$story->controler->dbDeleteFromDb($_POST);
		  } else if ($_POST["switch"] == "iteration") {
				@($iteration = IterationControler::httpGetPostItem($_POST));
				// Notice the param to dbDeleteFromDb() function
				$iteration->controler->dbDeleteFromDb($_POST);
		  }
	 } else if ($_POST["id"] != 0) {
	 	// Modify
		  if ($_POST["switch"] == "task") {
		  	@($task = & TaskControler::httpGetPostItem($_POST));
			$task->controler->dbUpdateInDb();
		  } else if ($_POST["switch"] == "story") {
				@($story = & StoryControler::httpGetPostItem($_POST));
				$story->controler->dbUpdateInDb();
		  } else if ($_POST["switch"] == "iteration") {
				@($iteration = & IterationControler::httpGetPostItem($_POST));
				$iteration->controler->dbUpdateInDb();
		  }
	 } else {
	 	// Insertion
		  if ($_POST["switch"] == "task") {
		  	@($task = & TaskControler::httpGetPostItem($_POST));

			//echo "TEST ".$_POST["endDate"]."<br/>";

			$task->controler->dbAddToDb();
		  } else if ($_POST["switch"] == "story") {
				@($story = & StoryControler::httpGetPostItem($_POST));
				$story->controler->dbAddToDb();
		  } else if ($_POST["switch"] == "iteration") {
				@($iteration = & IterationControler::httpGetPostItem($_POST));
				$iteration->controler->dbAddToDb();
		  }
	 }
	 unSet($_POST);
}
