<?php


// *********************************************************************
// Class ProjectListControler
// *********************************************************************
class ProjectListControler extends XPItemListControler {
	
	function & buildInit($type) {
		return new ProjectControler();
	}

	function dbGetAllProjectLinkTypes() {
		
		$selectFields = "*";
		
		$selectTableNames = array(
			"project_link_types"
		);
		
		$selectConstraints = array(
		);
		
		$selectGroupBy = "";/*array(
			"id"
		);*/
		
		$selectOrderBy = array(
			"name asc"
		);
		
		$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
		
		while (!$result->EOF) {
			$this->item->availableLinkTypes[$result->Fields("id")] = $result->Fields("name");
			$result->MoveNext();
		}
	}

	/**
	 * STATIC methods
	 */
	function dbGetListOfAllProjects() {
		
				
		$selectFields = array(
			"id",
			"name"
		);
		
		$selectTableNames = array(
			$this->dbIncludedTableName()
		);
		
		$selectConstraints = array(
		);
		
		$selectGroupBy = "";/*array(
			"id"
		);*/
		
		$selectOrderBy = array(
			"name asc"
		);
		
		$result = $GLOBALS["gDb"]->selectEntries(
			$selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
		
		$allProjects = array();
			
		while (!$result->EOF) {
			
			$allProjects[$result->Fields("id")] = $result->Fields("name");

			$result->MoveNext();
		}
		
		
		return $allProjects;
	}
	
	function dbPopulate() {
		
		
		if (empty($this->item->availableLinkTypes)) {
			$this->dbGetAllProjectLinkTypes();
		}
		
		$selectFields = "*";
		
		$selectTableNames = array(
			$this->dbIncludedTableName()
		);
		
		$selectConstraints = array(
		);
		
		$selectGroupBy = "";/*array(
			"id"
		);*/
		
		$selectOrderBy = array(
			"name asc"
		);
		
		$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
		
		while (!$result->EOF) {
			
			$fields = $this->dbIncludedFields($result);
					
			$fields["availableLinkTypes"] = $this->item->availableLinkTypes;
			
			$projectToAdd = & $GLOBALS["gFactory"]->newProject($fields);
			
			// Projects that you cannot see aren't even populated (so that they don't
			// appear in lists)
			if (!XP_CANT_SELECT_CANT_SEE_PROJECT
				|| $projectToAdd->isAllowedToSelect()) {
									
				$projectToAdd->controler->dbPopulate();
				
				$this->item->addIncluded($projectToAdd);
				
			}
				
			unSet($projectToAdd);
			
			$result->MoveNext();
		}
	}
}
// *********************************************************************
// Class ProjectControler
// *********************************************************************
class ProjectControler extends XPItemWithIncludedControler {

	// /////////////////////////////////////////////////////////////
	// DB METHODS
	// /////////////////////////////////////////////////////////////
	
	function & buildInit($type) {
		return new ProjectLinkControler();
	}
	
	/// Name of parent id column in included item
	function dbIncludedParentId($type = "") {
		return "project_id";
	}
	
	/// DB table name
	function dbGetTableName() {
		return "projects";
	}

	/// Correspondance <db column name> <=> <class property name>
	function dbProperties() {

		// db column name => class property name
		$fields = array(
			"id"			=> "id",
			"name"			=> "title",
			"description"	=> "description"
		);

		return $fields;
	}

	function dbPopulate() {
		
		
		$selectFields = "*";
		
		$selectTableNames = array(
			$this->dbIncludedTableName()
		);
		$selectConstraints = array(
			$this->dbIncludedParentId() => $this->item->id
		);
		$selectGroupBy = "";/*array(
			"id"
		);*/
		$selectOrderBy = array(
			"comment asc"
		);
		
		reset($this->item->availableLinkTypes);
		
		while(list($typeId, $typeName) = each($this->item->availableLinkTypes)) {
			// For each available type
			$selectConstraints["type"] = $typeId;
			
			$result = $GLOBALS["gDb"]->selectEntries(
				$selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
			
			while (!$result->EOF) {
								
				$fields = $this->dbIncludedFields($result);

				$fields["linkType"]	= $typeId;
				$fields["linkTypeName"]	= $typeName;

				$projectLinkToAdd = & $GLOBALS["gFactory"]->newProjectLink($fields);

				$this->item->addProjectLink($projectLinkToAdd);
				unSet($projectLinkToAdd);
				$result->MoveNext();
			}
		}
	}

	/**
	* DB update functions
	*/
	
	function dbUpdateInDb() {
		
		if ($this->item->isAllowedToUpdate()) {
			
			$tableName = $this->dbGetTableName();
			$constraints =  array(
				"id" => $this->item->id
			);
			$fields = $this->dbGetFields();
			$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
			
			$this->item->setItemModified();
			
			if ($this->item->isCurrent) {
				// This is the current project and the name might have changed
				$this->httpSelectAsMyCurrentProject();
			}
		}		
	}
	
	function dbAddToDb() {
		
		

		if ($this->item->isAllowedToInsert()) {

			$tableName = $this->dbGetTableName();
						
			$this->dbInsertItem($tableName, $this->dbGetFields());

			// By default, choose this new project as the current one...

			$this->httpSelectAsMyCurrentProject();

			/**
			* Automatically add user to the project
			*/
			if (!$GLOBALS["gConnectedUser"]->isAdmin && !$GLOBALS["gConnectedUser"]->isAnonymous) {
								
				require "Model/Team_model.inc.php";
				require "Controler/Team_controler.inc.php";

				$factory = & new TeamFactory();

				$tempMember = & $factory->newMember(
					array(
						"id"			=> $GLOBALS["gConnectedUser"]->id,
						"profileId"		=> 0,
						"title"			=> "",
						"description"	=> "")); // Only id counts...
				
				$tempMember->controler->dbAffectToProject($this->item->id);
			}
		} else {
			//echoDebug("not allowed");
		}
	}
	
	function dbDeleteFromDb($args) {
		

		if ($this->item->isAllowedToDelete()) {
			
			echoDebug("deleteProject");
			
			$constraints = array(
				$this->dbIncludedParentId() => $this->item->id
			);
			
			if ($args["whatEverything"] == "delete") {
				
				echoDebug("also delete content");
				
				// Tasks
				$deleteTableName = "tasks";
				$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $constraints);
				
				// Stories
				$deleteTableName = "stories";
				$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $constraints);
				
				// Iterations
				$deleteTableName = "iterations";
				$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $constraints);
				
				// Members
				$deleteTableName = "projects_affectation";
				$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $constraints);
				
				// Metaphor
				$deleteTableName = "metaphor_words";
				$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $constraints);
				
				// Project links
				$deleteTableName = "project_links";
				$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $constraints);
				
				
				// Web categories
				// @attention web links must also be deleted!
				$deleteTableName = "web_categories";
				
				// Select the category ids that must be deleted
				$categoriesToDelete = $GLOBALS["gDb"]->selectEntries(array($deleteTableName), $constraints, array("id"));
				
				while (!$categoriesToDelete->EOF) {
					
					// delete the corresponding web links
					$deleteTableName2 = "web_links";
				
					$constraints2 = array(
						"category_id" => $categoriesToDelete->Fields("id")
					);
					
					$GLOBALS["gDb"]->deleteFromTable($deleteTableName2, $constraints2);
					
					$categoriesToDelete->MoveNext();
				}
				
				// Now delete the categories
				$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $constraints);
				
				
			} else if ($args["whatEverything"] == "move") {
				
				echoDebug("move content to project ".$args["moveToProject"]);
				
				$newProjectId = $args["moveToProject"];
				
				$fields = array(
					$this->dbIncludedParentId() => $newProjectId
				);
				
				// Tasks
				$tableName = "tasks";
				$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);

				// Stories
				$tableName = "stories";
				$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);

				// Iterations
				$tableName = "iterations";
				$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);				

				// Members
				$tableName = "projects_affectation";
				$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
				
				// Metaphor
				$tableName = "metaphor_words";
				$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
				
				// Project links
				$tableName = "project_links";
				$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
				
				// Web categories and web links: moving categories will
				// move the links too.
				$tableName = "web_categories";
				$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
			}
			
			// Now time to delete the project
			$constraints = array(
				"id" => $this->item->id
			);
			
			$deleteTableName = $this->dbGetTableName();
			$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $constraints);
			
			if ($this->item->isCurrent) {
				// This was the current project
				$factory = & ProjectsFactory::GetFactory();

				$newCurrentProject = & $factory->newProject(
					array(
						"id"			=> 0,
						"title"			=> "",
						"description"		=> "",
						"availableLinkTypes"	=> array()
					)
				);
				$newCurrentProject->controler->httpSelectAsMyCurrentProject();
			}
		}
	}
	

	// /////////////////////////////////////////////////////////////
	// HTTP METHODS
	// /////////////////////////////////////////////////////////////

	function httpProperties() {

		// http post field => class property name
		$fields = array(
			"id"			=> "id",
			"name"			=> "title",
			"description"	=> "description"
		);

		return $fields;
	}
	
	function httpSelectAsMyCurrentProject() {
		global $gCurrentProjectId, $gCurrentProjectName;

		if ($this->item->isAllowedToSelect()) {

			setXPCookie(XP_PROJECT_NAME, $this->item->title);
			setXPCookie(XP_PROJECT_ID, $this->item->id);

			$gCurrentProjectId = $this->item->id;
			$gCurrentProjectName = $this->item->title;
		}
	}
}
// *********************************************************************
// Class ProjectLinkControler
// *********************************************************************
class ProjectLinkControler extends XPItemControler {

	// /////////////////////////////////////////////////////////////
	// DB METHODS
	// /////////////////////////////////////////////////////////////

	/// DB table name
	function dbGetTableName() {
		return "project_links";
	}

	/// Correspondance <db column name> <=> <class property name>
	function dbProperties() {

		// db column name => class property name
		$fields = array(
			"id"			=> "id",
		
			"url"			=> "url",

			"comment"		=> "title",
			"project_id"	=> "projectId",
			"type"			=> "linkType"
		);

		return $fields;
	}

	function getParentTypeAndProperty() {

		return array("project", "projectId");

	}

	function getPropertyForDb($propertyName) {

		if ($propertyName == "url") {

			return addslashes(parent::getPropertyForDb($propertyName));

		} else {

			return parent::getPropertyForDb($propertyName);

		}
	}

	// /////////////////////////////////////////////////////////////
	// HTTP METHODS
	// /////////////////////////////////////////////////////////////

	function httpProperties() {

		// http post field => class property name
		$fields = array(
			"id"			=> "id",
			"comment"		=> "title",
			"projectId"		=> "projectId",
			"type"			=> "linkType",
			""				=> "",
			"url"			=> "url"
		);

		return $fields;
	}

	function getHttpProperty($httpPostName) {
		
		if ($httpPostName == "url") {

			return ereg_replace("/$", "", parent::getHttpProperty($httpPostName));

		} else {

			return parent::getHttpProperty($httpPostName);

		}
	}

}
?>
