<?php

// *********************************************************************
// Class TeamControler
// *********************************************************************
class TeamControler extends XPItemMultiListControler {
	
	function dbPopulate() {
		
		if ($this->item->projectId != 0) {
			
			$this->dbPopulateAffected();
			
		} else {
			
			$this->item->notToInclude = XP_IMPOSSIBLE_ID .",0"; //Adds in the Nobody Id
			
		}
		
		$this->dbPopulateNotAffected();
	}
	
	function dbPopulateAffected() {
		
		
		$selectFields = "*";
		
		$selectTableNames = array(
			"team_members",
			"projects_affectation"
		);
		$selectConstraints = array(
			"project_id" => $this->item->projectId,
			"team_member_id" => "team_members.id",
			" team_member_id" => array("!=","0") //Filters out the special Nobdy user needed by PostgreSQL
		);
		$selectGroupBy = "";/*array(
			"team_members.id",
			"team_members.profile_id"
		);*/
		$selectOrderBy = array(
			"name asc"
		);
		
		$result = $GLOBALS["gDb"]->selectEntries(
			$selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
			
		while (!$result->EOF) {

			$factory = & TeamFactory::GetFactory();
		
			$memberToAdd = & $factory->newMember(
				array(
					"id"			=> $result->Fields("team_member_id"),
					"profileId"		=> $result->Fields("profile_id"),
					"title"			=> $result->Fields("name"),
					"description"	=> $result->Fields("description")
				)
			);
			
			$memberToAdd->controler->dbGetAllAffectations();
			
			$this->item->addMember($memberToAdd, true);
			
			unSet($memberToAdd);
			
			// For exclusion later in not affected
			$this->item->notToInclude .= $result->Fields("team_member_id").", ";
			$result->MoveNext();
		}
		
		$this->item->notToInclude .= XP_IMPOSSIBLE_ID .",0";
		
		// $this->item->notToInclude now contains the list of affected ids.
	}
	
	function dbPopulateNotAffected() {
		
		
		
		$query = "SELECT * FROM team_members WHERE (id NOT IN (".$this->item->notToInclude.")) ORDER BY name";
		
		$result = $GLOBALS["gDb"]->executeQuery($query);
		while (!$result->EOF) {
			
			$memberToAdd = $GLOBALS["gTeamFactory"]->newMember(
				array(
					"id"			=> $result->Fields("id"),
					"profileId"		=> $result->Fields("profile_id"),
					"title"			=> $result->Fields("name"),
					"description"	=> $result->Fields("description")
				)
			);
			
			$memberToAdd->controler->dbGetAllAffectations();
			
			$this->item->addMember($memberToAdd, false);
			unSet($memberToAdd);
			$result->MoveNext();
		}

	}
}
// *********************************************************************
// Class MemberControler
// *********************************************************************
class MemberControler extends XPItemControler {
	
	function dbGetAllAffectations() {
		
		$this->item->affectations = array();
		if ($this->item->id != 0) {
			$selectFields = array(
				"project_id",
				"name"
			);
			$selectTableNames = array(
				"projects_affectation",
				"projects"
			);
			$selectConstraints = array(
				"team_member_id" => $this->item->id,
				"project_id" => "projects.id"
			);
			$selectGroupBy = "";/*array(
				"project_id"
			);*/
			
			$selectOrderBy = array(
				"name asc"
			);
			
			$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
			while (!$result->EOF) {
				$this->item->affectations[$result->Fields("project_id")] = $result->Fields("name");
				$result->MoveNext();
			}
		}
	}


	/**
	* DB update functions
	*/
	
	function dbDeleteFromDb() {
		$this->dbGetAllAffectations();
		if ($this->item->isAllowedToDelete()) {
			$this->dbDoTheDeleteFromDB();	
			lib_login_delete_user($this->item->id);
		}
	}
	
	function dbDoTheDeleteFromDB() {
		
		// Remove responsibility and pair from all tasks
		$this->dbRemoveParticipationInTasks();
		
		// Remove all affectations from projects
		$query = "DELETE FROM projects_affectation WHERE team_member_id = ".$this->item->id;
		$GLOBALS["gDb"]->executeQuery($query);
		
		// Delete member
		$query = "DELETE FROM team_members WHERE id = ".$this->item->id;
		$GLOBALS["gDb"]->executeQuery($query);
	}
	
	function dbUpdateInDb() {
		
		$this->dbGetAllAffectations();
		if ($this->item->isAllowedToUpdate()) {
			$tableName = "team_members";
			$constraints =  array(
				"id" => $this->item->id
			);
			$fields = $this->dbGetFields();
			$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);

			$this->item->setItemModified();
		}
	}
	
	function dbInsertMemberInProject($password, $passwordAgain, $question, $answer, $memberProjectId = 0) {
		
		if (empty($memberProjectId)) {
			$memberProjectId = 0;
		}
		if ($this->item->isAllowedToInsert($memberProjectId)) {
			// Insertion of new member
			$tableName = "team_members";
			
			$this->dbInsertItem($tableName, $this->dbGetFields());
			
			$error = lib_login_create_account($this->item->id, $password, $passwordAgain, "test@test.com", $question, $answer, false);
			
			if ($error != "success") {
				// There was an error : the transaction should be "rolled back"
				// It's time to delete the user we just added...
				$this->dbDoTheDeleteFromDB();
				
				$this->item->id = "insert";

				$error = getLang("error_create_member")."\\n".$error;
			} else {
				$error = "";
				// Affect member to project
				if ($memberProjectId != 0) {
					$this->dbAffectToProject($memberProjectId);
				}
			}
			
			$this->item->setItemModified($error);
		}
	}
	
	function dbGetFields() {
		$fields = array(
			"profile_id"	=> $this->item->profileId,
			"name"			=> $this->item->title,
			"description"	=> $this->item->description
		);
		return $fields;
	}
	
	function dbAffectToProject($projectId) {
		$this->dbGetAllAffectations();
		
		if ($this->item->isAllowedToOrDeAffectFromToProject($projectId)) {
			$this->dbDoAffectToProject($projectId);

			$this->item->setItemModified();
		}
	}
	
	function dbDoAffectToProject($projectId) {
		
		$tableName = "projects_affectation";
		$fields = array(
			"team_member_id" => $this->item->id,
			"project_id" => $projectId
		);
		$GLOBALS["gDb"]->insertIntoTable($tableName, $fields);
	}
	
	function dbDeAffectFromProject($projectId) {
		
		$this->dbGetAllAffectations();
		if ($this->item->isAllowedToOrDeAffectFromToProject($projectId)) {
			// De-affect from project
			$query = "DELETE FROM projects_affectation WHERE team_member_id = ".$this->item->id." AND project_id = ".$projectId;
			$GLOBALS["gDb"]->executeQuery($query);
			
			// Remove responsibility from tasks
			$this->dbRemoveParticipationInTasks($projectId);

			$this->item->setItemModified();
		}
	}
	
	function dbRemoveParticipationInTasks($projectId = 0) {
		

		$tableName = "tasks";

		// Remove responsability

		$constraints =  array(
			"responsible_team_member_id" => $this->item->id
		);
		if ($projectId != 0) {
			$constraints["project_id"] = $projectId;
		}

		$fields = array(
			"responsible_team_member_id" => "0"
		);
		
		$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);

		// Remove pairing

		$constraints =  array(
			"pair_team_member_id" => $this->item->id
		);
		if ($projectId != 0) {
			$constraints["project_id"] = $projectId;
		}
		$fields = array(
			"pair_team_member_id" => "0"
		);

		$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
	}
	
	function dbUpdatePassword($newPassword, $newPasswordAgain) {
		

		if ($this->item->isAllowedToChangePassword()) {

			// param false => no automatic reload (just return a result...)
			$error = urldecode(
				lib_login_update_password_any_user($this->item->id, $newPassword, $newPasswordAgain, false));

			if ($error == "success") {
				$error = "";
			} else {
				$error = getLang("error_update_member")."\\n".$error;
			}
			
			$this->item->setItemModified($error);

		}
	}
	
	/**
	* STATIC functions
	*/
	
	function & httpGetPostItem($val) {
		if ($val["id"] < 0) {
			$val["id"] = - $val["id"];
		}
		$factory = & TeamFactory::GetFactory();
		return $factory->newMember(
			array(
				"id"			=> $val["id"],
				"profileId"		=> $val["profileId"],
				"title"			=> $val["name"],
				"description"	=> $val["description"]
			)
		);
	}
}
// *********************************************************************
// Class ProfilesListControler
// *********************************************************************
class ProfilesListControler extends XPItemControler {

	function dbPopulateAllProfiles() {
		
		global $gExistingProfiles;
		global $gDefaultProfileId;
		global $gDevelopperProfileId;

		if (empty($gExistingProfiles)) {
			$gExistingProfiles = array();
			
			/**
			* Get all profiles names
			*/
			$selectFields = "*";
			
			$selectTableNames = array(
				"user_profiles"
			);
			$selectConstraints = array(
			);
			$selectGroupBy = "";/*array(
				"id"
			);*/
			
			$selectOrderBy = array(
				"id"
			);
			
			$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
			while (!$result->EOF) {

				$id = $result->Fields("id");
				$name = $result->Fields("name");

				$gExistingProfiles[$id] = $name;

				if ($name == "Anonymous") {

					$gDefaultProfileId = $id;

				} elseif($name == "Developper") {

					$gDevelopperProfileId = $id;

				}
				$result->MoveNext();
			}
		}
	}

	function dbPrepare() {
		
		global $gProtectedSections;
		global $gRightsNames;		
		
		$this->dbPopulateAllProfiles();
		
		$gRightsNames = array();
		$gProtectedSections = array();

		/**
		* Get all protected sections names
		*/
		$selectFields = "*";
		
		$selectTableNames = array(
			"protected_sections"
		);
		$selectConstraints = array(
		);
		$selectGroupBy = "";/*array(
			"id"
		);*/
		
		$selectOrderBy = array(
			"name1 desc",
			"name2"
		);
		
		$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
		while (!$result->EOF) {
			$sectionId = $result->Fields("id");
			$name1 = $result->Fields("name1");
			$name2 = $result->Fields("name2");
			
			/**
			* Get all protected section available rights
			*/
			$selectFields2 = "*";
			
			$selectTableNames2 = array(
				"available_section_rights"
			);
			$selectConstraints2 = array(
				"protected_section_id" => $sectionId
			);
			
			$result2 = $GLOBALS["gDb"]->selectEntries($selectTableNames2, $selectConstraints2, $selectFields2);
			$availableRights = array();
			while (!$result2->EOF) {
				$availableRights[] = $result2->Fields("right_type_id");
				$result2->MoveNext();
			}
			
			$gProtectedSections[$sectionId] =
				array(
					$name1,
					$name2,
					$availableRights
				);
			unset($availableRights);
			$result->MoveNext();
		}
		
		/**
		* Get rights names
		*/
		$selectFields = "*";
		
		$selectTableNames = array(
			"right_types "
		);
		$selectConstraints = array(
		);
		$selectGroupBy = "";/*array(
			"id"
		);
		*/
		$selectOrderBy = array(
			"type_name"
		);
		
		$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
		while (!$result->EOF) {
			$gRightsNames[$result->Fields("id")] = $result->Fields("type_name");
			$result->MoveNext();
		}
	}
	
	function dbPopulate() {
		
		$selectFields = "*";
		
		$selectTableNames = array(
			"user_profiles"
		);
		$selectConstraints = array(
		);
		$selectGroupBy = "";/*array(
			"id"
		);*/
		
		$selectOrderBy = array(
			"id"
		);
		
		$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
		while (!$result->EOF) {
			$id = $result->Fields("id");

			$factory = & TeamFactory::GetFactory();
		
			$profileToAdd = & $factory->newProfile(
				array(
					"id"			=> $id,
					"title"			=> $result->Fields("name"),
					"description"	=> $result->Fields("description")
				)
			);
			
			$profileToAdd->controler->dbPopulateRights();
			
			echoDebug("profileToAdd size: ".sizeof($profileToAdd->multiIncludedItems));
			
			$this->item->addIncluded($profileToAdd);
			
			unSet($profileToAdd);
			$result->MoveNext();
		}
	}

}
// *********************************************************************
// Class ProfileControler
// *********************************************************************
class ProfileControler extends XPItemControler {

	function dbPopulateRights() {
		
		
		if ($this->item->id != 0) {
			$selectFields = array(
				"protected_section_id",
				"right_type_id",
				"name1"
			);
			$selectTableNames = array(
				"profiles_rights",
				"protected_sections"
			);
			
			// First: populate SECTIONS!!
			$selectConstraints = array(
				"name1" => "section",
				"profile_id" => $this->item->id,
				"protected_section_id" => "protected_sections.id"
			);
			
			$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields);
			
			$this->dbNewIncludedRights($result);
			
			echoDebug("my size: ".sizeof($this->item->multiIncludedItems));
			
			// Second: populate other rights...
			unset($selectConstraints["name1"]);
			
			$customConstraints = "name1 <>  " . $GLOBALS["gDb"]->quote("section");
			
			$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields, "", "", $customConstraints);
	
			$this->dbNewIncludedRights($result);
			
			echoDebug("my size2: ".sizeof($this->item->multiIncludedItems));
		}
	}
	
	function dbNewIncludedRights(& $result) {
		
		$factory = & TeamFactory::GetFactory();
		
		while (!$result->EOF) {
			// For each available right

			$profileRightToAdd = & $factory->newProfileRight(
				array(
					"id" => $result->Fields("right_type_id")
				)
			);
			
			//echoDebug("adding section ".$result->Fields("protected_section_id")." id ".$result->Fields("right_type_id"));
			
			$this->item->addMultiIncluded($result->Fields("protected_section_id"), $profileRightToAdd);
			
			$result->MoveNext();
		}
	}
	
	function dbPopulateDefaultRights() {
		
		$selectFields = array(
			"id"
		);
		$selectTableNames = array(
			"user_profiles"
		);
		$selectConstraints = array(
			"name" => "Anonymous"
		);
		
		$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields);
		
		$defaultId = $result->Fields("id");
		
		$realId = $this->item->id;
		
		/**
		* "I'm the anonymous profile for a while"
		*/
		$this->item->id = $defaultId;
				
		$this->dbPopulateRights();
		
		/**
		* Back to my real id
		*/
		$this->item->id = $realId;
		
	}
			
	function dbGetTableName() {
		return "user_profiles";
	}

	function dbAddToDb() {
		/**
		* 1: insert entry
		*/
		if (parent::dbAddToDb()) {

			/**
			* 2: get default rights from anonymous
			*/
			$this->dbPopulateDefaultRights();
			
			/**
			* 3: insert corresponding rights
			*/
			$this->dbUpdateAllRights();

			return true;

		} else {

			return false;

		}

	}
	
	function dbDeleteFromDb($val) {
		

		if ($this->item->isAllowedToDelete()) {
			$newProfileId = $val["profileId"];
			/**
			* 1: change members profiles
			*/
			$tableName = "team_members";
			$constraints = array(
				"profile_id" => $this->item->id
			);
			$fields = array(
				"profile_id" => $newProfileId
			);
			$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
			
			/**
			* 2: delete rights
			*/
			$deleteTableName = "profiles_rights";
			$deleteConstraints = array(
				"profile_id" => $this->item->id
			);
			$GLOBALS["gDb"]->deleteFromTable($deleteTableName, $deleteConstraints);
			
			/**
			* 3: delete entry
			*/
			parent::dbDeleteFromDb($val);

		}
	}
	
	/**
	* PRIVATE functions
	*/
	
	function dbGetFields() {
		$fields =  array(
			"name" => $this->item->title,
			"description" => $this->item->description
		);
		return $fields;
	}
	
	function dbUpdateRight($sectionId, $rightId) {
		
		if ($this->item->isAllowedToUpdateRights()) {
			$tableName = "profiles_rights";
			
			$fields = array(
				"right_type_id" => $rightId
			);
			
			// Verify if allready exists an entry
			$selectFields = array(
				"right_type_id"
			);
			$selectTableNames = array(
				"profiles_rights"
			);
			$selectConstraints = array(
				"profile_id" => $this->item->id,
				"protected_section_id" => $sectionId
			);
			
			$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields);

			if (!$result->EOF) {
				// There is allready an entry
				
				if ($result->Fields("right_type_id") != $rightId) {
					// Update only if necessary
					$GLOBALS["gDb"]->updateTable($tableName, $fields, $selectConstraints);
				}
				
				$result->MoveNext();
			} else {
				// There isn't any entry: add it
				$fields["profile_id"] = $this->item->id;
				$fields["protected_section_id"] = $sectionId;
				$GLOBALS["gDb"]->insertIntoTable($tableName, $fields);
			}
		}
	}
	
	function dbUpdateAllRights() {
		global $gProtectedSections;
		/**
		* For all protected sections in db,
		* if there is a value in includedItems (with the section_id),
		* update it in db. If not, put "no".
		*/
		while(list($sectionId, $value) = each($gProtectedSections)) {
			
			if (isSet($this->item->includedItems[$sectionId])) {
				$rightId = $this->item->includedItems[$sectionId]->id;
			} else {
				/**
				* TODO
				*/
				$rightId = 1;
			}
			
			//echoDebug("time to update right ".$sectionId." = ".$rightId);
			
			$this->dbUpdateRight($sectionId, $rightId);
		}

		$this->item->setItemModified();
	}
		
	/**
	* STATIC functions
	*/
	
	function & httpGetPostItem($val) {
		
		if ($val["id"] < 0) {
			$val["id"] = - $val["id"];
		}

		$factory = & TeamFactory::GetFactory();

		$profile = & $factory->newProfile(
			array(
				"id"			=> $val["id"],
				"title"			=> $val["name"],
				"description"	=> $val["description"]
			)
		);
		
		echoDebug("Profile id ".$val["id"]);
		
		if (isSet($val["numRights"])) {
			
			$numRights = $val["numRights"];
			for ($i = 0; $i<$numRights; $i++) {
				
				$sectionId = $val["section_".$i."_id"];
				$rightId = $val["right_".$sectionId];

				$factory = & TeamFactory::GetFactory();
						
				$profileRightToAdd = & $factory->newProfileRight(
					array(
						"id" => $rightId
					)
				);
				
				echoDebug("right: ".$sectionId." = ".$rightId);
				
				$profile->includedItems[$sectionId] = $profileRightToAdd;
			}
		}
		
		return $profile;
	}
}
// *********************************************************************
// Class ProfileRightControler
// *********************************************************************
class ProfileRightControler extends XPItemControler {
}
?>
