<?php
define("PRINT_ALL_SQL",false);

require_once(XP_ADODB_BASEDIR."/adodb.inc.php");

class QueryConstraints {
	var $fields;
	var $tableNames;
	var $constraints;
	var $groupBy;
	var $orderBy;
	var $customConstraints;

	function QueryConstraints(
		$tableNames = array(),
		$constraints = array(),
		$fields = array("*"),
		$groupBy = "",
		$orderBy = "",
		$customConstraints = "") {
		
		$this->fields				= $fields;
		$this->tableNames			= $tableNames;
		$this->constraints			= $constraints;
		$this->groupBy				= $groupBy;
		$this->orderBy				= $orderBy;
		$this->customConstraints	= $customConstraints;
	}

}

class BaseXpDatabase extends XPItem {
	var $connection;
	var $databaseType;
	var $location;
	var $account;
	var $password;
	var $database;
	var $connected;
	var $status = -1;
	var $ERROR_NO_CONNECTION = 1;
	var $ERROR_NO_DB = 2;
	var $CONNECTION_OK = 0;
	
	var $version;
	var $type;
	var $id = "db";
	
	var $printUpdate = false;
	
	// Specific SQL characters
	var $SQL_QUOTE_CHAR = '"';
	var $SQL_ID_QUOTE_CHAR = '`';
	var $SQL_IF_NOT_EXISTS	= " IF NOT EXISTS ";
	
	/**
	 * Tells if any post needs to be processed
	 */
	function isInPost() {

		$switch = @$_GET["switch"];

		return (ereg("init", $switch) || ereg("update",$switch));
	}

	function postPrintUpdate() {

		$switch = @$_GET["switch"];

		if (ereg("print", $switch)) {

			$this->switchToPrintUpdate();

		} else {

			$this->switchToPrintUpdate(false);

		}

	}

	/**
	 * Function to process post
	 */
	function processPost() {
		
		$switch = @$_GET["switch"];
		
		if (ereg("init", $switch)) {
			
			// Build full database
			$result = $this->buildFullDatabase();
	
		} elseif (ereg("update",$switch)) {
	
			// Update from existing DB
			$result = $this->updateToLatestVersion();
	
		}
	
		if (!$this->printUpdate) {
	
			// Was actually executed...
	
			if ($result) {
				$this->connect();
			}
		    
		}
	
		return $result;
		
	}
	
	function switchToPrintUpdate($printUpdate = true) {
		$this->printUpdate = $printUpdate;
	}
	
	function handleQuotes($value) {
		// We first replace \" by ", because in recent versions of PHP
		// the replacement is already done!!
		$result = str_replace("\\\"", "\"", $value);
		return str_replace("\"", "\\\"", $result);
	}
	
	function BaseXpDatabase($databaseType) {

		$this->databaseType = $databaseType;
	}

	function isReadyToRock() {
		return ($this->isConnected() && $this->isLastVersion());
	}
	
	/**
	* Connects to the current database host and database name
	*/
	function connect() {
		$this->connected = false;
		$this->connection = ADONewConnection($this->databaseType);
		//$this->connection->debug = true;
		$this->connected = @$this->connection->PConnect(
			$this->location,
			$this->account,
			$this->password,
			$this->database);
			
		$errorMsg = $this->connection->ErrorMsg();
		
		if ($this->connected) {
			
			$this->status = $this->CONNECTION_OK;
			
		} else {

			if ($errorMsg == "") {
				// not connected but error message empty : pb with host
				$this->status = $this->ERROR_NO_CONNECTION;
			} elseif (ereg($this->database,$errorMsg)) {
				// db doesn't exist
				$this->status = $this->ERROR_NO_DB;
			}
		}
		return $this->connected;
	}

	/**
	* Connect to the current database host only
	*/
	function connectToHost() {	
		$this->connected = false;
		$this->connection = ADONewConnection($this->databaseType);
		return @$this->connection->PConnect($this->location, $this->account, $this->password);
	}
	
	/**
	* Gives status of current object
	*/
	function isConnected() {
		return $this->connected;
	}
	
	/**
	* Disconnects from currently connected database
	*/ 
	function disconnect() {
		$this->connection->close();
		$this->connected = false;
	}
	
	/**
	* Test the existance of a database
	* named as in Config.inc.php
	*/
	function dbExists() {
		if (!$this->isConnected()) {
			$this->connectToHost();
		}
		$databases = $this->connection->MetaDatabases();
		for ($i=0; $i < sizeOf($databases); $i++) {
			$name = $databases[$i]["name"];
			if ($name == $this->database) {
				return true;
			}
		}
		return false;
	}
	
	/**
	* Creates a brand new database for latest XPWeb version
	*/
	function buildFullDatabase() {

		$modified = ($this->createEmptyDatabase() != false);
				
		$this->disconnect();
		$this->connect();

		$modified &= $this->updateFromVersion(0);
		
		return $modified;
	}
	
	/**
	* tests version of an existing database
	*/
	function getVersion() {
		if (!isset($this->version)) {
			$this->version = $this->refreshVersion();
		}
		return $this->version;
	}
	
	function refreshVersion() {
		if (!$this->isConnected()) {
			$this->connectToHost();
		}
		// Empty database
		$tables = $this->connection->MetaTables();
		if (sizeOf($tables) == 0) {
			return 0;
		}
		
		// 2.2 and over
		$keepGoing = false;
		for ($i=0; $i < sizeOf($tables); $i++) {
			$name = $tables[$i];
			if ($name == "iterations") {
				$keepGoing = true;
			}
			if ($name == "version_info") {
				$query = "SELECT version_no FROM `version_info` ORDER BY `version_no` DESC  LIMIT 1";
				$result = $this->executeQuery($query);
				if (!$result->EOF) {
					return $result->Fields("version_no");
				}
			}
		}
		
		if (!$keepGoing) {
			// table "iterations" has not been found
			return 0;
		}
		
		// 2.1 and over
		$query = "DESCRIBE ".$this->database.".iterations";
		$result = $this->executeQuery($query);
		while ($result && !$result->EOF) {
			$collumn = $result->Fields("Field");
			if ($collumn == "id") {
				if (ereg("unsigned",$result->Fields("Type"))) {
					// the collumn type is up to date
					return 2.1;
					break;
				}
			}
			$result->MoveNext();
		}
		// 2.0
		for ($i=0; $i < sizeOf($tables); $i++) {
			$name = $tables[$i];
			if ($name == "tbl_users") {
				return 2;
			}
		}
		// else ==> 1.x
		return 1;
	}
	
	/**
	* Checks if database version is the same than scripts version
	*/
	function isLastVersion(){
		return $this->getVersion() == XP_SOFT_VERSION;
	}

	function echoSQLComment($text) {
		if ($this->printUpdate) {
			echo "-- $text<br/>";
		} else {
			echoDebug($text);
		}
	}

	function updateFromVersion($version){
		
		ob_start();
		
		$result = true;
		$i = 0;
		
		//setDebug();
		
		$totalTimeSpent = 0;
		
		switch($version){
			case 0:
				startMesure();
				$this->echoSQLComment("Update From 0");
				$result &= $this->createVersion1Tables();
				$totalTimeSpent += endMesure();
			case 1:
				startMesure();
				$this->echoSQLComment("Update From 1 ($result)");
				$result &= $this->updateToVersion2();
				$totalTimeSpent += endMesure();
				
				// @attention Exists (break doesn't work) so that it doesn't rich timeout (30 seconds)
				// when installing from scratch...
				// Redirects to update
				
				// Only if not printing...
				if (!$this->printUpdate) {
					
					if ($result) {
						
						displayRedirectToUrl(getReloadUrl($GLOBALS["gThisPage"])."?switch=update".($this->printUpdate?"print":"")."&id=".$this->id);
						exit(1);
						
					} else {
						
						return $result;
						
					}
					
				}
			case 2:
				startMesure();
				$this->echoSQLComment("Update From 2 ($result)");
				$result &= $this->updateToVersion21();
				$totalTimeSpent += endMesure();
			case 2.1:
				startMesure();
				$this->echoSQLComment("Update From 2.1 ($result)");
				$result &= $this->updateToVersion22();
				echoDebug("updateToVersion22: $result");
				$result &= $this->updateVersion21EntriestoVersion22();
				echoDebug("updateVersion21EntriestoVersion22: $result");
				$result &= $this->cleanUpAfterVersion22();
				echoDebug("cleanUpAfterVersion22: $result");
				$totalTimeSpent += endMesure();
			case 2.2:
				startMesure();
				$this->echoSQLComment("Update From 2.2 ($result)");
				$result &= $this->updateToVersion23();
				$totalTimeSpent += endMesure();
			case "2.3":
				startMesure();
				$this->echoSQLComment("Update From 2.3 ($result)");
				$result &= $this->updateToVersion231();
				$totalTimeSpent += endMesure();
			case "2.3.1":
				startMesure();
				$this->echoSQLComment("Update From 2.3.1 ($result)");
				$result &= $this->updateToVersion24();
				$totalTimeSpent += endMesure();
			case "2.4":
				startMesure();
				$this->echoSQLComment("Update From 2.4 ($result)");
				$result &= $this->updateToVersion25();
				$totalTimeSpent += endMesure();
			case "2.5":
				startMesure();
				$this->echoSQLComment("Update From 2.5 ($result)");
				$result &= $this->updateToVersion251();
				$totalTimeSpent += endMesure();
			case "2.5.1":
				startMesure();
				$this->echoSQLComment("Update From 2.5.1 ($result) => nothing to do");
				$totalTimeSpent += endMesure();
			case "2.5.2":
				startMesure();
				$this->echoSQLComment("Update From 2.5.2 ($result)");
				$result &= $this->updateToVersion3();
				$totalTimeSpent += endMesure();
			case "3.0":
				startMesure();
				$this->echoSQLComment("Update From 3.0 ($result)");
				$result &= $this->updateToVersion301();
				$totalTimeSpent += endMesure();
			case "3.0.1":
				startMesure();
				$this->echoSQLComment("Update From 3.0.1 ($result)");
				$result &= $this->updateToVersion31();
				$totalTimeSpent += endMesure();
			case "3.1":
				startMesure();
				$this->echoSQLComment("Update From 3.1 ($result)");
				$result &= $this->updateToVersion32();
				$totalTimeSpent += endMesure();
			case "3.2":
			case "3.2.1":
				startMesure();
				$this->echoSQLComment("Update From 3.2 or 3.2.1 ($result)");
				$result &= $this->updateToVersion33();
				$totalTimeSpent += endMesure();
			case "3.3":
				// Nothing to do...
			case "3.3.1":
				// Nothing to do...
		} // switch
		// Update version number
		$this->echoSQLComment("Time to set version in DB ($result) : ".XP_SOFT_VERSION);
		
		echoDebug("Spent $totalTimeSpent seconds updating DB");
		
		$result &= $this->setVersionNumber(XP_SOFT_VERSION);
		
		ob_end_flush();
		
		return $result;
	}

	
	/**
	* Update an existing version 1.x database into the
	* new database structure for version 2,
	* updating existing data
	*/
	function updateToVersion2() {
		echoDebug("createVersion2Tables");
		$modified = $this->createVersion2Tables();
		echoDebug("updateVersion1EntriesToVersion2");
		$modified &= $this->updateVersion1EntriesToVersion2();
		echoDebug("done");
		return $modified;
	}

	/**
	* Updates existing database from version 1.x or 2.0 to 2.1
	*/
	function updateToVersion21() {
		$result = true;
		$result &= $this->updateColumnType("projects","id", true);
		$result &= $this->updateColumnType("team_members","id", true);
		$result &= $this->updateColumnType("projects_affectation","project_id");
		$result &= $this->updateColumnType("projects_affectation","team_member_id");
		$result &= $this->updateColumnType("metaphor_words","id", true);
		$result &= $this->updateColumnType("metaphor_words","project_id");
		$result &= $this->updateColumnType("project_links","id", true);
		$result &= $this->updateColumnType("project_links","project_id");
		$result &= $this->updateColumnType("web_categories","id", true);
		$result &= $this->updateColumnType("web_categories","project_id");
		$result &= $this->updateColumnType("web_links","id", true);
		$result &= $this->updateColumnType("web_links","category_id");
		$result &= $this->updateColumnType("iterations","id", true);
		$result &= $this->updateColumnType("iterations","project_id");
		$result &= $this->updateColumnType("stories","id", true);
		$result &= $this->updateColumnType("stories","iteration_id");
		$result &= $this->updateColumnType("stories","project_id");
		$result &= $this->updateColumnType("tasks","id", true);
		$result &= $this->updateColumnType("tasks","story_id");
		$result &= $this->updateColumnType("tasks","iteration_id");
		$result &= $this->updateColumnType("tasks","project_id");
		$result &= $this->updateColumnType("tasks","responsible_team_member_id");
		return $result;
	}
	
	function getSQLAfterField($fieldName) {
		return " AFTER $fieldName";
	}

	/**
	* Updates existing database from version 1.x or 2.x to 2.2
	*/
	function updateToVersion22() {
		
		$result = true;
		
		$query = "ALTER TABLE tasks ADD done FLOAT DEFAULT '0' NOT NULL".$this->getSQLAfterField("weight");		
		$result &= 	$this->tryToExecuteQuery($query);
		$query = "ALTER TABLE tasks ADD todo FLOAT DEFAULT '0' NOT NULL".$this->getSQLAfterField("done");		
		$result &= 	$this->tryToExecuteQuery($query);
		$query = "ALTER TABLE tasks ADD end_date DATE DEFAULT '".XP_FORBIDDEN_DATE."' NOT NULL".$this->getSQLAfterField("todo");	
		$result &= 	$this->tryToExecuteQuery($query);
		$query = "ALTER TABLE tasks ADD theory_end_comp FLOAT DEFAULT '".XP_IMPOSSIBLE_ID."' NOT NULL".$this->getSQLAfterField("end_date");	
		$result &= 	$this->tryToExecuteQuery($query);
	
		$query = "CREATE TABLE version_info (version_no FLOAT NOT NULL ,PRIMARY KEY ( version_no ))";
		$result &= 	$this->tryToExecuteQuery($query);
		//		
		// Reports:  New feature addition.  Needs all the protected section stuff.
		//
				
		// Add protected section entry
		$result &= $this->updateAddProtectedSection("Reports");
		return $result;
	}
	
	/**
	* Updates existing database from version 2.2 to 2.3
	*/
	function updateToVersion23() {
    	// Add missing "no" right to Docs section...
        $constraints = array(
        	"name1" => "section",
            "name2" => $this->quote("Doc"),
            "type_name" => $this->quote("no")
        );
        
        $customConstraints = "protected_sections.id = available_section_rights.protected_section_id";
        $customConstraints .= " AND right_type_id = right_types.id";
        
        $tableNames = array(
            "right_types",
            "available_section_rights",
            "protected_sections"
        );

        // Verify this right doesn't exist
        $result = & $this->selectOneEntry($tableNames, $constraints, array("*"), "", "", $customConstraints);
	
        if (($result != false) && $result->EOF) {
		// Needs to be added
		// Add this right
		$query = "
    			INSERT INTO	available_section_rights (protected_section_id, right_type_id)
    			SELECT		protected_sections.id as protectedId, right_types.id as rightId
    			FROM		protected_sections, right_types
    			WHERE		protected_sections.name1 = ".$this->quote("section")."
    			AND		protected_sections.name2 = ".$this->quote("Doc")."
    			AND		right_types.type_name = ".$this->quote("no");

			$bResult = ($this->executeQuery($query) != false);

        } else {
        	$bResult = true;
        }
        
        return $bResult;
    }
    
    function updateToVersion231() {
        // Change "version-info" structure
        $query = "ALTER TABLE version_info CHANGE version_no version_no VARCHAR(10) DEFAULT '0' NOT  NULL";
        $result = $this->tryToExecuteQuery($query);
        
        return $result;
    }
    
    function updateToVersion24() {
	    	$query = "ALTER TABLE tasks ADD auto INT DEFAULT '1' NOT NULL".$this->getSQLAfterField("end_date");
	    	$result = $this->tryToExecuteQuery($query);
	    	echoDebug("updateToVersion24: calculateAllTasksEndDates true");
	    	$this->calculateAllTasksEndDates();
	    	echoDebug("updateToVersion24: calculateAllTasksEndDates done");
	    	
	    	return $result;
    }
    
    function updateAddProtectedSection($sectionName) {
		// Add protected section entry
		$this->addProtectedSection("section", $sectionName, array("no", "all", "proj"));

		// Add default "all" right to profiles
		$query = "
			INSERT INTO	profiles_rights (profile_id, protected_section_id, right_type_id)
			SELECT		user_profiles.id as profileId, protected_sections.id as protectedId, right_types.id as rightId
			FROM		user_profiles, protected_sections, right_types
			WHERE		protected_sections.name1 = ".$this->quote("section")."
			AND		protected_sections.name2 = ".$this->quote($sectionName)."
			AND		right_types.type_name = ".$this->quote("all");
				
		$result = $this->tryToExecuteQuery($query);
		
		return $result;
    }
    
    function updateToVersion25() {
    	// Add protected section "XPCalendar"
	echoDebug("updateToVersion25: addprotected XPCalendar");
    	$result = $this->updateAddProtectedSection("XPCalendar");
    	
    	// Completion calculation has been updated: it is necessary to re-update tasks
	// (BUG FIX)
	echoDebug("updateToVersion25: calculateAllTasksEndDates false");
    	$this->calculateAllTasksEndDates(false);
    	return $result;
    }
    
    function updateToVersion251() {
    	// Need to get rid of these stupid tinyint!!!!!
	$result = true;
	
	$result &= $this->updateColumnType("available_section_rights", "protected_section_id");
	$result &= $this->updateColumnType("available_section_rights", "right_type_id");
	
	$result &= $this->updateColumnType("profiles_rights", "profile_id");
	$result &= $this->updateColumnType("profiles_rights", "protected_section_id");
	$result &= $this->updateColumnType("profiles_rights", "right_type_id");
	
	$result &= $this->updateColumnType("project_link_types", "id", true);
	
	$result &= $this->updateColumnType("project_links", "type");
	
	$result &= $this->updateColumnType("protected_sections", "id", true);
	
    	$result &= $this->updateColumnType("right_types", "id", true);
	
	$result &= $this->updateColumnType("team_members", "profile_id");
    	
    	$result &= $this->updateColumnType("user_profiles", "id", true);
    	
	return $result;
    }
	
	function updateToVersion3() {
		// Fix bug introduced in previous version...
		// Need to remove and rewrite properly all available rights...
		
		
		// Delete all rights of sections
		$query = "DELETE FROM available_section_rights";
		$result = $this->tryToExecuteQuery($query);
						
		// Add all sections rights again
		$result &= $this->insertProtectedSections(true);
		
		// Add newer sections... (not included in insertProtectedSections)
		$result &= $this->addProtectedSection("section", "Reports", array("no", "all", "proj"), true);
		$result &= $this->addProtectedSection("section", "XPCalendar", array("no", "all", "proj"), true);
		
		return $result;
	}

	function updateToVersion301() {
		// Add all the foreign keys at the end...

		$result = true;
		
		$query = "ALTER TABLE team_members ADD CONSTRAINT FOREIGN KEY ( profile_id ) REFERENCES user_profiles( id )";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE iterations ADD CONSTRAINT FOREIGN KEY ( working_days_name ) REFERENCES working_days( setup_name )";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE tasks ADD CONSTRAINT FOREIGN KEY ( working_days_name ) REFERENCES working_days( setup_name )";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE projects_affectation  ADD CONSTRAINT FOREIGN KEY ( project_id ) REFERENCES projects ( id ), ADD CONSTRAINT FOREIGN KEY ( team_member_id ) REFERENCES team_members( id ) ";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE metaphor_words  ADD CONSTRAINT FOREIGN KEY ( project_id ) REFERENCES projects( id )";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE project_links  ADD CONSTRAINT FOREIGN KEY ( project_id ) REFERENCES projects( id ), ADD CONSTRAINT FOREIGN KEY ( type ) REFERENCES project_link_types( id ) ";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE web_categories ADD CONSTRAINT FOREIGN KEY ( project_id ) REFERENCES projects( id )";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE web_links  ADD CONSTRAINT FOREIGN KEY ( category_id ) REFERENCES web_categories( id ) ";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE iterations  ADD CONSTRAINT FOREIGN KEY ( project_id ) REFERENCES projects( id )";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE stories  ADD CONSTRAINT FOREIGN KEY ( project_id ) REFERENCES projects( id )";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE tasks  ADD CONSTRAINT FOREIGN KEY ( project_id ) REFERENCES projects( id )";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE available_section_rights  ADD CONSTRAINT FOREIGN KEY ( protected_section_id ) REFERENCES protected_sections( id ), ADD CONSTRAINT FOREIGN KEY ( right_type_id ) REFERENCES right_types( id )";
		$result &= $this->tryToExecuteQuery($query);

		$query = "ALTER TABLE profiles_rights  ADD CONSTRAINT FOREIGN KEY ( profile_id ) REFERENCES user_profiles( id ), ADD CONSTRAINT FOREIGN KEY ( protected_section_id ) REFERENCES protected_sections( id ), ADD CONSTRAINT FOREIGN KEY ( right_type_id ) REFERENCES right_types( id )";
		$result &= $this->tryToExecuteQuery($query);
		
		return $result;
	}

	function updateToVersion31() {
		// Default to "-1" was stupid!

		$query = "ALTER TABLE ".$this->quoteIdentifier("iterations")." CHANGE ".$this->quoteIdentifier("working_days_name")." ".$this->quoteIdentifier("working_days_name")." VARCHAR( 11 ) DEFAULT ".$this->quote("ite_0")." NOT NULL ";
		$result = $this->tryToExecuteQuery($query);
		
		return $result;
	}

	function updateToVersion32() {

		// Convert older cookies

		if (!$this->printUpdate) {
		
			if (isset($_COOKIE["workingXPProjectName"])) {
				setXPCookie(XP_PROJECT_NAME, $_COOKIE["workingXPProjectName"]);
				deleteXPCookie("workingXPProjectName");
			}

			if (isset($_COOKIE["workingXPProject"])) {
				setXPCookie(XP_PROJECT_ID, $_COOKIE["workingXPProject"]);
				deleteXPCookie("workingXPProject");
			}

			if (isset($_COOKIE["gXPLanguage"])) {
				setXPCookie(XP_LANGUAGE, $_COOKIE["gXPLanguage"]);
				deleteXPCookie("gXPLanguage");
			}

			// Set default sort order cookie
			setXPCookie(XP_PLANNING_POP_STORIES_ORDERBY, "|priority=DESC||risk=ASC|");

		}
		
		// Let's support pair programming!
		$query = "ALTER TABLE ".$this->quoteIdentifier("tasks")." ADD ".$this->quoteIdentifier("pair_team_member_id")." INT DEFAULT ".$this->quote("0")." NOT NULL".$this->getSQLAfterField("responsible_team_member_id");
		$result = $this->tryToExecuteQuery($query);

		// Fix bug that WASN'T fixed by update to 2.3!!!
		$this->updateToVersion23();
		
		return $result;
	}
	
	function updateToVersion33() {
		return true;
	}
    
    function calculateAllTasksEndDates($completedOnly = true) {
		$tasks = array();
		echoDebug("calculateAllTasksEndDates include");
		include_once "Controler/Projects_controler.inc.php";
		include_once "Model/Projects_model.inc.php";
		include_once "Model/Planning_model.inc.php";
		include_once "Controler/Planning_controler.inc.php";
		echoDebug("calculateAllTasksEndDates include done");
    	
    		$projectsFactory = & new ProjectsFactory();
		$planningFactory = & new PlanningFactory();
		
		$projectList = & $projectsFactory->newProjectList();
		$projectList->controler->dbPopulate();
		
		// For all projects
		echoDebug("calculateAllTasksEndDates read projects");
		reset($projectList->includedItems);
		
		while(list($key, $project) = each($projectList->includedItems)) {	
			// Build a Planning object
			echoDebug("Project to update: ".$project->id);
			$planning = & $planningFactory->newPlanning(array(
				"projectid" => $project->id)
			);
			$planning->controler->dbPopulateAllTasksFromProject();
			
			// Add tasks from this project
			$tasks = array_merge($tasks, $planning->multiIncludedItems["task"]);
		}
		echoDebug("calculateAllTasksEndDates read projects done");
				
		// Now $tasks contains ALL TASKS
		// For all tasks
		echoDebug("calculateAllTasksEndDates read tasks");
		reset($tasks);
		while(list($key, $task) = each($tasks)) {
			// The task must not be completed...
			echoDebug("Task to update: ".$task->id);
			$task->calculateActualCompletion();
			if ($completedOnly && $task->isCompleted()) {
				echoDebug("Don't update task ".$task->id.": completed");
			} else {
        		// Calculate end date
				//$task->calculateEndDate();
				
        		// update END DATE only
				$tableName = "tasks";
        		
        		$constraints = array(
        			"id" => $task->id
        		);
        		
        		$fields =  array(
        			"end_date" => $task->endDate
        		);
        		
        		$this->updateTable($tableName, $fields, $constraints);
			}
		}
		echoDebug("calculateAllTasksEndDates read tasks done");
    }
    
	function setVersionNumber($versionNum) {
		// Remove all previous entries
		$query = "DELETE FROM version_info";
        $result = $this->tryToExecuteQuery($query);
		// Update in db
		$query = "INSERT INTO version_info VALUES(".$this->quote($versionNum).")";
		$result &= $this->tryToExecuteQuery($query);
		// Now update version for object...
		unset($this->version);
		$this->getVersion();
		return $result;
	}
	
	/**
	* Removes useless field after tables have been updated to version 2.2
	*/
	function cleanUpAfterVersion22() {		
		$query = "ALTER TABLE tasks DROP completion";
		$result = $this->tryToExecuteQuery($query);
		$query = "ALTER TABLE tasks CHANGE weight weight FLOAT DEFAULT " . XP_IMPOSSIBLE_ID ." NOT NULL";
		$result &= $this->tryToExecuteQuery($query);
		return $result;
	}

	/**
	* Update to latest version, whatever the current one
	*/
	function updateToLatestVersion() {
		return $this->updateFromVersion($this->getVersion());
	}
	
	/**
	* Just creates the empty database
	*/
	function createEmptyDatabase() {
		$query = "CREATE DATABASE".$this->SQL_IF_NOT_EXISTS." ".$this->database;
		$result = $this->executeQuery($query);
		return $result;
	}
	
    /** This handles quoting a string
    * 
    */
    function quote($value) {
        return($this->SQL_QUOTE_CHAR . $this->handleQuotes($value) . $this->SQL_QUOTE_CHAR);
    }
    /** This handles quoting an identifier (i.e. table name, field name) 
    */
    function quoteIdentifier($value) {
        return($this->SQL_ID_QUOTE_CHAR . $value . $this->SQL_ID_QUOTE_CHAR);
    }
	
	/**
	* Selects entries in table $tableName, with constraints $constraints.
	* Fields can be specified in an array
	* or left blank (all fields are returned)
	*/
	function selectEntries($tableNames, $constraints, $fields = "*", $groupBy = "", $orderBy = "", $customConstraints = "") {
		$query = $this->_getQueryForSelect($tableNames, $constraints, $fields,  $groupBy, $orderBy, $customConstraints);
		
		$result = $this->executeQuery($query);
		
		return $result;
	}

	function selectEntriesObj(& $queryConstraints) {
		
		return $this->selectEntries(
			$queryConstraints->tableNames,
			$queryConstraints->constraints, 
			$queryConstraints->fields,
			$queryConstraints->groupBy,
			$queryConstraints->orderBy,
			$queryConstraints->customConstraints);
	}
	
	/**
	* Private function used by selectEntries and selectOneEntry
	*/
	function _getQueryForSelect($tableNames, $constraints, $fields = "*", $groupBy = "", $orderBy = "",
		$customConstraints = "") {

		$query = "SELECT ";
		
		if (!is_array($fields)) {
		
			$query .= $fields;
			
		} else {
		
			reset($fields);
			list($key,$value) = each($fields);
			$query .= $this->quoteIdentifier($value);
			while(list($key,$value) = each($fields)) {
				$query .= ", ".$this->quoteIdentifier($value);
			}
			
		}
		
		$query .= " FROM ";
		
		reset($tableNames);
		list($key,$value) = each($tableNames);
		$query .= $value;
		while(list($key,$value) = each($tableNames)) {
			$query .= ", ".$value;
		}
		
		reset($constraints);
		if (list($key,$value) = each($constraints)) {
			$query .= " WHERE (";
            if (is_array($value)) {
			    $query .= "$key $value[0] " . $this->quote( $value[1]);
            } else {
			    $query .= "$key = " . $this->quote( $value);
            }
			while(list($key,$value) = each($constraints)) {
				// UGLY : here there are quote before and after $value
                if (is_array($value)) {
                	$query .= " AND $key $value[0] $value[1] ";
                } else {
				    $query .= " AND $key = $value ";
                }
			}
			
			if ($customConstraints != "") {
				$query .= " AND (";
				$query .= $customConstraints;
				$query .= ")";
			}
					
			$query .= ")";
		} else {
			if ($customConstraints != "") {
				$query .= " WHERE (";
				$query .= $customConstraints;
				$query .= ")";
			}
		}
						
		if ($groupBy != "") {
			$query .= " GROUP BY (";
			
			reset($groupBy);
			
			list($key,$value) = each($groupBy);
			$query .= $value;
			while(list($key,$value) = each($groupBy)) {
				$query .= ", ".$value;
			}
			
			$query .= ")";
		}
		
		if ($orderBy != "") {
			$query .= " ORDER BY ";
			
			reset($orderBy);
			list($key,$value) = each($orderBy);
			$query .= $value;
			while(list($key,$value) = each($orderBy)) {
				$query .= ", ".$value;
			}
		}
		
		return $query;
	}
	
	/**
	* Same as selectEntries but return only first result (LIMIT = 1)
	*/
	function & selectOneEntry($tableNames, $constraints, $fields = "*", $groupBy = "", $orderBy = "",
		$customConstraints = "") {

		$query = $this->_getQueryForSelect($tableNames, $constraints, $fields, $groupBy, $orderBy, $customConstraints);
		
		echoDebug($query);
		
		// @note: need "@" because warning in PHP4...
		$result = @$this->connection->SelectLimit($query, 1);
		
		return $result;
	}
	
	/**
	* Updates the entry defined by $constraints, in table $tableName,
	* with the given $fields values
	*/
	function updateTable($tableName, $fields, $constraints) {
		$query = "UPDATE $tableName SET";
		
		reset($fields);
		list($key,$value) = each($fields);
		$query .= " ".$this->quoteIdentifier($key)." = " . $this->quote( $value);
		while(list($key,$value) = each($fields)) {
			$query .= ", ".$this->quoteIdentifier($key)." = " . $this->quote( $value);
		}
		
		$query .= " WHERE (";
		
		reset($constraints);
		list($key,$value) = each($constraints);
		$query .= "$key = ". $this->quote( $value);
		while(list($key,$value) = each($constraints)) {
			$query .= " AND $key = " . $this->quote( $value);
		}
		
		$query .= ")";
		
		return $this->executeQuery($query);
	}
	
	/**
	* Add an entry in table $tableName,
	* with the given $fields values
	*/
	function insertIntoTable($tableName, $fields) {
		$query = "INSERT INTO  $tableName (";
		unset($fields["id"]);
		$keys = "";
		$values = "";
		
		reset($fields);
		list($key,$value) = each($fields);
		$keys .= $this->quoteIdentifier($key);
		$values .= $this->quote($value);
		while(list($key,$value) = each($fields)) {
			$keys .= ", ".$this->quoteIdentifier($key);
			$values .= ", " . $this->quote($value);
		}
		
		$query .= $keys.")";
		$query .= " VALUES (".$values.")";

		return $this->executeQuery($query);
	}
	
	/**
	* Delete an entry from table $tableName,
	* with the given $constraints
	*/
	function deleteFromTable($tableName, $constraints) {
		$query = "DELETE FROM $tableName";
		
		$query .= " WHERE (";
		
		reset($constraints);
		list($key,$value) = each($constraints);
		$query .= "$key = " . $this->quote( $value);
		while(list($key,$value) = each($constraints)) {
			$query .= " AND $key = " . $this->quote( $value);
		}
		
		$query .= ")";
		
		return $this->executeQuery($query);
	}

	
	/**
	* Executes the given string sql query and returns a RecordSet object
	*/
	function executeQuery($query) {
        if (PRINT_ALL_SQL) {
            print("$query<br/>");
        }
        if ($this->printUpdate) {
		
        	if (substr($query, 0, 7) != "SELECT " && substr($query, 0, 9) != "DESCRIBE ") {
        		// We only print updates and creations
        		echo "$query;<br/>";
        	}
        	$result = & new ADORecordSet_empty();
		
        } else {
		
    		@($result = & $this->connection->Execute($query));
		
    		if ($result == false) {
			$sqlErrorMsg = "SQL ERROR:<br/>$query<br/>".$this->connection->ErrorMsg();
			
    			echoSpan("error", "$sqlErrorMsg<br/>");
			// Also generates an error
			error_log($sqlErrorMsg);
			
    		} else {
    			echoDebug($query);
    		}
        }
		return $result;
	}
	
	/**
	 * This one returns a BOOLEAN
	 */
	function tryToExecuteQuery($query) {
		return ($this->executeQuery($query) != false);
	}

	/**
	* Changes a collumn type into unsigned int
	*/
	function updateColumnType($tableName, $columnName, $autoIncrement = false) {
		$query = "ALTER TABLE ".$this->quoteIdentifier($tableName)." CHANGE ".$this->quoteIdentifier($columnName)." ".$this->quoteIdentifier($columnName)." INT UNSIGNED NOT NULL";
		
		if ($autoIncrement) {
			$query .= " AUTO_INCREMENT";
		} else {
			$query .= " DEFAULT ".$this->quote("0")."";
		}
		$result = $this->tryToExecuteQuery($query);
		return $result;
	}
	
	/**
	* Creates the tables necessary to use XPWeb version 1.x
	*/
	function createVersion1Tables() {
		$result = true;
		$query = "
	CREATE TABLE projects (
	  id tinyint(4) NOT NULL auto_increment,
	  name varchar(255) NOT NULL default '',
	  description longtext NOT NULL,
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		$query = "
	CREATE TABLE team_members (
	  id tinyint(4) NOT NULL auto_increment,
	  name varchar(255) NOT NULL default '',
	  description longtext NOT NULL,
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		$query = "
	CREATE TABLE projects_affectation (
	  project_id tinyint(4) NOT NULL default '0',
	  team_member_id tinyint(4) NOT NULL default '0',
	  PRIMARY KEY  (project_id,team_member_id)	  
	)";
		$result &= $this->tryToExecuteQuery($query);
		$query = "
	CREATE TABLE metaphor_words (
	  id tinyint(4) NOT NULL auto_increment,
	  name varchar(255) NOT NULL default '',
	  description longtext NOT NULL,
	  project_id tinyint(4) NOT NULL default '0',
	  PRIMARY KEY  (id)  
	)";
		$result &= $this->tryToExecuteQuery($query);
		$query = "
	CREATE TABLE project_link_types (
	  id tinyint(4) NOT NULL auto_increment,
	  name varchar(255) NOT NULL default '',
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		$query = "
	CREATE TABLE project_links (
	  id tinyint(4) NOT NULL auto_increment,
	  url varchar(255) NOT NULL default '',
	  comment varchar(255) NOT NULL default '',
	  project_id tinyint(4) NOT NULL default '0',
	  type tinyint(4) NOT NULL default '0',
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		$query = "
	CREATE TABLE web_categories (
	  id tinyint(4) NOT NULL auto_increment,
	  name varchar(255) NOT NULL default '',
	  project_id tinyint(4) NOT NULL default '0',
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		$query = "
	CREATE TABLE web_links (
	  id tinyint(4) NOT NULL auto_increment,
	  url varchar(255) NOT NULL default '',
	  name varchar(255) NOT NULL default '',
	  description longtext NOT NULL,
	  category_id tinyint(4) NOT NULL default '0',
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		$query = "
	CREATE TABLE iterations (
	  id tinyint(4) NOT NULL auto_increment,
	  project_id tinyint(4) NOT NULL default '0',
	  name varchar(255) NOT NULL default '',
	  description longtext NOT NULL,
	  date date NOT NULL default '0000-00-00',
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		$query = "
	CREATE TABLE stories (
	  id tinyint(4) NOT NULL auto_increment,
	  iteration_id tinyint(4) NOT NULL default '0',
	  project_id tinyint(4) NOT NULL default '0',
	  name varchar(255) NOT NULL default '',
	  description longtext NOT NULL,
	  validation_criteria longtext NOT NULL,
	  priority tinyint(2) NOT NULL default '0',
	  risk tinyint(2) NOT NULL default '0',
	  weight tinyint(2) NOT NULL default '0',
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		$query = "
	CREATE TABLE tasks (
	  id tinyint(4) NOT NULL auto_increment,
	  story_id tinyint(4) NOT NULL default '0',
	  iteration_id tinyint(4) NOT NULL default '0',
	  project_id tinyint(4) NOT NULL default '0',
	  responsible_team_member_id tinyint(4) NOT NULL default '0',
	  name varchar(255) NOT NULL default '',
	  description longtext NOT NULL,
	  start_date date NOT NULL default '0000-00-00',
	  completion tinyint(2) NOT NULL default '0',
	  weight tinyint(2) NOT NULL default '0',
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		
		/**
		* Insert entries for link types
		*/
		$query = "INSERT INTO project_link_types VALUES (1, 'modeling')";
		$result &= $this->tryToExecuteQuery($query);
		
		$query = "INSERT INTO project_link_types VALUES (2, 'test')";
		$result &= $this->tryToExecuteQuery($query);
		
		$query = "INSERT INTO project_link_types VALUES (3, 'doc')";
		$result &= $this->tryToExecuteQuery($query);
		
		$query = "INSERT INTO project_link_types VALUES (4, 'cvs')";
		$result &= $this->tryToExecuteQuery($query);
		
		$query = "INSERT INTO project_link_types VALUES (5, 'directory')";
		$result &= $this->tryToExecuteQuery($query);
		
		$query = "INSERT INTO project_link_types VALUES (6, 'javadoc')";
		$result &= $this->tryToExecuteQuery($query);
		
		return $result;
	}
	
	/**
	* Creates the new tables necessary to use XPWeb version 2
	* Includes tables used by php_lib_login
	* Also updates existing version 1.x tables
	*/
	function createVersion2Tables() {
		global $XP_DEFAULT_ITERATION_WORKING_PACE;
		global $gDeveloperProfileId;
		
		$result = true;
	
		// php_lib_login tables creation
		
		$query = "
	CREATE TABLE tbl_users(
	username varchar(40) PRIMARY KEY,
	password varchar(60) NOT NULL,
	email varchar(40) NOT NULL,
	question varchar(200),
	answer varchar(30),
	lastlogin integer
	)";
		$result &= $this->tryToExecuteQuery($query);
		
		$query = "
	CREATE TABLE tbl_log(
	timestamp integer NOT NULL,
	ip varchar(16) NOT NULL DEFAULT 'no ip logged' ,
	username varchar(40),
	action varchar(40),
	PRIMARY KEY(timestamp, ip) 
	)";
		$result &= $this->tryToExecuteQuery($query);
		
		$query = "
	CREATE TABLE tbl_banned(
	ip varchar(16) PRIMARY KEY
	)";
		$result &= $this->tryToExecuteQuery($query);
		
		$query = "
	CREATE TABLE tbl_tries(
	username varchar(40) PRIMARY KEY,
	tries int,
	nextlogin int
	)";
		$result &= $this->tryToExecuteQuery($query);
		
		$query = "
	CREATE TABLE tbl_group(
	username varchar(40) PRIMARY KEY,
	gid int
	)";
		$result &= $this->tryToExecuteQuery($query);
		
		$query = "
	insert into tbl_group select username, 0  from tbl_users";
		$result &= $this->tryToExecuteQuery($query);
		
		// New in XPWeb
		
		// ------------------------------------------------------------------
		// Regarding user rights...
		
		// Profiles
		$query = "
	CREATE TABLE user_profiles (
	  id tinyint(4) NOT NULL auto_increment,
	  name varchar(255) NOT NULL default '',
	  description longtext NOT NULL,
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		
		// Sections that need to be protected
		$query = "
	CREATE TABLE protected_sections (
	  id tinyint(4) NOT NULL auto_increment,
	  name1 varchar(100) NOT NULL default '',
	  name2 varchar(100) NOT NULL default '',
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		
		// Existing right types (ex "all", "proj", etc..)
		$query = "
	CREATE TABLE right_types (
	  id tinyint(4) NOT NULL auto_increment,
	  type_name varchar(100) NOT NULL default '',
	  PRIMARY KEY  (id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		
		// Rights that can be chosen for a specific section
		$query = "
	CREATE TABLE available_section_rights (
	  protected_section_id tinyint(4) NOT NULL default '0',
	  right_type_id tinyint(4) NOT NULL default '0',
	  PRIMARY KEY  (protected_section_id, right_type_id)
	)";
		$result &= $this->tryToExecuteQuery($query);
		
		// Rights that have been chosen for a profile and a protected section
		$query = "
		CREATE TABLE profiles_rights (
		profile_id tinyint(4) NOT NULL default '0',
		protected_section_id tinyint(4) NOT NULL default '0',
		right_type_id tinyint(4) NOT NULL default '0',
		PRIMARY KEY  (profile_id, protected_section_id)
		)";

		$result &= $this->tryToExecuteQuery($query);
		
		// Time to insert available rights
		$this->insertAllAvailableRights();
		
		// Time to add protected sections and the corresponding available rights
		$this->insertProtectedSections();
		
		// Time to add default profiles such as anonymous and admin
		$this->insertDefaultProfiles();
		
		// Now everything is ready for rights so it's time to update table Members
		// Existing members will have the developper profile by default.
		$query = "
	ALTER TABLE team_members ADD profile_id INT DEFAULT '".$gDeveloperProfileId."' NOT NULL".$this->getSQLAfterField("id");
		$result &= $this->tryToExecuteQuery($query);
		

		// ------------------------------------------------------------------
		// Working days for tasks and iterations
		$query = "
	CREATE TABLE working_days (
	setup_name VARCHAR( 11 ) DEFAULT '-1' NOT NULL ,
	Mon TINYINT( 1 ) DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Mon"]."' ,
	Tue TINYINT( 1 ) DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Tue"]."' ,
	Wed TINYINT( 1 ) DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Wed"]."' ,
	Thu TINYINT( 1 ) DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Thu"]."' ,
	Fri TINYINT( 1 ) DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Fri"]."' ,
	Sat TINYINT( 1 ) DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Sat"]."' ,
	Sun TINYINT( 1 ) DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Sun"]."',
	PRIMARY KEY ( setup_name ))";
		$result &= $this->tryToExecuteQuery($query);
		
		// Insert default working days
		$query = "
	INSERT INTO working_days (setup_name ,Mon ,Tue ,Wed ,Thu ,Fri ,Sat ,Sun )
	VALUES (
	'ite_0' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Mon"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Tue"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Wed"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Thu"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Fri"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Sat"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Sun"]."' 
	)";
		
		$result &= $this->tryToExecuteQuery($query);
			
		$query = "
	ALTER TABLE iterations ADD working_days_name VARCHAR( 11 ) DEFAULT '-1' NOT NULL".$this->getSQLAfterField("project_id");
		$result &= $this->tryToExecuteQuery($query);
				
		$query ="
	ALTER TABLE tasks ADD working_days_name VARCHAR( 11 ) DEFAULT 'ite_0' NOT NULL".$this->getSQLAfterField("responsible_team_member_id");
		$result &= $this->tryToExecuteQuery($query);
		
		// ------------------------------------------------------------------
		// Description for web links categories
		$query = "
	ALTER TABLE web_categories ADD description LONGTEXT".$this->getSQLAfterField("name");
		$result &= $this->tryToExecuteQuery($query);
		
		// ------------------------------------------------------------------
		
		// Load factor for iteration
		$query = "
	ALTER TABLE iterations ADD load_factor FLOAT DEFAULT '".XP_DEFAULT_LOAD_FACTOR."' NOT NULL".$this->getSQLAfterField("working_days_name");
		$result &= $this->tryToExecuteQuery($query);
		
		return $result;
	}
	
	/**
	 * Required by Postgre to use lowercase field names	
	 * @param $result
	 * @return array
	 */
	function getWorkingDaysFromResult($result) {
		return array(
			"Mon" => $result->Fields("Mon"),
			"Tue" => $result->Fields("Tue"),
			"Wed" => $result->Fields("Wed"),
			"Thu" => $result->Fields("Thu"),
			"Fri" => $result->Fields("Fri"),
			"Sat" => $result->Fields("Sat"),
			"Sun" => $result->Fields("Sun")
		);
	}
	
	function getWorkingDaysFields() {
		return array(
			"Mon",
			"Tue",
			"Wed",
			"Thu",
			"Fri",
			"Sat",
			"Sun"
		);
	}
	
	/**
	* Updates data from XPWeb version 1.x for them to be compatible with
	* new version 2 structure
	*/
	function updateVersion1EntriesToVersion2() {
		// * insert working_days for existing iterations
		// Get default working_days
		$query = "SELECT * FROM working_days WHERE setup_name = 'ite_0' LIMIT 1";
		$workingResult = $this->executeQuery($query);
		
		$query = "SELECT id FROM iterations";
		$result = $this->executeQuery($query);
		while (!$result->EOF) {
			// For all iterations
			$setup_name ="ite_".$result->Fields("id");
			
			// Insert entry in working_days
			$tableName = "working_days";
			$fields = $this->getWorkingDaysFromResult($workingResult);
			
			$fields["setup_name"] = $setup_name;
									
			$this->insertIntoTable($tableName, $fields);
			
			// Update entry in iterations
			$tableName = "iterations";
			$fields = array("working_days_name" => $setup_name);
			$constraints = array("id" => $result->Fields("id"));
			$this->updateTable($tableName, $fields, $constraints);
			
			// Update all tasks from this iteration
			$tableName = "tasks";
			$fields = array("working_days_name" => $setup_name);
			$constraints = array("iteration_id" => $result->Fields("id"));
			$this->updateTable($tableName, $fields, $constraints);
			
			// Update all tasks from stories from this iteration
			
			// get all tasks
			$query = "SELECT tasks.id FROM tasks, stories WHERE (tasks.story_id = stories.id AND stories.iteration_id = ".$result->Fields("id").")";
			$result2 = $this->executeQuery($query);
			while (!$result2->EOF) {
				// Set working_days and iteration_id fields...
				$query = "UPDATE $tableName SET working_days_name = " . $this->quote($setup_name) . ", iteration_id = " . $this->quote( $result->Fields("id")) . " WHERE (id=".$result2->Fields("id").")";
				$this->executeQuery($query);
				$result2->MoveNext();
			}
			
			$result->MoveNext();
			
		}
	
		$query = "SELECT id FROM team_members";
		$result = $this->executeQuery($query);
		while (!$result->EOF) {
			$memberId = $result->Fields("id");
			$error = lib_login_create_account($memberId, XP_DEFAULT_PASSWORD, XP_DEFAULT_PASSWORD, "test@test.com", "2+2?", "4");
			$result->MoveNext();
		}
		
		return true;
	}
	
	/**
	* Update tasks table from 2.1 to 2.2 format
	*/
	function updateVersion21EntriestoVersion22() {
		// First update done and todo
		$query = "SELECT id, load_factor FROM iterations";
		$result = $this->executeQuery($query);
		$lbIsolatedTasks = true;
		while($lbIsolatedTasks || !$result->EOF){
			// update of done and todo fields of the tasks table
			// update is based on completion %, weight and load factor
			if ($lbIsolatedTasks) {
				$iterationId = 0;
				$loadFactor = XP_DEFAULT_LOAD_FACTOR;
			} else {
				$iterationId = $result->Fields("id");
				$loadFactor = $result->Fields("load_factor");
			}

			$query2 = "UPDATE tasks SET done = ((completion*weight)/100)*$loadFactor,";
			$query2 .= " todo = ((100-completion)*weight)/100";
			$query2 .= " WHERE iteration_id = $iterationId";
			
			//echo "<br/>Request : $query2<br/>";
			$result2 = $this->executeQuery($query2);
			if ($lbIsolatedTasks) {
				$lbIsolatedTasks = false;
				// do the isolated tasks only once!
			} else {
				$result->MoveNext();
			}
			
		} // while

		$bResult = true;

		$query = "UPDATE tasks SET todo=-1 where completion <> 100 and todo = 0";
		$bResult &= $this->tryToExecuteQuery($query);
		$query = "UPDATE tasks SET done=1 where done< 0";
		$bResult &= $this->tryToExecuteQuery($query);
		
		
		// Update tasks => set "theory end completion", and end date...
		require_once "Model/Planning_model.inc.php";
		
		$resultProjects = $this->selectEntries( array("projects"), array(), array("id") );
		
		while (!$resultProjects->EOF ) {
			// for all projects
			$projectId = $resultProjects->Fields("id");
			$planning =  new Planning($projectId);
			$planning->controler->dbPopulate();
			$iterations = $planning->multiIncludedItems["iteration"];
			for ($iteI = 0; $iteI < sizeOf($iterations); $iteI ++) {
				// For all iterations of current project
				$stories = $iterations[$iteI]->multiIncludedItems["story"];
				for ($stoI = 0; $stoI < sizeOf($stories); $stoI ++) {
					// For all stories of current iteration
					$tasks = $stories[$stoI]->multiIncludedItems["task"];
					for ($taskI = 0; $taskI < sizeOf($tasks); $taskI ++) {
						// For all tasks of current story
						$currentTask = $tasks[$taskI];
						$this->updateTaskToVersion22($currentTask);
					}
				}
				$tasks = $iterations[$iteI]->multiIncludedItems["task"];
				for ($taskI = 0; $taskI < sizeOf($tasks); $taskI ++) {
					// For all tasks of current iteration
					$currentTask = $tasks[$taskI];
					$this->updateTaskToVersion22($currentTask);
				}
			}
			$stories = $planning->multiIncludedItems["story"];
			for ($stoI = 0; $stoI < sizeOf($stories); $stoI ++) {
				// For all isolated stories of current project
				$tasks = $stories[$stoI]->multiIncludedItems["task"];
				for ($taskI = 0; $taskI < sizeOf($tasks); $taskI ++) {
					// For all tasks of current story
					$currentTask = $tasks[$taskI];
					$this->updateTaskToVersion22($currentTask);
				}
			}	
			$tasks = $planning->multiIncludedItems["task"];
			for ($taskI = 0; $taskI < sizeOf($tasks); $taskI ++) {
				// For all isolated tasks of current project
				$currentTask = $tasks[$taskI];
				$this->updateTaskToVersion22($currentTask);
			}
			$resultProjects->MoveNext();
		}
			
		return $bResult;
	}
	
	/**
	* Updates an existing task to version 2.2
	* by setting "theory end completion", weight, and end date
	*/
	function updateTaskToVersion22(& $task) {
		global $gNow;
		
		
		$task->calculateActualCompletion();
		if ($task->actualCompletion >= 100) {
			$oldGNow = $gNow;
    		// End date is calculated based "now" and adds "todo".
    		// but we want to force it so it's calculated based on start date...
    		$gNow = $task->startTime;
    		
    		// Also, it's calculated with todo which is null here.
			// we'll use weight
			$oldTodo = $task->todo;
    		$task->todo = $task->weight;
    		
    		
			$task->calculateTheoryCompletion();
			$tableName = "tasks";
			// task is completed and will be updated in db
			// => considered as completed perfectly on time
			// (end date = calculated end date &
			// theory end completion = 100)
			$fields =  array(
    			"theory_end_comp" => 100,
    			"end_date" => $task->endDate
    		);
    		
    		$constraints = array(
    			"id" => $task->id
    		);
    		
    		$GLOBALS["gDb"]->updateTable($tableName, $fields, $constraints);
    		
    		$task->todo = $oldTodo;
    		$gNow = $oldGNow;
		}
	}
	
	
	/**
	* Inserts all available rights
	*/
	function insertAllAvailableRights() {
		$this->addAvailableRight("no");
		$this->addAvailableRight("all");
		$this->addAvailableRight("proj");
		$this->addAvailableRight("me");
		$this->addAvailableRight("my");
	}
	
	/**
	* Inserts available protected sections in version 2 and the rights 
	* that can be chosen for these sections
	*/
	function insertProtectedSections($rightsOnly = false) {

		$result = true;
		
		// -------------------------------
		// Sections visibility
		
		// Index
		$name2 = "index";
		$availableTypes = array(
			"no",
			"all"
		);
		// No "proj" rights for index... doesn't depend on project
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);
		
		// Projects
		$name2 = "Projects";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		/**
		* "proj" rights for Projects is a bit special:
		* It will not display content of other projects if false...
		*/
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);
		
		// Team
		$name2 = "Team";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);
		
		// UserProfiles
		$name2 = "UserProfiles";
		$availableTypes = array(
			"no",
			"all"
		);
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);
		
		// Planning
		$name2 = "Planning";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);
		
		// UserConfig
		$name2 = "UserConfig";
		$availableTypes = array(
			"no",
			"all"
		);
		// No "proj" rights for UserConfig... doesn't depend on project
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);
		
		// AdminConfig
		$name2 = "AdminConfig";
		$availableTypes = array(
			"no",
			"all"
		);
		// No "proj" rights for adminconfig... doesn't depend on project
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);

		// Metaphor
		$name2 = "Metaphor";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);
		
		// Metaphor
		$name2 = "Doc";
		$availableTypes = array(
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);
		
		// Tests
		$name2 = "Tests";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);
		
		// CVS
		$name2 = "CVS";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);
		
		// Links
		$name2 = "Links";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection("section", $name2, $availableTypes, $rightsOnly);
		
		// -------------------------------
		// Available actions
		
		// project
		$name1 = "project";
		$name2 = "select";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "modify";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "delete";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "insert";
		$availableTypes = array(
			"no",
			"all"
		);
		/**
		* No "insert" -> "proj" for project... would have
		* no sense!
		*/
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		
		// projectLink
		$name1 = "projectLink";
		$name2 = "modify";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "delete";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "insert";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		
		// profile
		$name1 = "profile";
		$name2 = "modify";
		$availableTypes = array(
			"no",
			"all"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "delete";
		$availableTypes = array(
			"no",
			"all"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "insert";
		$availableTypes = array(
			"no",
			"all"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		
		// member
		$name1 = "member";
		$name2 = "affect";
		$availableTypes = array(
			"no",
			"all",
			"proj",
			"me"
		);
		/**
		* For affectation, "proj" doesn't mean "is from the same
		* project as me", but "allowed to affect / deaffect
		* to my projects only"
		*/
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "modify";
		$availableTypes = array(
			"no",
			"all",
			"proj",
			"me"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "changePass";
		$availableTypes = array(
			"no",
			"all",
			"proj",
			"me"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "changeProfile";
		$availableTypes = array(
			"no",
			"all",
			"proj",
			"me"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "delete";
		$availableTypes = array(
			"no",
			"all",
			"proj",
			"me"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "insert";
		$availableTypes = array(
			"no",
			"all"
		);
		/**
		* No ["insert"]["proj"] for member... 
		* The right to add a member and to add it to project
		* is composed by the rights to add "all" and to
		* affect to project.
		*/
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		
		// iteration
		$name1 = "iteration";
		$name2 = "modify";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "delete";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "insert";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		
		// story
		$name1 = "story";
		$name2 = "modify";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "delete";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "insert";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		
		// task
		$name1 = "task";
		$name2 = "modify";
		$availableTypes = array(
			"no",
			"all",
			"proj",
			"my"
		);
		/**
		* NOTE: if user is allowed to delete all stories or
		* stories from the same project as a task,
		* idem for iterations, then it implies he is allowed
		* to delete tasks, no matter the value of 
		* $this->rights["task"]["delete"]["all"],
		* $this->rights["task"]["delete"]["proj"] and
		* $this->rights["task"]["delete"]["my"]
		*/
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "delete";
		$availableTypes = array(
			"no",
			"all",
			"proj",
			"my"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "insert";
		$availableTypes = array(
			"no",
			"all",
			"proj",
			"my"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		
		// metaphorWord
		$name1 = "metaphorWord";
		$name2 = "modify";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "delete";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "insert";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		
		// webLink
		$name1 = "webLink";
		$name2 = "modify";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "delete";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "insert";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		
		// webCategory
		$name1 = "webCategory";
		$name2 = "modify";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "delete";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);
		$name2 = "insert";
		$availableTypes = array(
			"no",
			"all",
			"proj"
		);
		$result &= $this->addProtectedSection($name1, $name2, $availableTypes, $rightsOnly);

		return $result;
	}
	
	/**
	* Inserts default profiles and their rights
	*/
	function insertDefaultProfiles() {
		global $gDeveloperProfileId;
		
		// **************************************************
		// Anonymous
		
		$name = "Anonymous";
		$description = "Access without logging in";
		
		/**
		* SECTIONS
		*/
		
		$profileRights["section"]["index"]			= "all";
		$profileRights["section"]["AdminConfig"]	= "no";
		$profileRights["section"]["Projects"]		= "all";
		$profileRights["section"]["Team"]			= "all";
		$profileRights["section"]["UserProfiles"]	= "no";
		$profileRights["section"]["Planning"]		= "all";
		$profileRights["section"]["UserConfig"]		= "all";
		$profileRights["section"]["Metaphor"]		= "all";
		$profileRights["section"]["Doc"]			= "all";
		$profileRights["section"]["Tests"]			= "all";
		$profileRights["section"]["CVS"]			= "all";
		$profileRights["section"]["Links"]			= "all";
		
		/**
		* ACTIONS
		*/
		$profileRights["project"]["select"] 		= "all";
			
		$profileRights["project"]["modify"] 		= "no";
		$profileRights["project"]["delete"] 		= "no";
		$profileRights["project"]["insert"] 		= "no";
		
		$profileRights["projectLink"]["modify"] 	= "no";
		$profileRights["projectLink"]["delete"] 	= "no";
		$profileRights["projectLink"]["insert"] 	= "no";
		
		$profileRights["profile"]["modify"] 		= "no";
		$profileRights["profile"]["delete"] 		= "no";
		$profileRights["profile"]["insert"] 		= "no";
		
		$profileRights["member"]["affect"] 			= "no";
		$profileRights["member"]["modify"] 			= "no";
		$profileRights["member"]["changePass"] 		= "no";
		$profileRights["member"]["changeProfile"] 	= "no";
		$profileRights["member"]["delete"] 			= "no";
		$profileRights["member"]["insert"] 			= "no";
		
		$profileRights["iteration"]["modify"] 		= "no";
		$profileRights["iteration"]["delete"] 		= "no";
		$profileRights["iteration"]["insert"] 		= "no";
		
		$profileRights["story"]["modify"] 			= "no";
		$profileRights["story"]["delete"] 			= "no";
		$profileRights["story"]["insert"] 			= "no";
					
		$profileRights["task"]["modify"] 			= "no";
		$profileRights["task"]["delete"] 			= "no";
		$profileRights["task"]["insert"] 			= "no";
		
		$profileRights["metaphorWord"]["modify"] 	= "no";
		$profileRights["metaphorWord"]["delete"] 	= "no";
		$profileRights["metaphorWord"]["insert"] 	= "no";
		
		$profileRights["webLink"]["modify"] 		= "no";
		$profileRights["webLink"]["delete"] 		= "no";
		$profileRights["webLink"]["insert"] 		= "no";
		
		$profileRights["webCategory"]["modify"] 	= "no";
		$profileRights["webCategory"]["delete"] 	= "no";
		$profileRights["webCategory"]["insert"] 	= "no";
		
		$this->addIncluded($name, $description, $profileRights);
		
		// **************************************************
		// Admin
		
		$name = "Admin";
		$description = "Access with admin login";
		
		/**
		* SECTIONS
		*/
		
		$profileRights["section"]["index"]			= "all";
		$profileRights["section"]["AdminConfig"]	= "all";
		$profileRights["section"]["Projects"]		= "all";
		$profileRights["section"]["Team"]			= "all";
		$profileRights["section"]["UserProfiles"]	= "all";
		$profileRights["section"]["Planning"]		= "all";
		$profileRights["section"]["UserConfig"]		= "all";
		$profileRights["section"]["Metaphor"]		= "all";
		$profileRights["section"]["Doc"]			= "all";
		$profileRights["section"]["Tests"]			= "all";
		$profileRights["section"]["CVS"]			= "all";
		$profileRights["section"]["Links"]			= "all";
		
		/**
		* ACTIONS
		*/
		$profileRights["project"]["select"] 		= "all";
			
		$profileRights["project"]["modify"] 		= "all";
		$profileRights["project"]["delete"] 		= "all";
		$profileRights["project"]["insert"] 		= "all";
		
		$profileRights["projectLink"]["modify"] 	= "all";
		$profileRights["projectLink"]["delete"] 	= "all";
		$profileRights["projectLink"]["insert"] 	= "all";
		
		$profileRights["profile"]["modify"] 		= "all";
		$profileRights["profile"]["delete"] 		= "all";
		$profileRights["profile"]["insert"] 		= "all";
		
		$profileRights["member"]["affect"] 			= "all";
		$profileRights["member"]["modify"] 			= "all";
		$profileRights["member"]["changePass"] 		= "all";
		$profileRights["member"]["changeProfile"] 	= "all";
		$profileRights["member"]["delete"] 			= "all";
		$profileRights["member"]["insert"] 			= "all";
		
		$profileRights["iteration"]["modify"] 		= "all";
		$profileRights["iteration"]["delete"] 		= "all";
		$profileRights["iteration"]["insert"] 		= "all";
		
		$profileRights["story"]["modify"] 			= "all";
		$profileRights["story"]["delete"] 			= "all";
		$profileRights["story"]["insert"] 			= "all";
					
		$profileRights["task"]["modify"] 			= "all";
		$profileRights["task"]["delete"] 			= "all";
		$profileRights["task"]["insert"] 			= "all";
		
		$profileRights["metaphorWord"]["modify"] 	= "all";
		$profileRights["metaphorWord"]["delete"] 	= "all";
		$profileRights["metaphorWord"]["insert"] 	= "all";
		
		$profileRights["webLink"]["modify"] 		= "all";
		$profileRights["webLink"]["delete"] 		= "all";
		$profileRights["webLink"]["insert"] 		= "all";
		
		$profileRights["webCategory"]["modify"] 	= "all";
		$profileRights["webCategory"]["delete"] 	= "all";
		$profileRights["webCategory"]["insert"] 	= "all";
		
		$this->addIncluded($name, $description, $profileRights);
		
		// **************************************************
		// Developer
		
		$name = "Developer";
		$description = "Member of the developing team";
		
		/**
		* SECTIONS
		*/
		
		$profileRights["section"]["index"]			= "all";
		$profileRights["section"]["AdminConfig"]	= "no";
		$profileRights["section"]["Projects"]		= "all";
		$profileRights["section"]["Team"]			= "all";
		$profileRights["section"]["UserProfiles"]	= "no";
		$profileRights["section"]["Planning"]		= "all";
		$profileRights["section"]["UserConfig"]		= "all";
		$profileRights["section"]["Metaphor"]		= "all";
		$profileRights["section"]["Doc"]			= "all";
		$profileRights["section"]["Tests"]			= "all";
		$profileRights["section"]["CVS"]			= "all";
		$profileRights["section"]["Links"]			= "all";
		
		/**
		* ACTIONS
		*/
		$profileRights["project"]["select"] 		= "proj";
			
		$profileRights["project"]["modify"] 		= "no";
		$profileRights["project"]["delete"] 		= "no";
		$profileRights["project"]["insert"] 		= "no";
		
		$profileRights["projectLink"]["modify"] 	= "no";
		$profileRights["projectLink"]["delete"] 	= "no";
		$profileRights["projectLink"]["insert"] 	= "no";
		
		$profileRights["profile"]["modify"] 		= "no";
		$profileRights["profile"]["delete"] 		= "no";
		$profileRights["profile"]["insert"] 		= "no";
		
		$profileRights["member"]["affect"] 			= "no";
		$profileRights["member"]["modify"] 			= "me";
		$profileRights["member"]["changePass"] 		= "me";
		$profileRights["member"]["changeProfile"] 	= "no";
		$profileRights["member"]["delete"] 			= "no";
		$profileRights["member"]["insert"] 			= "no";
		
		$profileRights["iteration"]["modify"] 		= "proj";
		$profileRights["iteration"]["delete"] 		= "proj";
		$profileRights["iteration"]["insert"] 		= "proj";
		
		$profileRights["story"]["modify"] 			= "proj";
		$profileRights["story"]["delete"] 			= "proj";
		$profileRights["story"]["insert"] 			= "proj";
					
		$profileRights["task"]["modify"] 			= "my";
		$profileRights["task"]["delete"] 			= "my";
		$profileRights["task"]["insert"] 			= "my";
		
		$profileRights["metaphorWord"]["modify"] 	= "proj";
		$profileRights["metaphorWord"]["delete"] 	= "proj";
		$profileRights["metaphorWord"]["insert"] 	= "proj";
		
		$profileRights["webLink"]["modify"] 		= "proj";
		$profileRights["webLink"]["delete"] 		= "proj";
		$profileRights["webLink"]["insert"] 		= "proj";
		
		$profileRights["webCategory"]["modify"] 	= "proj";
		$profileRights["webCategory"]["delete"] 	= "proj";
		$profileRights["webCategory"]["insert"] 	= "proj";
		
		$id = $this->addIncluded($name, $description, $profileRights);
		
		$gDeveloperProfileId = $id;
				
		// **************************************************
		// Coach
		
		$name = "Coach";
		$description = "Team coach";
		
		/**
		* SECTIONS
		*/
		
		$profileRights["section"]["index"]			= "all";
		$profileRights["section"]["AdminConfig"]	= "no";
		$profileRights["section"]["Projects"]		= "all";
		$profileRights["section"]["Team"]			= "all";
		$profileRights["section"]["UserProfiles"]	= "all";
		$profileRights["section"]["Planning"]		= "all";
		$profileRights["section"]["UserConfig"]		= "all";
		$profileRights["section"]["Metaphor"]		= "all";
		$profileRights["section"]["Doc"]			= "all";
		$profileRights["section"]["Tests"]			= "all";
		$profileRights["section"]["CVS"]			= "all";
		$profileRights["section"]["Links"]			= "all";		
		
		/**
		* ACTIONS
		*/
		$profileRights["project"]["select"] 		= "all";
			
		$profileRights["project"]["modify"] 		= "proj";
		$profileRights["project"]["delete"] 		= "proj";
		$profileRights["project"]["insert"] 		= "all";
		
		$profileRights["projectLink"]["modify"] 	= "proj";
		$profileRights["projectLink"]["delete"] 	= "proj";
		$profileRights["projectLink"]["insert"] 	= "proj";
		
		$profileRights["profile"]["modify"] 		= "no";
		$profileRights["profile"]["delete"] 		= "no";
		$profileRights["profile"]["insert"] 		= "no";
		
		$profileRights["member"]["affect"] 			= "proj";
		$profileRights["member"]["modify"] 			= "proj";
		$profileRights["member"]["changePass"] 		= "proj";
		$profileRights["member"]["changeProfile"] 	= "proj";
		$profileRights["member"]["delete"] 			= "proj";
		$profileRights["member"]["insert"] 			= "proj";
		
		$profileRights["iteration"]["modify"] 		= "proj";
		$profileRights["iteration"]["delete"] 		= "proj";
		$profileRights["iteration"]["insert"] 		= "proj";
		
		$profileRights["story"]["modify"] 			= "proj";
		$profileRights["story"]["delete"] 			= "proj";
		$profileRights["story"]["insert"] 			= "proj";
					
		$profileRights["task"]["modify"] 			= "proj";
		$profileRights["task"]["delete"] 			= "proj";
		$profileRights["task"]["insert"] 			= "proj";
		
		$profileRights["metaphorWord"]["modify"] 	= "proj";
		$profileRights["metaphorWord"]["delete"] 	= "proj";
		$profileRights["metaphorWord"]["insert"] 	= "proj";
		
		$profileRights["webLink"]["modify"] 		= "proj";
		$profileRights["webLink"]["delete"] 		= "proj";
		$profileRights["webLink"]["insert"] 		= "proj";
		
		$profileRights["webCategory"]["modify"] 	= "proj";
		$profileRights["webCategory"]["delete"] 	= "proj";
		$profileRights["webCategory"]["insert"] 	= "proj";
		
		$this->addIncluded($name, $description, $profileRights);
		
	}
	
	/**
	* Returns id of last inserted item 
	*/
	function lastInsertedId($tableName, $fieldName = "id") {
	    $id = $this->connection->Insert_ID($tableName,$fieldName);
		return $id;
	}
	
	
	
	/**
	* Returns the total number of a table
	*/
	function getNbItemsInTable($tableName) {
		
		$selectFields = array(
			"id"
		);
		$selectTableNames = array(
			$tableName
		);
		$selectConstraints = array();
		
		$result = $this->selectEntries(
			$selectTableNames,
			$selectConstraints,
			$selectFields
		);
		return $result->RecordCount();
	}
	
	/** PRIVATE
	*/
	
	function buildAvailableRights() {
		global $gRightTypes;
		$resultRightTypes = $this->selectEntries(
			array("right_types"),
			array(),
			array("*")
		);
		
		while (!$resultRightTypes->EOF) {			
			$id = $resultRightTypes->Fields("id");
			$name = $resultRightTypes->Fields("type_name");
			$gRightTypes[$name] = $id;
			$resultRightTypes->MoveNext();
		}
	}
	
	function addAvailableRight($name) {
		global $gRightTypes;
		$query = "
	INSERT INTO right_types (type_name)
		VALUES ('$name')";
		
		$result = $this->executeQuery($query);

		$id = $this->lastInsertedId("right_types");
		
		$gRightTypes[$name] = $id;
	}
	
	function addProtectedSection($name1, $name2, $availableTypes, $rightsOnly = false) {
		
		$result = true;

		if (!$rightsOnly) {
			// Add section
			$query = "
		INSERT INTO protected_sections (name1 ,name2)
			VALUES ('$name1' ,'$name2')";
			
			$result &= $this->tryToExecuteQuery($query);
		}
		
		$rightIds = "";
		$isFirst = true;
		
		foreach($availableTypes as $rightId) {
		
			if (!$isFirst) {
				$rightIds .= ", ";
			}
			
			$rightIds .= "'$rightId'";
			$isFirst = false;
		}
		
		$query = "
INSERT  INTO available_section_rights( protected_section_id, right_type_id ) 
SELECT protected_sections.id AS id1, right_types.id AS id2
FROM protected_sections, right_types
WHERE protected_sections.name1 =  '$name1' AND protected_sections.name2 =  '$name2'
AND right_types.type_name IN ($rightIds) 
";

		$result &= $this->tryToExecuteQuery($query);
	
		return $result;
	}
	
	function addIncluded($name, $description, $profileRights) {
		
		$result = true;
		
		// Add profile
		
		$query = "
			INSERT INTO user_profiles (name, description)
			VALUES ('$name' ,'$description')";
		
		$result &= $this->tryToExecuteQuery($query);
				
		// Add rights
		
		if ($result) {
			
			$lastInsertedId = $this->lastInsertedId("user_profiles");

			// for all rights of this profile
			reset($profileRights);
			while(list($key1,$value1) = each($profileRights)) {
				$name1 = $key1;
				
				// For all subsections in the section
				reset($profileRights[$name1]);
				while(list($key2,$value2) = each($profileRights[$name1])) {
					$name2 = $key2;
										
					$query = "
						INSERT INTO	profiles_rights (profile_id, protected_section_id, right_type_id)
						SELECT		user_profiles.id as profileId, protected_sections.id as protectedId, right_types.id as rightId
						FROM		user_profiles, protected_sections, right_types
						WHERE		user_profiles.name = '$name'
						AND		protected_sections.name1 = '$name1'
						AND		protected_sections.name2 = '$name2'
						AND		right_types.type_name = '$value2'";
					
					$result &= $this->tryToExecuteQuery($query);
				}
	
			}
			
		}
		
		return $lastInsertedId;
	}
    
}
?>
