<?php
function getMemberInfo($memberId) {
	
	$selectFields = array(
		"name",
		"profile_id"
	);
	$selectTableNames = array(
		"team_members"
	);
	$selectConstraints = array(
		"id" => $memberId
	);
	
	$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints, $selectFields);
	
	$name = $result->Fields("name");
	$profileId = $result->Fields("profile_id");
	
	$selectFields = array(
		"name"
	);
	$selectTableNames = array(
		"user_profiles"
	);
	$selectConstraints = array(
		"id" => $profileId
	);
	
	$groupBy = "";
	$orderBy = array(
		"id"
	);
	// Ordered by id ensures that we take the OLDEST profile,
	// preventing from newer profiles that would have the same name...
	
	$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints,
		$selectFields, $groupBy, $orderBy);
	
	$profileName = $result->Fields("name");
	
	return array(
		"name" => $name,
		"profileId" => $profileId,
		"profileName" => $profileName
	);
}

function populateMembers() {
	global $gMembers;
	global $gMemberProfilesNames;
	
	
	$maxMemberNameLenght = 0;
	$maxProfileNameLenght = 0;
	
	$selectFields = array(
		"id"
	);
	$selectTableNames = array(
		"team_members"
	);
	$selectConstraints = array(
            "id"=>array("!=",0)
	);
	$selectGroupBy = "";/*array(
		"id"
	);*/
	
	$selectOrderBy = array(
		"name asc"
	);
	
	$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
	while (!$result->EOF) {
		$properties = getMemberInfo($result->Fields("id"));
		$gMembers[$result->Fields("id")] = $properties["name"];
		$gMemberProfilesNames[] = $properties["profileName"];
		
		$result->MoveNext();
	}
}

class User {
	var $rights;
	var $id;
	var $profileId;
	var $profileName;
	var $isAdmin;
	var $isAnonymous;
	var $name;
	var $affectations;
	
	function copy($userToCopy) {
		$this->rights = $userToCopy->rights;
		$this->id = $userToCopy->id;
		$this->profileId = $userToCopy->profileId;
		$this->profileName = $userToCopy->profileName;
		$this->isAdmin = $userToCopy->isAdmin;
		$this->isAnonymous = $userToCopy->isAnonymous;
		$this->name = $userToCopy->name;
		$this->affectations = $userToCopy->affectations;
	}
	
	function User($id) {
		
		
		//echoDebug("User:Constructor => start");
		
		$this->affectations = array();
		$this->isAdmin = false;
		$this->isAnonymous = false;
		$this->id = $id;
		if ($this->id == XP_UBER_USER) {
			$this->isAdmin = true;
            $this->setAllSectionsRights(true);
            $this->setAllRights(true);
		} elseif ($this->id == "") {
			$this->isAnonymous = true;
		}
		
		$this->updateAffectations();
		
		if ($this->isAnonymous) {
			//echoDebug("User: anonymous");
			$this->name = "Anonymous";
			
			$selectFields = array(
				"id",
				"name"
			);
			$selectTableNames = array(
				"user_profiles"
			);
			$selectConstraints = array(
				"name" => "Anonymous"
			);
			$groupBy = "";
			$orderBy = array(
				"id"
			);
			// Ordered by id ensures that we take the OLDEST "Anonymous" profile,
			// preventing from newer profiles that would have the same name...
			
			$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints,
				$selectFields, $groupBy, $orderBy);
			
			if ($result != false) {
				$this->profileId = $result->Fields("id");
				$this->profileName = $result->Fields("name");
			}
			
			// No rights to modification
		
		} elseif ($this->isAdmin) {
			//echoDebug("User: admin");
			
			$this->name = "Admin";
			
			$selectFields = array(
				"id",
				"name"
			);
			$selectTableNames = array(
				"user_profiles"
			);
			$selectConstraints = array(
				"name" => "Admin"
			);
			
			$groupBy = "";
			$orderBy = array(
				"id"
			);
			
			// Ordered by id ensures that we take the OLDEST "Admin" profile,
			// preventing from newer profiles that would have the same name...
			
			$result = & $GLOBALS["gDb"]->selectOneEntry($selectTableNames, $selectConstraints,
				$selectFields, $groupBy, $orderBy);
			
			if ($result != null) {
				$this->profileId = $result->Fields("id");
				$this->profileName = $result->Fields("name");
			}
		} else {
			//echoDebug("User: team member");
			// A team member
			$properties = getMemberInfo($this->id);
			
			$this->name = $properties["name"];
			
			$this->profileId = $properties["profileId"];
			$this->profileName = $properties["profileName"];
		}
		
		$this->buildRightsFromProfile();
		
		//echoDebug("User:Constructor => end");
	}
	
	function isCurrentUser() {
		
		return $this->isMyself($GLOBALS["gConnectedUser"]);
	}
	
	function hasAdminProfile() {
		return ($this->profileName == "Admin");
	}
	
	function buildRightsFromProfile() {
		
		$selectFields = array(
			"name1",
			"name2",
			"type_name"
		);
		$selectTableNames = array(
			"profiles_rights",
			"protected_sections",
			"right_types"
		);
		$selectConstraints = array(
			"profile_id" => $this->profileId,
			"protected_section_id" => "protected_sections.id",
			"right_type_id" => "right_types.id"
		);
		
		$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields);
		
		
		if($result != false) {
			while (!$result->EOF) {
				// For each available right
				$this->rights[$result->Fields("name1")][$result->Fields("name2")][$result->Fields("type_name")]		= true;
				$result->MoveNext();
			}
		}
	}
	
	function setAllSectionsRights($setTo = false) {
		// Sections

		$this->rights["section"]["index"]["all"]			= $setTo;
		// No "proj" rights for index... doesn't depend on project
		
		$this->rights["section"]["Projects"]["all"]			= $setTo;
		$this->rights["section"]["Projects"]["proj"]		= $setTo;
		
		
		$this->rights["section"]["Team"]["all"]				= $setTo;
		$this->rights["section"]["Team"]["proj"]			= $setTo;
		
		$this->rights["section"]["Planning"]["all"]			= $setTo;
		$this->rights["section"]["Planning"]["proj"]		= $setTo;
		
		$this->rights["section"]["UserConfig"]["all"]		= $setTo;
		// No "proj" rights for UserConfig... doesn't depend on project

		$this->rights["section"]["Reports"]["all"]			= $setTo;
		$this->rights["section"]["Reports"]["proj"]			= $setTo;
		
		$this->rights["section"]["Metaphor"]["all"]			= $setTo;
		$this->rights["section"]["Metaphor"]["proj"]		= $setTo;
		
		$this->rights["section"]["Doc"]["all"]				= $setTo;
		$this->rights["section"]["Doc"]["proj"]				= $setTo;
		
		$this->rights["section"]["Tests"]["all"]			= $setTo;
		$this->rights["section"]["Tests"]["proj"]			= $setTo;
		
		$this->rights["section"]["CVS"]["all"]				= $setTo;
		$this->rights["section"]["CVS"]["proj"]				= $setTo;
		
		$this->rights["section"]["Links"]["all"]			= $setTo;
		$this->rights["section"]["Links"]["proj"]			= $setTo;
		
		$this->rights["section"]["XPCalendar"]["all"]			= $setTo;
		$this->rights["section"]["XPCalendar"]["proj"]			= $setTo;
	}
	
	function setAllRights($setTo = false) {
		
		
		// Rights on seeing projects
		
		$this->rights["project"]["select"]["all"]		= $setTo;
		$this->rights["project"]["select"]["proj"]		= $setTo;
		
		// Modifications
		
		$this->rights["project"]["modify"]["all"]		= $setTo;
		$this->rights["project"]["modify"]["proj"]		= $setTo;
		$this->rights["project"]["delete"]["all"]		= $setTo;
		$this->rights["project"]["delete"]["proj"]		= $setTo;
		$this->rights["project"]["insert"]["all"]		= $setTo;
		
		$this->rights["projectLink"]["modify"]["all"]		= $setTo;
		$this->rights["projectLink"]["modify"]["proj"]		= $setTo;
		$this->rights["projectLink"]["delete"]["all"]		= $setTo;
		$this->rights["projectLink"]["delete"]["proj"]		= $setTo;
		$this->rights["projectLink"]["insert"]["all"]		= $setTo;
		$this->rights["projectLink"]["insert"]["proj"]		= $setTo;
		
		$this->rights["member"]["affect"]["all"]	= $setTo;
		$this->rights["member"]["affect"]["proj"]	= $setTo;
		$this->rights["member"]["affect"]["me"]		= $setTo;
		$this->rights["member"]["modify"]["all"]	= $setTo;
		$this->rights["member"]["modify"]["proj"]	= $setTo;
		$this->rights["member"]["modify"]["me"]		= $setTo;
		$this->rights["member"]["delete"]["all"]	= $setTo;
		$this->rights["member"]["delete"]["proj"]	= $setTo;
		$this->rights["member"]["delete"]["me"]		= $setTo;
		$this->rights["member"]["insert"]["all"]	= $setTo;
		
		$this->rights["iteration"]["modify"]["all"]		= $setTo;
		$this->rights["iteration"]["modify"]["proj"]	= $setTo;
		$this->rights["iteration"]["delete"]["all"]		= $setTo;
		$this->rights["iteration"]["delete"]["proj"]	= $setTo;
		$this->rights["iteration"]["insert"]["all"]		= $setTo;
		$this->rights["iteration"]["insert"]["proj"]	= $setTo;
		
		$this->rights["story"]["modify"]["all"]			= $setTo;
		$this->rights["story"]["modify"]["proj"]		= $setTo;
		$this->rights["story"]["delete"]["all"]			= $setTo;
		$this->rights["story"]["delete"]["proj"]		= $setTo;
		$this->rights["story"]["insert"]["all"]			= $setTo;
		$this->rights["story"]["insert"]["proj"]		= $setTo;
					
		$this->rights["task"]["modify"]["all"]		= $setTo;
		$this->rights["task"]["modify"]["proj"]		= $setTo;
		$this->rights["task"]["modify"]["my"]		= $setTo;
		
		$this->rights["task"]["delete"]["all"]		= $setTo;
		$this->rights["task"]["delete"]["proj"]		= $setTo;
		$this->rights["task"]["delete"]["my"]		= $setTo;
		$this->rights["task"]["insert"]["all"]		= $setTo;
		$this->rights["task"]["insert"]["proj"]		= $setTo;
		$this->rights["task"]["insert"]["my"]		= $setTo;
		
		$this->rights["metaphorWord"]["modify"]["all"]		= $setTo;
		$this->rights["metaphorWord"]["modify"]["proj"]		= $setTo;
		$this->rights["metaphorWord"]["delete"]["all"]		= $setTo;
		$this->rights["metaphorWord"]["delete"]["proj"]		= $setTo;
		$this->rights["metaphorWord"]["insert"]["all"]		= $setTo;
		$this->rights["metaphorWord"]["insert"]["proj"]		= $setTo;
		
		$this->rights["webLink"]["modify"]["all"]		= $setTo;
		$this->rights["webLink"]["modify"]["proj"]		= $setTo;
		$this->rights["webLink"]["delete"]["all"]		= $setTo;
		$this->rights["webLink"]["delete"]["proj"]		= $setTo;
		$this->rights["webLink"]["insert"]["all"]		= $setTo;
		$this->rights["webLink"]["insert"]["proj"]		= $setTo;
		
		$this->rights["webCategory"]["modify"]["all"]		= $setTo;
		$this->rights["webCategory"]["modify"]["proj"]		= $setTo;
		$this->rights["webCategory"]["delete"]["all"]		= $setTo;
		$this->rights["webCategory"]["delete"]["proj"]		= $setTo;
		$this->rights["webCategory"]["insert"]["all"]		= $setTo;
		$this->rights["webCategory"]["insert"]["proj"]		= $setTo;
	}
	
	function isMemberOf($projectId) {
		// Look in affectations array to check if this user is
		// member of projectId.
		return (isSet($this->affectations[$projectId]));
	}
	
	function isMemberOfAnyOf($listOfProjectIds) {
		reset($listOfProjectIds);
		while(list($key,$value) = each($listOfProjectIds)) {
			if ($this->isMemberOf($key)) {
				return true;
			}
		}
		return false;
	}
	
	function isMyself($member) {
		return @($member->id == $this->id);
	}
	
	function isAllowedToDisplaySection($content) {

		$content = str_replace("?print", "", $content);

		$result = @($this->rights["section"][$content]["all"] || ($this->rights["section"][$content]["proj"] && ($this->isMemberOf(getCurrentProjectId()) || $content == "Projects")));
		// If section "Projects", can display even if "proj"...
		return $result;
	}
	
	function updateAffectations() {
		
		// Update affectations
		
		if (!($this->isAdmin || $this->isAnonymous)) {
		
			$selectFields = array(
				"project_id",
				"name"
			);
			$selectTableNames = array(
				"projects_affectation",
				"projects"
			);
			$selectConstraints = array(
				"team_member_id" => $this->id,
				"project_id" => "projects.id"
			);
			$selectGroupBy = "";/*array(
				"project_id"
			);*/
			$selectOrderBy = array(
    			"name asc"
    		);
			
			$result = $GLOBALS["gDb"]->selectEntries($selectTableNames, $selectConstraints, $selectFields, $selectGroupBy, $selectOrderBy);
			
			$isCurrent = $this->isCurrentUser();
			
			while (!$result->EOF) {
				$id = $result->Fields("project_id");
				$name = $result->Fields("name");
				$this->affectations[$id] = $name;
				$result->MoveNext();
			}
		}
	}
	
	function update($newId) {
		$this->updateAffectations();
		
		if ($this->id == $newId) {
			return true;
		} else {
			$this->copy(new User($newId));
			return false;
		}
	}
	
	function getCurrentUser($gui = false) {
		global $gUser;
		
		
		//echoDebug("User:getCurrentUser => start");
		$GLOBALS["gConnectedUser"] = @$_SESSION["sCurrentUser"];
		
		//echoDebug("gConnectedUser from session: ".$GLOBALS["gConnectedUser"]->id);
		
		// get user id from lib_login
		
		$gUser = @lib_login_check_expire(false);
		
		//echoDebug("gUser from lib_login: ".$gUser);
		
		// update object with new id, if necessary
		if (!empty($GLOBALS["gConnectedUser"])
		&& is_subclass_of($GLOBALS["gConnectedUser"], "User")
		) {
			//echoDebug("gConnectedUser not empty, time to update");
			$GLOBALS["gConnectedUser"]->update($gUser);
		} else {
			// object didn't exist: create it from
			// id generated by lib_login
			//echoDebug("gConnectedUser empty, time to create new");
			if ($gui) {
				$GLOBALS["gConnectedUser"] = & new UserView($gUser);
			} else {
				$GLOBALS["gConnectedUser"] = & new User($gUser);
			}
		}
		
		$_SESSION["sCurrentUser"] = $GLOBALS["gConnectedUser"];
		
		//echoDebug("currentUser registered in session: ".$GLOBALS["gConnectedUser"]->id);
		
		//echoDebug("User:getCurrentUser => end, result: ".$GLOBALS["gConnectedUser"]->id);
		
		return $GLOBALS["gConnectedUser"];
	}
	
	function unsetCurrentUser() {
		if (XP_SESSION_PATH != "") {
			session_save_path(XP_SESSION_PATH);
		}
		session_name("login");
		unset($_SESSION["sCurrentUser"]);
	}
	
	function returnUser() {
		
		return $GLOBALS["gConnectedUser"];
	}
}
?>
