<?php
require_once("Model/BaseDatabase_model.inc.php");

class DatabaseFactory {

	// STATIC
	function & GetFactory() {
		global $gDatabaseFactory;		
		return $gDatabaseFactory;
	}

	function & newDatabase($databaseType) {

		$database = & new XpDatabase($databaseType);

		return $database;
	}
		
}

$gDatabaseFactory = & new DatabaseFactory();

class XpDatabase extends BaseXPDatabase {

	function XpDatabase($databaseType) {
		
		parent::BaseXPDatabase($databaseType);

		$this->SQL_QUOTE_CHAR		= "'";
		$this->SQL_ID_QUOTE_CHAR	= '"';
		// Not supported in postgresql
		$this->SQL_IF_NOT_EXISTS	= ""; 
	}
	
	
	function lastInsertedId($tableName, $fieldName = "id") {
		
		$query = "SELECT MAX(".$fieldName.") as maxId FROM ".$tableName;
		
		$result = & $this->executeQuery($query);
		
		if ($result->EOF) {
			return -1;
		} else {
			return $result->Fields(0);
		}
		
	} 
	
	/**
	* tests version of an existing database
	*/
	function getVersion() {
		if (!$this->isConnected()) {
			$this->connectToHost();
		}
		// Empty database
		$tables = $this->connection->MetaTables();
		if (sizeOf($tables) == 0) {
			return 0;
		}
		// 2.2 and over
		for ($i=0; $i < sizeOf($tables); $i++) {
			$name = $tables[$i];
			if ($name == "version_info") {
				$query = "SELECT version_no FROM version_info";
				$result = $this->executeQuery($query);
				return $result->Fields("version_no");
			}
		}
		return 2.2; //First version that supported PostgreSQL
	}
	
	function getSQLAfterField($fieldName) {
		// "after" SQL statement not supported
		return "";
	}

	/**
	* Changes a collumn type into unsigned int
	*/
	function updateColumnType($tableName, $collumnName, $autoIncrement = false) {
		return(false);
		//I don't believe this is actually supported under postgresql - would have to dump an reload
	}
	
	/*
	function updateFromVersion($version){
		$result = true;
		$i = 0;
		
		switch($version){
			case 0:
			case 1:
			case 2:
			case 2.1:
			case 2.2:
			case "2.3":
			case "2.3.1":
			case "2.4":
			case "2.5":
			case "2.5.1":
			case "2.5.2":
			case "3.0":
			case "3.0.1":
			case "3.1":
			case "3.2":
			case "3.2.1":
				$result &= $this->createVersion1Tables();
			case "3.3":
				// nothing to do
			case "3.3.1":
				// nothing to do
		}
		
		return $result;
	}
	*/

	function updateToVersion231() {
		//Really this is the first postgreSQL supported version
		global $XP_DEFAULT_ITERATION_WORKING_PACE;
		global $gProtectedSections;
		global $gRightTypes;
		global $gDeveloperProfileId;
		
		//This is actually the first version of XPWeb to support PostgreSQL
		
		$result = true;
		$queryList = array();
		
		$queryList[] = "
		CREATE TABLE projects (
		  id serial PRIMARY KEY,
		  name varchar(255) NOT NULL default '',
		  description text NOT NULL
		)";
		
		$queryList[] = "
		CREATE TABLE metaphor_words (
		  id serial PRIMARY KEY,
		  name varchar(255) NOT NULL default '',
		  description text NOT NULL,
		  project_id int4 NOT NULL default '0' REFERENCES projects( id )
		)";

		$queryList[] = "
		CREATE TABLE project_link_types (
		  id serial PRIMARY KEY,
		  name varchar(255) NOT NULL default ''
		)";
		
		$queryList[] = "
		CREATE TABLE project_links (
		  id serial PRIMARY KEY,
		  url varchar(255) NOT NULL default '',
		  comment varchar(255) NOT NULL default '',
		  project_id int4 NOT NULL default '0' REFERENCES projects(id),
		  type int4 NOT NULL default '0' REFERENCES project_link_types(id)
		)";
		
		$queryList[] = "
		CREATE TABLE web_categories (
		  id serial PRIMARY KEY,
		  name varchar(255) NOT NULL default '',
		  description text NOT NULL default '',
		  project_id int4 NOT NULL default '0' REFERENCES projects(id)
		)";
		
		$queryList[] = "
		CREATE TABLE web_links (
		  id serial PRIMARY KEY,
		  url varchar(255) NOT NULL default '',
		  name varchar(255) NOT NULL default '',
		  description text NOT NULL,
		  category_id int4 NOT NULL default '0' REFERENCES web_categories(id)
		)";
		
		// Working days for tasks and iterations
		
		$queryList[] = "
		CREATE TABLE working_days (
		setup_name VARCHAR( 11 ) DEFAULT '-1' PRIMARY KEY,
		\"Mon\" INT DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Mon"]."' ,
		\"Tue\" INT DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Tue"]."' ,
		\"Wed\" INT DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Wed"]."' ,
		\"Thu\" INT DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Thu"]."' ,
		\"Fri\" INT DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Fri"]."' ,
		\"Sat\" INT DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Sat"]."' ,
		\"Sun\" INT DEFAULT '".$XP_DEFAULT_ITERATION_WORKING_PACE["Sun"]."' 
		)";
		
		$queryList[] = "
		CREATE TABLE iterations (
		  id serial PRIMARY KEY,
		  project_id int4 NOT NULL default '0' REFERENCES projects(id),
		  working_days_name varchar(11) DEFAULT 'ite_0' NOT NULL REFERENCES working_days(setup_name),
		  load_factor float DEFAULT '".XP_DEFAULT_LOAD_FACTOR."',
		  name varchar(255) NOT NULL default '',
		  description text NOT NULL,
		  date date NOT NULL default '1969-01-01'
		)";
		
		$queryList[] = "
		CREATE TABLE stories (
		  id serial PRIMARY KEY,
		  iteration_id int4 NOT NULL default '0',
		  project_id int4 NOT NULL default '0' REFERENCES projects(id),
		  name varchar(255) NOT NULL default '',
		  description text NOT NULL,
		  validation_criteria text NOT NULL,
		  priority int4 NOT NULL default '0',
		  risk int4 NOT NULL default '0',
		  weight int4 NOT NULL default '0'
		)";
			
		// Profiles
		
		$queryList[] = "
		CREATE TABLE user_profiles (
		  id serial PRIMARY KEY,
		  name varchar(255) NOT NULL default '',
		  description text NOT NULL
		)";
		
		$queryList[] = "
		CREATE TABLE team_members (
		  id serial PRIMARY KEY,
		  profile_id varchar(11) DEFAULT '".$gDeveloperProfileId."' NOT NULL  REFERENCES user_profiles( id ),
		  name varchar(255) NOT NULL default '',
		  description text NOT NULL
		)";
		
		$queryList[] = "
		CREATE TABLE tasks (
		  id serial PRIMARY KEY,
		  story_id int4 NOT NULL default '0',
		  iteration_id int4 NOT NULL default '0',
		  project_id int4 NOT NULL default '0' REFERENCES projects(id),
		  responsible_team_member_id int4 NOT NULL default '0' REFERENCES team_members(id),
		  working_days_name varchar(11) default 'ite_0' NOT NULL REFERENCES working_days(setup_name),
		  name varchar(255) NOT NULL default '',
		  description text NOT NULL,
		  start_date date NOT NULL default '1969-01-01',
		  weight float NOT NULL default '".XP_IMPOSSIBLE_ID."',
		  done float NOT NULL default '0',
		  todo float NOT NULL default '0',
		  end_date date default '".XP_FORBIDDEN_DATE."' NOT NULL,
		   theory_end_comp FLOAT DEFAULT '".XP_IMPOSSIBLE_ID."' NOT NULL
		)";

		
		$queryList[] ="
		CREATE TABLE version_info (
			version_no VARCHAR(10) DEFAULT '0' NULL PRIMARY KEY
		)";
		
		$queryList[] = "INSERT INTO version_info VALUES(2.2)";

		
		/**
		* Insert entries for link types
		*/
		$queryList[] = "INSERT INTO project_link_types VALUES (1, 'modeling')";
		
		$queryList[] = "INSERT INTO project_link_types VALUES (2, 'test')";
		
		$queryList[] = "INSERT INTO project_link_types VALUES (3, 'doc')";
		
		$queryList[] = "INSERT INTO project_link_types VALUES (4, 'cvs')";
		
		$queryList[] = "INSERT INTO project_link_types VALUES (5, 'directory')";
		
		$queryList[] = "INSERT INTO project_link_types VALUES (6, 'javadoc')";

		$queryList[] = "select setval('project_link_types_id_seq',7)";
	
		// php_lib_login tables creation
		
		$queryList[] = "
		CREATE TABLE tbl_users(
		username varchar(40) PRIMARY KEY,
		password varchar(60) NOT NULL,
		email varchar(40) NOT NULL,
		question varchar(200),
		answer varchar(30),
		lastlogin integer
		)";
			
		
		$queryList[] = "
		CREATE TABLE tbl_log(
		timestamp integer NOT NULL,
		ip varchar(16) NOT NULL DEFAULT 'no ip logged' ,
		username varchar(40),
		action varchar(40),
		UNIQUE(timestamp, ip) 
		)";
			
		
		$queryList[] = "
		CREATE TABLE tbl_banned(
		ip varchar(16) PRIMARY KEY
		)";
			
		
		$queryList[] = "
		CREATE TABLE tbl_tries(
		username varchar(40) PRIMARY KEY,
		tries int,
		nextlogin int
		)";
			
		
		$queryList[] = "
		CREATE TABLE tbl_group(
		username varchar(40) PRIMARY KEY,
		gid int
		)";
			
		
		$queryList[] = "
		insert into tbl_group select username, 0  from tbl_users";
			
		// New in XPWeb
		
		// ------------------------------------------------------------------
		// Regarding user rights...
			
		
		$queryList[] = "
		CREATE TABLE projects_affectation (
		  project_id int4 NOT NULL default '0' REFERENCES projects(id),
		  team_member_id int4 NOT NULL default '0' REFERENCES team_members(id),
		  UNIQUE (project_id,team_member_id)
		)";
			
		// Sections that need to be protected
		
		$queryList[] = "
		CREATE TABLE protected_sections (
		  id serial PRIMARY KEY,
		  name1 varchar(100) NOT NULL default '',
		  name2 varchar(100) NOT NULL default ''
		)";
			
		// Existing right types (ex "all", "proj", etc..)
		$queryList[] = "
		CREATE TABLE right_types (
		  id serial PRIMARY KEY,
		  type_name varchar(100) NOT NULL default ''
		)";
			
		// Rights that can be chosen for a specific section
		$queryList[] = "
		CREATE TABLE available_section_rights (
		  protected_section_id int4 NOT NULL default '0' REFERENCES protected_sections(id),
		  right_type_id int4 NOT NULL default '0' REFERENCES right_types(id),
		  UNIQUE(protected_section_id, right_type_id)
		)";
			
		// Rights that have been chosen for a profile and a protected section
		$queryList[] = "
		CREATE TABLE profiles_rights (
		  profile_id int4 NOT NULL default '0' REFERENCES user_profiles( id ),
		  protected_section_id int4 NOT NULL default '0' REFERENCES protected_sections( id ),
		  right_type_id int4 NOT NULL default '0' REFERENCES right_types( id ),
		  UNIQUE(profile_id, protected_section_id)
		)";
			
			
		// Insert default working days
		$queryList[] = "
		INSERT INTO working_days (setup_name , \"Mon\" , \"Tue\" , \"Wed\" ,\"Thu\" ,\"Fri\" ,\"Sat\" ,\"Sun\" )
		VALUES (
		'ite_' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Mon"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Tue"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Wed"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Thu"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Fri"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Sat"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Sun"]."' 
		)";
			$queryList[] = "
		INSERT INTO working_days (setup_name ,\"Mon\" ,\"Tue\" ,\"Wed\" ,\"Thu\" ,\"Fri\" ,\"Sat\" ,\"Sun\" )
		VALUES (
		'ite_0' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Mon"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Tue"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Wed"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Thu"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Fri"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Sat"]."' ,'".$XP_DEFAULT_ITERATION_WORKING_PACE["Sun"]."' 
		)";
			
		/**
		* PostgreSQL does not support adding a column after another specific column
		* Lets hope I can talk them out of keeping this trend up :(
		* Several steps have to be taken to handle that type of function
		* 1. all the data must be pulled out of the current table into a temp table
		* 2. the old table and sequence must be dropped
		* 3. the new table is created
		* 4. The old data is moved back
		* 5. all foreign key constraints have to be added back on
		* 6. the primary key sequence needs to be reset
		* 7. the temp table can be dropped
		*/
		
		for ($i = 0; $i < count($queryList); $i++) {
			$result &= $this->tryToExecuteQuery($queryList[$i]);
		}
		
		// Time to insert available rights
		$this->insertAllAvailableRights();
		
		// Time to add protected sections and the corresponding available rights
		$this->insertProtectedSections();
		
		// Time to add default profiles such as anonymous and admin
		$this->insertDefaultProfiles();
	
	
		//		
		// Reports:  New feature addition.  Needs all the protected section stuff.
		//
				
		// Add protected section entry
		/*
		$this->addProtectedSection( "section", "Reports", array( "no", "all", "proj" ) );
		
		// Get the id for the all right type
		$resultRightTypes = & $this->selectOneEntry( array("right_types"), array("type_name" => "all"), array("*") );
		$intRightTypesAllID = $resultRightTypes->Fields("id"); 
		
		// Update the profile rights for the default profiles															
		$resultProfiles = $this->selectEntries( array("user_profiles"), array(), array("*") );
		while ( !$resultProfiles->EOF ) {			
			$fields = array(
				"profile_id" => $resultProfiles->Fields("id"),
				"protected_section_id" => $gProtectedSections["section"]["Reports"],
				"right_type_id" => $intRightTypesAllID				
			);				

			$result &= $this->insertIntoTable("profiles_rights", $fields );
										
			$resultProfiles->MoveNext();
		}
		*/
		
		// Add protected section entry
		//$result &= $this->updateAddProtectedSection("Reports");
	
		//Add a nobody account to make the referential intergrity work under postgresql
		$query = "INSERT INTO team_members (id,profile_id,name,description) VALUES (0,1,'Nobody','This is a special user. Do not delete');";
		$result &= $this->tryToExecuteQuery($query);
		
		return $result;
	}
	
	function updateToVersion301() {
		// No "add constraint" in postgres
		return true;
	}
	
	function updateToVersion251() {
		// useless, no tinyint
		return true;
	}
	
	function updateToVersion31() {
		// useless, was ok
		return true;
	}
}
?>
