<?php

class PlanningFactory extends ModelFactory {

	var $caller;

	// STATIC
	function & GetFactory() {

		if (isset($this)) {
			$GLOBALS["gPlanningFactory"]->caller = & $this;
		}
		
		//echoDebug("Planning factory called from ".@get_class($this));
				
		return $GLOBALS["gPlanningFactory"];
	}

	function & newPlanning($arguments) {

		$object = & new Planning($arguments);
		
		new PlanningControler($object);

		return $object;

	}
	
	function & newTask($arguments) {

		$object = & new Task($arguments);
		
		new TaskControler($object);

		return $object;
	}
		
	function & newStory($arguments) {
		
		$object = & new Story($arguments);
		
		new StoryControler($object);

		return $object;
	}
		
	function & newIteration($arguments) {

		$object = & new Iteration($arguments);
		
		new IterationControler($object);

		return $object;
	}

	function & newBasicIteration($arguments) {

		$object = & new BasicIteration($arguments);
		
		new IterationControler($object);

		return $object;
	}
	
}

$GLOBALS["gPlanningFactory"] = & new PlanningFactory();

// *********************************************************************
// Class BasicPlanningMember
// *********************************************************************
class BasicPlanningMember extends XPItem {
	var $title;

	function BasicPlanningMember($arguments) {

		while (list($key, $val) = each($arguments)) {
			${$key} = $val;
		}
		
		// Project id has no meaning here => 0
		$this->XPItem(array("basicPlanningMember", $id, 0));

		$this->title = $title;
	}
}

// *********************************************************************
// Class PlanningCard
// *********************************************************************
class PlanningCard extends XPItem {
	
	var $loadFactor;
	var $title;
	var $description;

	var $completionReady;
	
	var $workingDays;
	var $workingDaysName;

	function PlanningCard($arguments) {
		
		while (list($key, $val) = each($arguments)) {
			${$key} = $val;
		}
		
		
		$this->XPItem(array($type, $id, $projectId));
		
		$this->loadFactor		= ($loadFactor>0)?$loadFactor:XP_DEFAULT_LOAD_FACTOR;
		$this->title			= $title;
		$this->description		= $description;
		
		$this->workingDays		= $workingDays;
		$this->workingDaysNam	= $workingDaysName;
		
		$this->completionReady = false;
		
	}
	
	function isCompletionReady() {
		return $this->completionReady;
	}
	
	function calculateCompletion() {
		// To be overriden by child classes
	}
	
	// _________ Private functions
	
	function validateWorkingDay($time) {
		$time += PlanningCard::howMuchToWorkingDay($time);
		
		$day = date("D",$time);
		
		$date = date("Y-m-d",$time); 
		
		return array(
			"time" => $time,
			"date" => $date,
			"day" => $day
		);
	}
	
	function howMuchToWorkingDay($theTime) {
		global $XP_EVERY_YEAR_HOLIDAYS;
		global $XP_SPECIFIC_HOLIDAYS;
		
		$oldDay = date("D",$theTime);
		$day = $oldDay;
	
		$date = date("Y-m-d",$theTime);
		
		$dateNoYear = date("m-d",$theTime);
		
		$ready = false;
		$startTime = $theTime;
		while (!$ready) {
			$ready = true; 
			
			$nbTries = 0; 
			$maxTries = 7; 
			
			//echoDebug("working day: $day => ".$this->workingDays[$day]." / ".($this->workingDays[$day] == true));
			
			while (!($this->workingDays[$day])) { 
				// While the date is not working day 
				// Slide from one day
				$theTime += XP_SECONDS_IN_DAY;
				$day = date("D",$theTime); 
				$date = date("Y-m-d",$theTime); 
				$dateNoYear = date("m-d",$theTime); 
				$ready = false; 
				$nbTries ++; 
				if ($nbTries == $maxTries) { 
					return 0;
				}
			} 
			
			$nbTries = 0; 
			$maxTries = 365; 
			while (isSet($XP_EVERY_YEAR_HOLIDAYS[$dateNoYear]) || isSet($XP_SPECIFIC_HOLIDAYS[$date])) { 
				// While the date is holiday 
				// Slide from one day 
				$theTime += XP_SECONDS_IN_DAY; 
				$day = date("D",$theTime); 
				$date = date("Y-m-d",$theTime); 
				$dateNoYear = date("m-d",$theTime); 
				$ready = false; 
				$nbTries ++; 
				if ($nbTries == $maxTries) { 
					return 0;
				}
			} 
		}
		$timeToAdd = $theTime - $startTime;
		
		return $timeToAdd;
	}
	
	function diffTimes($timestamp1,$timestamp2) {
		$seconds = $timestamp2 - $timestamp1;
		return $seconds;
	}
}

// *********************************************************************
// Class Task
// *********************************************************************
class Task extends XPItem {
	var $actualCompletion;
	var $theoryEndCompletion;
	var $theoryCompletion;
	var $storyId;
	var $iterationId;
	
	var $startDate;
	var $startDateYear;
	var $startDateMonth;
	var $startDateDay;
	var $startWeekDay;
	var $startTime;
	
	var $wrongStartDate;
	var $wrongStartDateYear;
	var $wrongStartDateMonth;
	var $wrongStartDateDay;
	var $wrongStartWeekDay;
	var $wrongStartTime;
	
	var $outOfTime;
	
	// End date from DB
	//var $realEndDate;
	
	var $endDate;
	var $endDateYear;
	var $endDateMonth;
	var $endDateDay;
	var $endWeekDay;
	var $endTime;
	
	var $weight;
	var $weightFromDb;
	var $done;
	var $todo;
	
	var $memberId;
	var $pairId;
	var $loadFactor;
	
	var $iterationDeadlineTime;
	var $autoUpdate;
	
	var $isDefaultWorkingDays;

	function isCompletionReady() {
		return PlanningCard::isCompletionReady();
	}
	
	function Task($arguments) {
		
		$i = 0;
		
		while (list($key, $val) = each($arguments)) {
			${$key} = $val;
		}


		$arguments["type"] = "task";
		
		PlanningCard::PlanningCard($arguments);
				
		$this->workingDays = $workingDays;
		$this->workingDaysName = $workingDaysName;
		
		if ($this->workingDaysName == "task_".$this->id) {
			$this->isDefaultWorkingDays = false;
		} else {
			$this->isDefaultWorkingDays = true;
		}
		
		$this->storyId = $storyId;
		$this->iterationId = $iterationId;
		
		//echoDebug("Task $id: startdate= $startDate");
		
		$this->setStartDate($startDate);
		
		// Validate start day : is it a working day?
		$this->validateStartDate();
		
		//echoDebug("old start date: ".$this->wrongStartDate);
		//echoDebug("new start date: ".$this->startDate);

		// End date from DB
		//echoDebug($this->id." constructor: set realEndDate and end date ".$endDate);
		//$this->realEndDate = $endDate;
		$this->setEndDate($endDate);
		
		$this->todo = $todo;
		$this->done = $done;
				
		$this->outOfTime = 0;
		
		$this->theoryEndCompletion = $theoryEndCompletion;
		
		//$this->calculateActualCompletion();
		
		if (empty($memberId)) {
			$memberId = 0;
		}

		if (empty($pairId)) {
			$pairId = 0;
		}
		
		$this->memberId = $memberId;

		$this->pairId = $pairId;

		$this->iterationDeadlineTime = $iterationDeadlineTime;
		$this->autoUpdate = $autoUpdate;
		
		$this->setWeightFromDb($weight);
		
		if ($this->autoUpdate) {
			if ($this->weight == 0) {
				$this->setEndDate($this->startDate);
			} else {
				$this->validateEndDate();
			}
		}
		
		$this->setEndDateReady(false);
	}

	function & getObjectToInsert() {
		global $XP_DEFAULT_ITERATION_WORKING_PACE;
		global $XP_DEFAULT_ITERATION_WORKING_NAME;
		

		// Params
		$parameters				= func_get_arg(0);

		$i = 0;
		$itemType				= $parameters[$i++];
		
		$iterationId			= $parameters[$i++];
		$storyId				= $parameters[$i++];
		$nTasksResponsibleId	= $parameters[$i++];
		$nTasksPairId			= $parameters[$i++];
		
		$factory = & PlanningFactory::GetFactory();

		$newTask = & $factory->newTask(
			array(
				"id" 					=> "insert_0_0",
				"projectId" 			=> getCurrentProjectId(),
				"loadFactor" 			=> XP_DEFAULT_LOAD_FACTOR,
				"title" 				=> getLang("new_task"),
				"description" 			=> "",
				"workingDays" 			=> $XP_DEFAULT_ITERATION_WORKING_PACE,
				"workingDaysName" 		=> $XP_DEFAULT_ITERATION_WORKING_NAME,
				"storyId" 				=> 0,
				"iterationId" 			=> 0,
				"startDate" 			=> date("Y-m-d"),
				"endDate" 				=> date("Y-m-d"),
				"theoryEndCompletion" 	=> -1,
				"weight" 				=> 0,
				"done" 					=> 0,
				"todo" 					=> 0,
				"memberId" 				=> 0,
				//"memberName" 			=> "",
				"pairId" 				=> 0,
				//"pairName" 			=> "",
				"iterationDeadlineTime" => getDefaultIterationDeadline(),
				"autoUpdate"			=> true
			)
		);
						

		// By default, task's user is current user.
		$newTask->memberId = $GLOBALS["gConnectedUser"]->id;
 		//$newTask->memberName = $GLOBALS["gConnectedUser"]->name;

		// If is allowed to choose responsible member:
		// 	** If showing somebody's tasks and this user is member => this user by default
		if ($newTask->isAllowedToInsertAll()) {

			if ($nTasksResponsibleId != ALL_TASKS) {
				// Showing somebody's tasks
				$newTask->memberId		= $nTasksResponsibleId;
			}
		}

		if ($nTasksPairId != ALL_TASKS) {
			// Showing somebody's pair tasks
			$newTask->pairId		= $nTasksPairId;
		}
		
		return $newTask;
	}

	function setWeightFromDb($weightFromDb) {
		$this->weightFromDb = $weightFromDb;
		if (!$this->isStarted()) {
			// $weight = - 1 => never updated to db.
			// The weight we display is calculated from current
			// done and todo
			$this->setCalculatedWeight();
		} else {
			// We use the weight from db
			$this->weight = $this->weightFromDb;
		}
	}
	
	function validateStartDate() {
		// Validate start day : is it a working day?
		$validDate = PlanningCard::validateWorkingDay($this->startTime);
		$date = $validDate["date"];
		if ($this->startDate != $date) {

			// Wasn't a working day...
			$this->setWrongStartDate($this->startDate);
			$this->setStartDate($date);
			//echoDebug("task ".$this->id." wrong start date ".
			//	$this->wrongStartDate.
			//	" => ".
			//	$this->startDate);
		}
	}
	
	function validateEndDate() {
		// Validate start day : is it a working day?
		if ($this->endTime == -1) {
			$this->setEndDate($this->startDate);
		} else {
			$validDate = PlanningCard::validateWorkingDay($this->endTime);
    		$date = $validDate["date"];
    		$this->setEndDate($date);
		}
	}
		
	function setCalculatedWeight() {
		$this->weight = $this->getCalculatedWeight();
		//echoDebug("set calculated weight = ".$this->done." / ".$this->loadFactor." + ".$this->todo
		//		." = ".$this->weight);
	}
	
	function getCalculatedWeight() {
		if ($this->loadFactor == 0) {
			return 0;
		} else {
			return ($this->done / $this->loadFactor) + $this->todo;
		}
	}
	
	function setStartDate($startDate) {
		$this->startDate = $startDate;
		
		list($this->startDateYear,$this->startDateMonth,$this->startDateDay) = explode("-",$startDate);
		$this->startTime = @mktime(0,0,0,$this->startDateMonth,$this->startDateDay,$this->startDateYear);
		$this->startWeekDay = date("D",$this->startTime);
	}
	
	function setWrongStartDate($wrongStartDate) {
		
		$this->wrongStartDate = $wrongStartDate;
	
		list($this->wrongStartDateYear,$this->wrongStartDateMonth,$this->wrongStartDateDay) = explode("-",$wrongStartDate);
		$this->wrongStartTime = @mktime(0,0,0,$this->wrongStartDateMonth,$this->wrongStartDateDay,$this->wrongStartDateYear);
		$this->wrongStartWeekDay = date("D",$this->wrongStartTime);
	}
	
	function setEndDate($endDate) {
		
		//echoDebug($this->id." set enddate: ".$endDate);
		
		$this->endDate = $endDate;
		list($this->endDateYear,$this->endDateMonth,$this->endDateDay) = explode("-",$endDate);
		$this->endTime = @mktime(0,0,0,$this->endDateMonth,$this->endDateDay,$this->endDateYear);
		if ($endDate != XP_FORBIDDEN_DATE) {
			$this->endWeekDay = @date("D",$this->endTime);
		}
	}
	
	function isEndDateReady() {
		return $this->endDateReady;
	}
	
	function setEndDateReady($readyOrNot) {
		$this->endDateReady = $readyOrNot;
	}
	
	function isFromMember($memberId) {
		return ($memberId == ALL_TASKS || $memberId == $this->memberId);
	}

	function isFromPair($pairId) {
		return ($pairId == ALL_TASKS || $pairId == $this->pairId);
	}
	
	/**
	* Completion stuff
	*/
	
	function isStarted() {
		//echoDebug("task ".$this->id." is started? ".($this->done != 0));
		return ($this->done != 0);
	}
	
	function isCompleted() {
		$lbIsCompleted = ($this->actualCompletion >= 100);
		echoDebug("task ".$this->id." isCompleted? ".$this->actualCompletion.": ".
			$lbIsCompleted);
		return $lbIsCompleted;
	}
	
	function shouldBeCompleted() {
		return ($this->theoryCompletion >= 100);
	}
	
	function calculateCompletion() {
		$this->completionReady = false;
		$this->calculateActualCompletion();

		$this->outOfTime = XP_IMPOSSIBLE_ID;

		//echoDebug($this->id." actual completion : ".$this->actualCompletion);
		if ($this->isCompleted()) {
			// If the task is finished, theory completion is 
			// the theory end completion from database...
			$this->theoryCompletion = $this->theoryEndCompletion;
			//echoDebug($this->id." task is completed: theory completion and end date read FROM DB (not calculated)");
			//echoDebug($this->id." DB theory end completion: ".$this->theoryEndCompletion);
			//echoDebug($this->id." DB theory end date: ".$this->realEndDate);
			// get end date from DB (the actual date at which the task was finished)
			//$this->setEndDate($this->realEndDate);
		} else {
			$this->calculateTheoryCompletion();
		}


		if (!empty($this->iterationDeadlineTime)) {
			$diffTime = PlanningCard::diffTimes($this->iterationDeadlineTime, $this->endTime);
			if ($diffTime > 0) {
				$diffIterationTask = @round($diffTime / XP_SECONDS_IN_DAY);//date("d",$diffTime);
				$this->outOfTime = $diffIterationTask;
			}
		}

		$this->completionReady = true;
	}
	
	function calculateActualCompletion() {
		if (($this->isStarted() && $this->done == 0) || $this->weight == 0 || $this->loadFactor == 0) {
			$this->actualCompletion = 0;
		} else {
			$this->actualCompletion = @round($this->done * 100 / ($this->done + ($this->todo*$this->loadFactor)));
		}
	}
	
	function calculateTheoryCompletion() {
		global $gNow;
		global $gSecondsWorkedInADay;
		
		//echoDebug($this->id." = calculateTheoryCompletion ");
		
		$this->theoryCompletion = 0;
		
   		$time = $this->startTime;
   		   		
   		// Based on the initial weight to calculate theory completion.
		$daysTotalToWork = $this->weight * $this->loadFactor;
 		$secondsTotalToWork = $daysTotalToWork * $gSecondsWorkedInADay;

		$secondsLeftToWork = $secondsTotalToWork;

 		echoDebug($this->id."{calcTheory} $secondsTotalToWork to work");

		echoDebug($this->id."{calcTheory} start time ".date("Y-m-d H:i", $time)
			." / now ".date("Y-m-d H:i", $gNow));
   		
		if ($this->weight != 0 && PlanningCard::diffTimes($time, $gNow) >= 0) {
			// If there is time to spend, and today is AFTER start time
   			
			$secondsWorked = 0;
			
			// For all full days FROM start time TO now
			while (PlanningCard::diffTimes($time, $gNow) >= XP_SECONDS_IN_DAY){

				echoDebug($this->id."{calcTheory} PlanningCard::diffTimes(time, gNow) = "
					.PlanningCard::diffTimes($time, $gNow)." => goFuture");

				// Warning all params are modified
				$this->_goFuture($time, $secondsLeftToWork, $secondsWorked);

			}

			echoDebug($this->id."{calcTheory} $secondsWorked seconds worked");

			// Now we've handled all FULL days...
			// Time to check if today is in the middle of a day
					
			if (PlanningCard::howMuchToWorkingDay($gNow) == 0) {
				// Today is a working day

				// How much work time was spent from this morning to now?

				list($startHour, $startMinute) = explode(":", XP_START_HOUR);
				$timeTodayStartWork = mktime($startHour, $startMinute, 0, date("n", $gNow),date("j", $gNow),date("Y", $gNow));
				
				//echoDebug($this->id." = calculateTheoryCompletion: thismorning = "
				//	.date("d/m/Y H", $timeTodayStartWork));

				$secondsSinceStartWork = PlanningCard::diffTimes($timeTodayStartWork, $gNow);

				if ($secondsSinceStartWork > 0) {
					// We are AFTER start of working day
					
					$secondsWorkedToday = min($secondsSinceStartWork, $gSecondsWorkedInADay);

					echoDebug($this->id."{calcTheory} $secondsSinceStartWork seconds have been worked today");
    			
					// We add what has been worked today
    				$secondsWorked += $secondsWorkedToday;


					if ($secondsTotalToWork < $gSecondsWorkedInADay
						&& PlanningCard::diffTimes($this->startTime, $gNow) <= $gSecondsWorkedInADay) {
						// The task will last less than a day
						// and this is the last day

						// If a task weights 0.25, theory end comp
	            		// will only start at 75% of the day
						// (you don't HAVE to finish the task before 25% of the day...)
					
						// The decimal part
	            		$secondsPartDayToWork = ($daysTotalToWork - floor($daysTotalToWork));
                		$toRemove = (1 - $secondsPartDayToWork) * $gSecondsWorkedInADay;
    					$secondsWorked -= $toRemove;
					}
	    			
				}
			}
 		
			$this->theoryCompletion = @round($secondsWorked*100 / $secondsTotalToWork);
			
			echoDebug($this->id."{calcTheory} theory calculated = "
				."round($secondsWorked*100 / $secondsTotalToWork) = "
				.$this->theoryCompletion);
	
			// We want positive values!
			$this->theoryCompletion = max(0, $this->theoryCompletion);
		}			
	}
	
	function calculateEndDate() {
		global $gNow;
		global $gSecondsWorkedInADay;
		
		/**
		 * The end date calculation is:
			*	* 	 As long as the task is not finished (todo not = 0): 
			* 			max(start date, now) + todo * load factor (skipping non-working days)
		 	*	*	 Once the task is finished (todo = 0): 
			* 			start date + done (skipping non-working days)
		 */
		
		$this->setEndDateReady(false);
		
		if ($this->todo != 0) {
			// max(start date, now) + todo * load factor (skipping non-working days)
			$startTime = max($this->startTime, $gNow);
			$secondsToWork = $this->todo * $this->loadFactor * $gSecondsWorkedInADay;
			
		} else {
			// start date + done (skipping non-working days)
			$startTime = $this->startTime;
			$secondsToWork = $this->done;
		}

		$time = $startTime;

		echoDebug($this->id."{calcEndDate} start at ".date("Y-m-d H:i", $time)
			." / add $secondsToWork seconds");
		
		$this->setEndDate(date("Y-m-d", $time));
		
		$secondsWorked = 0;
		
		//echoDebug($this->id." = calculateEndDate: go future for end date (first time)");
		$this->_goFuture($time, $secondsToWork, $secondsWorked);

        while ($secondsToWork > 0) {
			//echoDebug("Still $secondsToWork Todo to work for end date.  => goFuture");
			// Note: we want the end date to be updated if necessary
			// (last argument = true)
			//echoDebug($this->id." = calculateTheoryCompletion: go future for end date");
			$this->_goFuture($time, $secondsToWork, $secondsWorked);
		}

		if ($secondsToWork <= 0) {
	 		// We have added all necessary weight to calculate
			// the theory end date (based on initial weight)
			// We decrease one day because first day counts
			// we don't do this for COMPLETION but for DATES we do
			$endTime = $time-XP_SECONDS_IN_DAY;
			// We check if workind day again!!...
			$toWorkingDay = PlanningCard::howMuchToWorkingDay($endTime);
			if ($toWorkingDay != 0) {
		  		// Was not a working day
		  		$endTime += $toWorkingDay;
		  	}
		  	$endDate = date("Y-m-d",$endTime);
			//echoDebug($this->id." = goFuture "." SET END DATE: $endDate");
			$this->setEndDate($endDate);
			$this->setEndDateReady(true);
	 	}
	}
	
	/**
	 * Moves $time from the amount of $secondsToWork, and increases $secondsWorked
	 * everytime a working day is spent.
	 * If $doUpdateEndDate is true, the end date is set to $time as soon as
	 * all $secondsToWork is spent
	 */
	function _goFuture(& $time, & $secondsToWork, & $secondsWorked) {
		global $gSecondsWorkedInADay;

		$toWorkingDay = PlanningCard::howMuchToWorkingDay($time);
		//echoDebug($this->id." = goFuture ".date("Y-m-d H:i",$time)." to working day: ".$toWorkingDay);
		//echoDebug($this->id." = goFuture weightToAdd = ".($secondsToWork / $gSecondsWorkedInADay));

		if ($toWorkingDay != 0) {
	 		// Was not a working day

			echoDebug($this->id."{goFuture} ".date("Y-m-d H:i",$time)
				." not working day => moving to ".date("Y-m-d H:i",$time + $toWorkingDay));
            
			// Move date but don't touch time spent at work
	 		$time += $toWorkingDay;
			
	 	} else {
	 		// Was a working day

			// real time spent at work
	 		$secondsWorked += $gSecondsWorkedInADay;

			echoDebug($this->id."{goFuture} seconds worked += 1 working day: $secondsWorked");

	 		// Move date
			$time += XP_SECONDS_IN_DAY;

			echoDebug($this->id."{goFuture} time += 1 day: ".date("Y-m-d H:i",$time));

	 		$secondsToWork -= $gSecondsWorkedInADay;
	 		//echoDebug($this->id." weightToAdd -= $gSecondsWorkedInADay => $secondsToWork");
	 	}

	 	//echoDebug($this->id." = goFuture "." weightToAdd: $secondsToWork");
	}
		
	/**
	* Functions giving rights to modify the team member
	*/

	function isAllowedToDelete() {
		

		//echoDebug($this->id."{isAllowedToDelete} ".$this->memberId." / ".$GLOBALS["gConnectedUser"]->id);

		// If is allowed to delete parent stories and iterations,
		// means is allowed to delete tasks...
		return @($GLOBALS["gConnectedUser"]->rights["task"]["delete"]["all"]
			|| ($GLOBALS["gConnectedUser"]->rights["task"]["delete"]["my"] && $this->isFromMember($GLOBALS["gConnectedUser"]->id))
			|| ($GLOBALS["gConnectedUser"]->rights["task"]["delete"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOf($this->projectId)));
	}
	
	function isAllowedToUpdate() {
		
		return @($this->isAllowedToUpdateAll() || ($GLOBALS["gConnectedUser"]->rights["task"]["modify"]["my"] && $this->isFromMember($GLOBALS["gConnectedUser"]->id)));
	}
	
	function isAllowedToUpdateAll() {
		
		return @($GLOBALS["gConnectedUser"]->rights["task"]["modify"]["all"] || ($GLOBALS["gConnectedUser"]->rights["task"]["modify"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOf($this->projectId)));
	}
	
	function isAllowedToInsert() {
		
		// allowed to insert for any project, or...
		// ...to insert with his name AND on this project
		return @($this->isAllowedToInsertAll()
			|| ($GLOBALS["gConnectedUser"]->isMemberOf($this->projectId) && $GLOBALS["gConnectedUser"]->rights["task"]["insert"]["my"] && $this->isFromMember($GLOBALS["gConnectedUser"]->id)));

	}
	
	function isAllowedToInsertAll() {
		
		return @($GLOBALS["gConnectedUser"]->rights["task"]["insert"]["all"] || ($GLOBALS["gConnectedUser"]->rights["task"]["insert"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOf($this->projectId)));
	}
	
	function isAllowedToInsertIfResponsible($memberId) {
		if ($this->isAllowedToInsert()) {
			return true;
		} else {
			// not allowed but maybe if he's the responsible?
			$this->memberId = $memberId;
			//$this->memberName = $memberName;
			return $this->isAllowedToInsert();
		}
	}
	
	/**
	 * Change the project id
	 * If the current responsible member
	 * is not member of the new project,
	 * the task is set to "not affected"
	 */
	function setNewProject($projectId) {
		// Set the new project
		echoDebug("setNewProject $projectId");
		$this->projectId = $projectId;
		// Let's verify if the responsible member is
		// member of the new project
		//echoDebug("setNewProject: ".$projectId);

		if ($this->memberId != 0) {
			//echoDebug("member Id: ".$this->memberId);	
			$currentUser = & new User($this->memberId);
			if (!$currentUser->isMemberOf($this->projectId)) {
	 			// He is not!
	 			// => set to "nobody"
				//echoDebug($this->memberId." not member of ".$this->projectId);	
	 			$this->memberId = 0;
	 		}
		}

		// Same thing for pair programmer
		if ($this->pairId != 0) {
			$currentUser = & new User($this->pairId);
			if (!$currentUser->isMemberOf($this->projectId)) {
	 			// He is not!
	 			// => set to "nobody"
	 			$this->pairId = 0;
	 		}
		}
	}
	
	function equals(& $compareTask) {
		if ($this->id != $compareTask->id) {
			//echoDebug("task different: id (".$this->id." != ".$compareTask->id.")");
			return false;
		}
		if ($this->projectId != $compareTask->projectId) {
			//echoDebug("task different: projectId (".$this->projectId." != ".$compareTask->projectId.")");
			return false;
		}
		if ($this->loadFactor != $compareTask->loadFactor) {
			//echoDebug("task different: loadFactor (".$this->loadFactor." != ".$compareTask->loadFactor.")");
			return false;
		}
		if ($this->title != $compareTask->title) {
			//echoDebug("task different: title(".$this->title." != ".$compareTask->title.")");
			return false;
		}
		if ($this->description != $compareTask->description) {
			//echoDebug("task different: description (".$this->description." != ".$compareTask->description.")");
			return false;
		}
		if ($this->workingDays != $compareTask->workingDays) {
			//echoDebug("task different: workingDays (".$this->workingDays." != ".$compareTask->workingDays.")");
			return false;
		}
		if ($this->workingDaysName != $compareTask->workingDaysName) {
			//echoDebug("task different: workingDaysName (".$this->workingDaysName." != ".$compareTask->workingDaysName.")");
			return false;
		}
		if ($this->storyId != $compareTask->storyId) {
			//echoDebug("task different: storyId (".$this->storyId." != ".$compareTask->storyId.")");
			return false;
		}
		if ($this->iterationId != $compareTask->iterationId) {
			//echoDebug("task different: iterationId (".$this->iterationId." != ".$compareTask->iterationId.")");
			return false;
		}
		if ($this->startDate != $compareTask->startDate) {
			//echoDebug("task different: startDate (".$this->startDate." != ".$compareTask->startDate.")");
			return false;
		}
		if ($this->endDate != $compareTask->endDate) {
			//echoDebug("task different: endDate (".$this->endDate." != ".$compareTask->endDate.")");
			return false;
		}
		if ($this->theoryEndCompletion != $compareTask->theoryEndCompletion) {
			//echoDebug("task different: theoryEndCompletion (".$this->theoryEndCompletion." != ".$compareTask->theoryEndCompletion.")");
			return false;
		}
		if ($this->weight != $compareTask->weight) {
			//echoDebug("task different: weight (".$this->weight." != ".$compareTask->weight.")");
			return false;
		}
		if ($this->done != $compareTask->done) {
			//echoDebug("task different: done (".$this->done." != ".$compareTask->done.")");
			return false;
		}
		if ($this->todo != $compareTask->todo) {
			//echoDebug("task different: todo (".$this->todo." != ".$compareTask->todo.")");
			return false;
		}
		if ($this->memberId != $compareTask->memberId) {
			//echoDebug("task different: memberId (".$this->memberId." != ".$compareTask->memberId.")");
			return false;
		}
		if ($this->pairId != $compareTask->pairId) {
			//echoDebug("task different: pairId (".$this->pairId." != ".$compareTask->pairId.")");
			return false;
		}

		/*if ($this->memberName != $compareTask->memberName) {
			//echoDebug("task different: memberName (".$this->memberName." != ".$compareTask->memberName.")");
			return false;
		}*/
		if ($this->iterationDeadlineTime != $compareTask->iterationDeadlineTime) {
			//echoDebug("task different: iterationDeadlineTime (".$this->iterationDeadlineTime." != ".$compareTask->iterationDeadlineTime.")");
			return false;
		}
		if ($this->auto != $compareTask->auto) {
			//echoDebug("task different: iterationDeadlineTime (".$this->iterationDeadlineTime." != ".$compareTask->iterationDeadlineTime.")");
			return false;
		}
		return true;
	}
}

// *********************************************************************
// TaskContainer
// *********************************************************************
class TaskContainer extends XPItemWithIncluded {
	var $totalTaskWeight;
	var $totalTaskDone;

	var $actualCompletion;
	var $theoryCompletion;

	var $nTasksResponsibleId;
	var $nTasksPairId;

	var $outOfDateTasks;

	function TaskContainer($arguments) {

		XPItemWithIncluded::XPItemWithIncluded($arguments);

		$this->initSimple("task");

		$this->outOfDateTasks = 0;
	}

	function setOptions(
		$nTasksResponsibleId	= ALL_TASKS,
		$nTasksPairId			= ALL_TASKS) {

	 	$this->nTasksResponsibleId	= $nTasksResponsibleId;
		$this->nTasksPairId			= $nTasksPairId;
	}

	function isAllTasks() {
		return $this->isAllResponsible() && $this->isAllPair();
	}

	function isAllResponsible() {
		return $this->nTasksResponsibleId == ALL_TASKS;
	}

	function isAllPair() {
		return $this->nTasksPairId == ALL_TASKS;
	}

	function addTask(& $task) {

		parent::addMultiIncluded("task", $task);
		
		$this->totalTaskWeight += $task->weight;
		$this->totalTaskDone += $task->done;
		
		if (isset($GLOBALS["gCurrentStatistics"])) {
			$GLOBALS["gCurrentStatistics"]->addTaskStatistic($task);
		}
		
	}

	function containsTaskWithRespAndPair($memberId, $pairId) {
		
		$nbTasks = sizeOf($this->multiIncludedItems["task"]);

		for ($k = 0; $k < $nbTasks; $k++) {
			$currentTask = & $this->multiIncludedItems["task"][$k];

			if ($currentTask->isFromMember($memberId) && $currentTask->isFromPair($pairId)) {
				return true;
			}
		}
		
		return false;
	}

}

// *********************************************************************
// TaskAndStoryContainer
// *********************************************************************
class TaskAndStoryContainer extends TaskContainer {
	var $totalStoryWeight;
	var $totalStoryWeightDone;

	var $outOfDateStories;

	var $storiesOrderBy = array();

	// List of all stories (id, name)
	var $basicStories = array();

	function TaskAndStoryContainer($arguments) {

		$this->TaskContainer($arguments);
		
		$this->initMulti(array("story", "task"));
		
		$this->outOfDateStories = 0;

	}
	
	/*
	function & getIncludedById($type, $id) {
		
		if ($type == "task") {
			// 1st try the usual way (isolated tasks)
			$item = & parent::getIncludedById($type, $id);
			
			if ($item == null) {
				// first try failed
				// lets look at stories !
				
				reset($this->multiIncludedItems);
		
				while (list($key, $val) = each($this->multiIncludedItems)) {
		
					if (($key != $type) && isset($val)) {
						
						reset($val);
						
						while (list($key2, $val2) = each($val)) {
							
							$val2->getIncludedById($type, $id);
							
						}
		
					}
		
				}
			}
		}
		
		return null;
	}
	*/

	function setOptions(
		$nTasksResponsibleId	= ALL_TASKS,
		$nTasksPairId			= ALL_TASKS,
		$storiesOrderBy			= "") {

		TaskContainer::setOptions(
			$nTasksResponsibleId,
			$nTasksPairId);

		$this->setStoriesOrderBy($storiesOrderBy);
	}
	
	function setStoriesOrderBy($orderBy	= "") {

		if (is_array($orderBy)) {
			$this->storiesOrderBy = $orderBy;
		} else {

			$this->storiesOrderBy = array();
		
			preg_match_all("/\|(\b[\w]+\b)=(\b[\w]+\b)\|/", $orderBy, $matches);

			for ($i = 0; $i < sizeof($matches[1]); $i ++) {
				
				$field = $matches[1][$i];

				$ascOrDesc = $matches[2][$i];

				$this->storiesOrderBy[$field] = $ascOrDesc;
			}
	 			
		}
	}

	function addMultiIncluded($where, & $item) {
		
		if ($where == "task") {
			echoDebug("****TaskAndStory:ADDTASK****");
			parent::addTask($item);

		} else {
			echoDebug("****parent:addMultiIncluded****");
			parent::addMultiIncluded($where, $item);
			
			$this->totalTaskWeight += $item->totalTaskWeight;
			$this->totalTaskDone += $item->totalTaskDone;

			if ($where == "story") {

				$this->totalStoryWeight += $item->weight;
			}
		}

	}

	function addTask(& $task) {
		
		$this->addMultiIncluded("task", $task);
	}

	function addStory(& $story) {
		
		$this->addMultiIncluded("story", $story);
	}

	function containsTaskWithRespAndPair($memberId, $pairId) {
		
		// First: check in tasks
		if (parent::containsTaskWithRespAndPair($memberId, $pairId)) {
			return true;
		}
		
		// Second: check in stories
		$nbStories = sizeOf($this->multiIncludedItems["story"]);
		for ($k = 0; $k < $nbStories; $k++) {
			$currentStory = & $this->multiIncludedItems["story"][$k];
			if ($currentStory->containsTaskWithRespAndPair($memberId, $pairId)) {
				return true;
			}
		}

		return false;
	}


}
// *********************************************************************
// Class BasicStory
// *********************************************************************
class BasicStory extends TaskContainer {

	var $iterationId;
	 
	function BasicStory($arguments) {
	
		if (!isset($arguments["type"]))			$arguments["type"]		= "basicStory";
		if (!isset($arguments["loadFactor"]))		$arguments["loadFactor"]	= 0;
		if (!isset($arguments["description"]))		$arguments["description"]	= "";
		if (!isset($arguments["workingDays"]))		$arguments["workingDays"]	= array();
		if (!isset($arguments["workingDaysName"]))	$arguments["workingDaysName"]	= "";
		
		PlanningCard::PlanningCard($arguments);

		$this->iterationId = $arguments["iterationId"];
	}

}

// *********************************************************************
// Class Story
// *********************************************************************
class Story extends BasicStory {
	
	var $weight;
	var $risk;
	var $priority;
	var $validationCriteria;
	var $bWeightedCompletion;
	
	var $iterationDeadlineTime;
	
	function isCompletionReady() {
		return PlanningCard::isCompletionReady();
	}
	
	function Story($arguments) {
	
		while (list($key, $val) = each($arguments)) {
			${$key} = $val;
		}
		
		$arguments["type"] = "story";
		
		BasicStory::BasicStory($arguments);
		
		
		$this->validationCriteria = $validationCriteria;
		$this->weight = $weight;
		$this->totalTaskWeight = 0;
		$this->totalTaskDone = 0;
		$this->risk = $risk;
		$this->priority = $priority;
		$this->iterationDeadlineTime = $iterationDeadlineTime;
		
		// This is stored into a class variable so that it can
		// be modified after all...
		$this->bWeightedCompletion = XP_WEIGHED_STORY_COMPLETION;
		
		$this->TaskContainer(array("story", $id, $projectId));
		
		$this->setOptions();
	}

	function & getObjectToInsert() {
		global $XP_DEFAULT_ITERATION_WORKING_PACE;
		global $XP_DEFAULT_ITERATION_WORKING_NAME;

		// Params
		$parameters				= func_get_arg(0);
		$itemType				= $parameters[0];

		$factory = & PlanningFactory::GetFactory();

		$newStory = & $factory->newStory(
			array(
				"id"					=> "insert_0",
				"projectId"				=> getCurrentProjectId(),
				"loadFactor"			=> XP_DEFAULT_LOAD_FACTOR,
				"title"					=> getLang("new_story"),
				"description"			=> "",
				"workingDays"			=> $XP_DEFAULT_ITERATION_WORKING_PACE,
				"workingDaysName"		=> $XP_DEFAULT_ITERATION_WORKING_NAME,
				"iterationId"			=> 0,
				"validationCriteria"	=> "",
				"weight"				=> 0,
				"risk"					=> 0,
				"priority"				=> 0,
				"iterationDeadlineTime" => getDefaultIterationDeadline()
			)
		);
			
		$result = & $newStory;
		
		return $result;
	}
	
	function shouldBeCompleted() {
		return $this->theoryCompletion >= 100;
	}
	
	function calculateCompletion() {
		// Calculate the story completion (theory and actual),
		// based on an average of included tasks completion

		//setDebug();
				
		$this->completionReady = false;

		$nbTasks = sizeOf($this->includedItems);

		$this->theoryCompletion = 0;
		$this->actualCompletion = 0;

		if ($nbTasks != 0) {

			$totalActualComp = 0;
			$totalWeight = 0;
			$totalTheoryComp = 0;

			$totalEndCompletion = 0;
			$shouldAllBeCompleted = true;
			
			$totalWeight = 0;

			for ($k = 0; $k < $nbTasks; $k++) {

				// ******** Warning: passing _reference_
				// Necessary to update Task completion...
				$currentTask = & $this->includedItems[$k];

				if (!$currentTask->isCompletionReady()) {
					$currentTask->calculateCompletion();
				}
				
				$currentTaskWeight = $currentTask->weight;
				
				if ($this->bWeightedCompletion) {
					$taskWeightForCalculation = $currentTaskWeight;
				} else {
					$taskWeightForCalculation = 1;
				}
							
				echoDebug("actualComp = $totalActualComp + ".$currentTask->actualCompletion." * ".$taskWeightForCalculation);
				$totalActualComp += $currentTask->actualCompletion * $taskWeightForCalculation;
				echoDebug("theoryComp = $totalTheoryComp + min(100, ".$currentTask->theoryCompletion.") * ".$taskWeightForCalculation);
				$totalTheoryComp += min(100, $currentTask->theoryCompletion) * $taskWeightForCalculation;
				
				if($shouldAllBeCompleted && $currentTask->shouldBeCompleted()) {
					// the task should be completed	
					$totalEndCompletion += $currentTask->theoryCompletion * $taskWeightForCalculation;
				} else {
					$shouldAllBeCompleted = false;
					//echoDebug("Story ".$this->id.": task not completed = ".$currentTask->theoryCompletion);
				}
				
				if ($currentTask->outOfTime > $this->outOfDateTasks) {
					// This tasks is not synchronized with iteration
					$this->outOfDateTasks = $currentTask->outOfTime;
				}

				if ($this->bWeightedCompletion) {
					$totalWeight += $currentTaskWeight;
				} else {
					$totalWeight ++;
				}

			}

			if ($shouldAllBeCompleted) {
				// All included tasks should be completed:
				// so we accept more than 100 completion...
				
				echoDebug("Should all be completed: totalTheoryComp = $totalEndCompletion");
				
				$totalTheoryComp = $totalEndCompletion;
				// the one that added all completions

				//echoDebug("Story ".$this->id.": should all be completed, total = ".$totalEndCompletion);
			}
			
			echoDebug("theoryCompletion = floor($totalTheoryComp / $totalWeight)");
			
			$this->theoryCompletion = @floor($totalTheoryComp / $totalWeight);
			
			$this->actualCompletion = @floor($totalActualComp / $totalWeight);
					
		}

		//setDebug(false);
		
		$this->completionReady = true;
	}

		
	
	// _________ Private functions
	
	/**
	* Functions giving rights to modify the team member
	*/
	
	function isAllowedToDelete() {
				
		// If is allowed to delete parent iterations,
		// means is allowed to delete stories...
		return @($GLOBALS["gConnectedUser"]->rights["story"]["delete"]["all"] ||  ($GLOBALS["gConnectedUser"]->rights["story"]["delete"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOf($this->projectId)));
	}
}

// *********************************************************************
// Class BasicIteration
// *********************************************************************
class BasicIteration extends TaskAndStoryContainer {
	 
	function BasicIteration($arguments) {
			
		while (list($key, $val) = each($arguments)) {
			${$key} = $val;
		}

		if (!isset($arguments["type"]))				$arguments["type"]				= "basicIteration";
		if (!isset($arguments["loadFactor"]))		$arguments["loadFactor"]		= 0;
		if (!isset($arguments["description"]))		$arguments["description"]		= "";
		if (!isset($arguments["workingDays"]))		$arguments["workingDays"]		= array();
		if (!isset($arguments["workingDaysName"]))	$arguments["workingDaysName"]	= "";
		
		PlanningCard::PlanningCard($arguments);
	}

}

// *********************************************************************
// Class Iteration
// *********************************************************************
class Iteration extends BasicIteration {
	
	var $deadlineDate;
	var $deadlineYear;
	var $deadlineMonth;
	var $deadlineDay;
	var $deadlineWeekDay;
	var $deadlineTime;
	 
	function Iteration($arguments) {
	
		while (list($key, $val) = each($arguments)) {
			${$key} = $val;
		}
		
		$arguments["type"] = "iteration";
		
		BasicIteration::BasicIteration($arguments);
		
		$this->TaskAndStoryContainer(array("iteration", $id, $projectId));
		
		$this->setDeadline($deadline);
		
		$this->workingDays = $workingDays;
		$this->workingDaysName = $workingDaysName;
		
		$this->totalTaskWeight = 0;
		$this->totalTaskDone = 0;
		$this->totalStoryWeight = 0;
		
		$this->setOptions();
	}

	function & getObjectToInsert() {
		global $XP_DEFAULT_ITERATION_WORKING_PACE;
		global $XP_DEFAULT_ITERATION_WORKING_NAME;

		$factory = & PlanningFactory::GetFactory();

		$newIteration = & $factory->newIteration(
			array(
				"id"				=> "insert",
				"projectId"			=> getCurrentProjectId(),
				"loadFactor"			=> $GLOBALS["gCurrentStatistics"]->idealLF,
				"title"				=> getLang("new_iteration"),
				"description"			=> "",
				"workingDays"			=> $XP_DEFAULT_ITERATION_WORKING_PACE,
				"workingDaysName"		=> $XP_DEFAULT_ITERATION_WORKING_NAME,
				"deadline"			=> getDefaultIterationDeadline()
			)
		);

		return $newIteration;
	}

	
	function setDeadline($deadline) {
		$this->deadlineDate = $deadline;
		
		list($this->deadlineYear,$this->deadlineMonth,$this->deadlineDay) = explode("-",$deadline);
		$this->deadlineTime = mktime(0,0,0,$this->deadlineMonth,$this->deadlineDay,$this->deadlineYear);
		$this->deadlineWeekDay = date("D",$this->deadlineTime);
	}
			
	function calculateCompletion() {
		// Calculate the iteration completion (theory and actual),
		// based on an average of included stories and tasks completions

		echoDebug("Iteration::calculateCompletion");
		
		$this->completionReady = false;
		$this->outOfDateTasks = 0;
		
		$nbTasks = sizeOf($this->multiIncludedItems["task"]);
		$nbStories = sizeOf($this->multiIncludedItems["story"]);
		
		$this->totalStoryWeightDone = 0;
		
		if ($nbTasks == 0 && $nbStories == 0) {
			$this->theoryCompletion = 0;
			$this->actualCompletion = 0;
		} else {
			$totalCompletion = 0;
			$totalIncludedCompletion = 0;
			$totalWeight = 0;
			
			$totalEndCompletion = 0;
			$shouldAllBeCompleted = true;
			
			for ($k = 0; $k < $nbStories; $k++) {
				// ******** Warning: passing _reference_
				// Necessary to update story completion...
				$currentStory = & $this->multiIncludedItems["story"][$k];	
			
				if (!$currentStory->isCompletionReady()) {

					$currentStory->calculateCompletion();
				}

				//echoDebug("Story ".$currentStory->id);
				
				if ($currentStory->outOfDateTasks > $this->outOfDateStories) {
					// This tasks is not synchronized with iteration
					$this->outOfDateStories = $currentStory->outOfDateTasks;

					echoDebug("Story ".$currentStory->id." out of date [".$currentStory->outOfDateTasks."] in iteration ".$this->id);

				}
				
				// each story's completion is weighted with its
				// number of tasks...
				
				$weight = sizeOf($currentStory->includedItems);
				
				$actualCompletion = $weight * $currentStory->actualCompletion;
				
				$totalCompletion += $actualCompletion;
				
				if($shouldAllBeCompleted && $currentStory->shouldBeCompleted()) {
					// the task should be completed
					$totalEndCompletion += $weight * $currentStory->theoryCompletion;
				} else {
					$shouldAllBeCompleted = false;
					//echoDebug("Iteration ".$this->id.": story not completed = ".$currentStory->theoryCompletion);
				}
				
				$totalIncludedCompletion += $weight * min(100, $currentStory->theoryCompletion);
				
				$this->totalStoryWeightDone += $currentStory->weight * $currentStory->actualCompletion / 100; 
				
				$totalWeight += $weight;
			}
			
			for ($k = 0; $k < $nbTasks; $k++) {
			
				// ******** Warning: passing _reference_
				// Necessary to update Task completion...
				$currentTask = & $this->multiIncludedItems["task"][$k];	

				if (!$currentTask->isCompletionReady()) {
					$currentTask->calculateCompletion();
				}

				
				if ($currentTask->outOfTime > $this->outOfDateTasks) {
					// This tasks is not synchronized with iteration
					$this->outOfDateTasks = $currentTask->outOfTime;
				}
				
				$totalCompletion += $currentTask->actualCompletion;
				
				if($shouldAllBeCompleted && $currentTask->shouldBeCompleted()) {
					// the task should be completed
					$totalEndCompletion += $currentTask->theoryCompletion;
				} else {
					$shouldAllBeCompleted = false;
					//echoDebug("Iteration ".$this->id.": task not completed = ".$currentTask->theoryCompletion);
				}
				$totalIncludedCompletion += min(100, $currentTask->theoryCompletion);

				// each task's completion is weighted by one only
				$totalWeight ++;
			}
			
			if ($shouldAllBeCompleted) {
				// All included tasks and stories should be completed:
				// so we accept more than 100 completion...
				$total = $totalEndCompletion;
				// the one that added all completions

				//echoDebug("Iteration ".$this->id.": should all be completed, total = ".$totalEndCompletion);
			} else {
				// we accept only less than 100 completion
				$total = $totalIncludedCompletion;
				// the one that added min(100, completion)...
			}
			
			$this->theoryCompletion = @floor($total / $totalWeight);
			$this->actualCompletion = @floor($totalCompletion / $totalWeight);
		}
		
		$this->completionReady = true;

		echoDebug("Iteration::calculateCompletion - end");
	}
		
}



// *********************************************************************
// Class Planning
// *********************************************************************
class Planning extends TaskAndStoryContainer {
	var $loadFactor;
	
	var $currentIterationId;
	
	var $populate;

	var $basicIterations = array();
	
	function setDefaultOptions() {

		$values = array();
		
		$values[XP_PLANNING_POP_RESPONSIBLE]		= ALL_TASKS;
		$values[XP_PLANNING_POP_PAIR]				= ALL_TASKS;
		$values[XP_PLANNING_POP_ITERATION_CURRENT]	= true;
	 	$values[XP_PLANNING_POP_ITERATION_OTHER]	= true;
	 	$values[XP_PLANNING_POP_STORY]				= true;
	 	$values[XP_PLANNING_POP_TASK]				= true;
		$values[XP_PLANNING_POP_STORIES_ORDERBY]	= "";

		$this->setOptions($values, false);
	}

	function setOptions($values, $bDoReset = false) {

		// Affect default values
		if ($bDoReset) {
			$this->setDefaultOptions();
		}

		if (isSet($values[XP_PLANNING_POP_RESPONSIBLE])) {

			$newResponsibleId = $values[XP_PLANNING_POP_RESPONSIBLE];

			// Only affect if member of the current project (might bug if switching from a project to another)
			// ...or if == 0 (nobody)
			if ($newResponsibleId == 0 || isset($GLOBALS["gBasicMembers"][$newResponsibleId])) {
				$this->nTasksResponsibleId = $newResponsibleId;
			} else {
				$this->nTasksResponsibleId = ALL_TASKS;
			}
		}

		if (isSet($values[XP_PLANNING_POP_PAIR])) {

			$newPairId = $values[XP_PLANNING_POP_PAIR];

			// Only affect if member of the current project (might bug if switching from a project to another)
			// ...or if == 0 (nobody)
			if ($newPairId == 0 || isset($GLOBALS["gBasicMembers"][$newPairId])) {
				$this->nTasksPairId = $newPairId;
			} else {
				$this->nTasksPairId = ALL_TASKS;
			}
		}

		// Show Current iteration?
		if (isSet($values[XP_PLANNING_POP_ITERATION_CURRENT])) {
			$this->populate[XP_PLANNING_POP_ITERATION_CURRENT] =
				$values[XP_PLANNING_POP_ITERATION_CURRENT];
		}

		// Show previous iterations?
		if (isSet($values[XP_PLANNING_POP_ITERATION_OTHER])) {
			$this->populate[XP_PLANNING_POP_ITERATION_OTHER] =
				$values[XP_PLANNING_POP_ITERATION_OTHER];
		}
		// Show isolated stories?
		if (isSet($values[XP_PLANNING_POP_STORY])) {
			$this->populate[XP_PLANNING_POP_STORY] =
				$values[XP_PLANNING_POP_STORY];
		}
		// Show isolated tasks?
		if (isSet($values[XP_PLANNING_POP_TASK])) {
			$this->populate[XP_PLANNING_POP_TASK] =
				$values[XP_PLANNING_POP_TASK];
		}

		// Order by
		if (isSet($values[XP_PLANNING_POP_STORIES_ORDERBY])) {
			$this->setStoriesOrderBy($values[XP_PLANNING_POP_STORIES_ORDERBY]);
		}

		return true;
	}

	function Planning($arguments) {
		global $XP_DEFAULT_ITERATION_WORKING_PACE;
		global $XP_DEFAULT_ITERATION_WORKING_NAME;
		
		$i = 0;

		$projectId 					= $arguments["projectId"];
		
		$this->workingDays = $XP_DEFAULT_ITERATION_WORKING_PACE;
		$this->workingDaysName = $XP_DEFAULT_ITERATION_WORKING_NAME;
		
		$GLOBALS["gCurrentStatistics"] = & new Statistics();
	
		//$gDefaultWorkingDays = array(
		//	0 => $XP_DEFAULT_ITERATION_WORKING_PACE);

		$this->projectId = $projectId;
		$this->loadFactor = XP_DEFAULT_LOAD_FACTOR;
		
		$this->multiIncludedItems["iteration"] = array();

		$this->TaskAndStoryContainer(array("planning", 0, $projectId));

		$this->initMulti(array("iteration"));

		// Both point to the same...
		// The selection will be done later...
		$this->multiIncludedItems[XP_PLANNING_POP_ITERATION_CURRENT] = & $this->multiIncludedItems["iteration"];
		$this->multiIncludedItems[XP_PLANNING_POP_ITERATION_OTHER] = & $this->multiIncludedItems["iteration"];
		
		$gCurrentPlanning = &$this;
		
		$this->setDefaultOptions();
	}

	function addStory(& $story) {

		parent::addStory($story);
	}
	
	function calculateCompletion() {
		
		for ($j = 0; $j < sizeOf($this->multiIncludedItems["iteration"]); $j ++) {
			$this->multiIncludedItems["iteration"][$j]->calculateCompletion();
		}
		
		// Stories not affected to an iteration yet...
		for ($j = 0; $j < sizeOf($this->multiIncludedItems["story"]); $j ++) {
			$this->multiIncludedItems["story"][$j]->calculateCompletion();
		}
		
		// Tasks not affected to an iteration yet, nor to a story...
		for ($j = 0; $j < sizeOf($this->multiIncludedItems["task"]); $j ++) {
			$this->multiIncludedItems["task"][$j]->calculateCompletion();
		}
		
		$GLOBALS["gCurrentStatistics"]->calculateIdealLF();
	}
	
	
	function addIteration(& $iteration) {
		// ** By reference so if iteration modified outside method,
		// changes will appear in this planning object...
		//global $gDefaultWorkingDays;
		
		//global $gIterationNames;
		//global $gInIterationStoryNames;

		if ($iteration->id == $this->currentIterationId) {
	 		$iteration->isCurrent = true;
	 	}
			
		parent::addMultiIncluded("iteration", $iteration);
		
		
		$nbStories = sizeOf($iteration->multiIncludedItems["story"]);
		
		$currentStories = array();
		
		for ($k = 0; $k < $nbStories; $k++) {
			$currentStory = & $iteration->multiIncludedItems["story"][$k];
			$currentStories[$currentStory->id] = $currentStory->title;
		}
	}
	
}
// *********************************************************************
// Others
// *********************************************************************
class Statistics {
	var $theoryEndComp = array();
	var $idealLF = XP_DEFAULT_LOAD_FACTOR;
	
	function addTaskStatistic(& $task) {
		if ($task->theoryEndCompletion != XP_IMPOSSIBLE_ID) {
			$this->theoryEndComp[] = $task->theoryEndCompletion;
		}
	}
	
	function calculateIdealLF() {
		$totalTheoryComp = 0;
		$nbTasks = 0;
		reset($this->theoryEndComp);
		while(list($key, $theoryComp) = each($this->theoryEndComp)) {
			$nbTasks ++;
			$totalTheoryComp += $theoryComp;
		}
		
		if ($nbTasks != 0) {
			$this->idealLF = $totalTheoryComp/($nbTasks * 100);
		}
	}
}
?>
