<?php

class ProjectsFactory extends ModelFactory {
	
	function & newProjectList($arguments = array()) {

		$item = & new ProjectList($arguments);

		$controler = & new ProjectListControler($item);

		return $item;
	}
	
	function & newProject($arguments) {
		
		$item = & new Project($arguments);
		
		$controler = & new ProjectControler($item);
		
		return $item;
	}
		
	function & newProjectLink($arguments) {

		$item = & new ProjectLink($arguments);

		new ProjectLinkControler($item);

		return $item;
	}
	
}

$GLOBALS["gProjectsFactory"] = & new ProjectsFactory();

// WARNING
// It is crucial to use $GLOBALS for references!!!!
$GLOBALS["gFactory"] = & $GLOBALS["gProjectsFactory"];

// *********************************************************************
// Class ProjectList
// *********************************************************************
class ProjectList extends XPItemList {
	var $availableLinkTypes;
	
	function ProjectList() {
		XPItemList::XPItemList(array(0, "project"));
	}
	
	function addIncluded($projectToAdd) {
		global $projectIds;
		global $projectNames;

		parent::addIncluded($projectToAdd);

		$projectIds[] = $projectToAdd->id;
		$projectNames[] = $projectToAdd->title;
	}
	
}

// *********************************************************************
// Class BaseProject
// *********************************************************************
class BaseProject extends XPItemWithIncluded {
	var $title;
    
	var $isCurrent;
	
	function BaseProject($arguments) {

		while (list($key, $val) = each($arguments)) {
			//echoDebug("KEY : $key / VAL $val");
			${$key} = $val;
		}
		
		// We use this id as the "project id"
		XPItemWithIncluded::XPItemWithIncluded(array("project", $id, $id));
		
		$this->title				= $title;
		
		if (compareCurrentProjectId($this->id)) {
			$this->isCurrent = true;
		} else {
			$this->isCurrent = false;
		}
		
	}
	
	/**
	* Functions giving rights to modify the team member
	*/
	
	function isAllowedToInsert() {
		
		/**
		* No ["insert"]["proj"] for project... would have
		* no sense!
		*/
		return @($GLOBALS["gConnectedUser"]->rights[$this->type]["insert"]["all"]);
	}
	
	function isAllowedToDelete() {
		
		return @($GLOBALS["gConnectedUser"]->rights[$this->type]["delete"]["all"] || ($GLOBALS["gConnectedUser"]->rights[$this->type]["delete"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOf($this->id)));
	}
	
	function isAllowedToUpdate() {
		
		return @($GLOBALS["gConnectedUser"]->rights[$this->type]["modify"]["all"] || ($GLOBALS["gConnectedUser"]->rights[$this->type]["modify"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOf($this->id)));
	}
	
	function isAllowedToSelect() {
		
		return @($GLOBALS["gConnectedUser"]->rights[$this->type]["select"]["all"] || ($GLOBALS["gConnectedUser"]->rights[$this->type]["select"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOf($this->id)));
	}
	
	function isAllowedToDisplayContent() {
		
		return @($GLOBALS["gConnectedUser"]->rights["section"]["Projects"]["all"] || ($GLOBALS["gConnectedUser"]->rights["section"]["Projects"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOf($this->id)));
	}
	
}
// *********************************************************************
// Class Project
// *********************************************************************
class Project extends BaseProject {
	var $description;
	var $availableLinkTypes;
	
	function Project($arguments) {

		while (list($key, $val) = each($arguments)) {
			//echoDebug("KEY : $key / VAL $val");
			${$key} = $val;
		}
		
		// We use this id as the "project id"
		BaseProject::BaseProject($arguments);
		
		$this->description			= $description;
		$this->availableLinkTypes	= isset($availableLinkTypes)?$availableLinkTypes:array();
		
		@$this->initMulti(array_keys($availableLinkTypes));
		
	}

	function & getObjectToInsert() {

		$arguments = func_get_arg(0);

		$availableLinkTypes = & $arguments[0];

		$factory = & ProjectsFactory::GetFactory();
		
		$newProject = & $factory->newProject(
			array(
				"id"			=> "insert",
				"title"			=> getLang("new_project"),
				"description"		=> "",
				"availableLinkTypes"	=> $availableLinkTypes
			)
		);
		
		$result = & $newProject; 

		return $result;
	}
	
	function addProjectLink($projectLinkToAdd) {
		parent::addMultiIncluded($projectLinkToAdd->linkType, $projectLinkToAdd);
	}
}



// *********************************************************************
// Class ProjectLink
// *********************************************************************
class ProjectLink extends XPItem {
	var $title;
	var $url;
	var $linkType;
	var $linkTypeName;
	
	function ProjectLink($arguments) {

		while (list($key, $val) = each($arguments)) {
			${$key} = $val;
		}
		
		$this->XPItem(array("projectLink", $id, $projectId));
		
		$this->linkType = $linkType;
		$this->linkTypeName = @$linkTypeName;
		$this->title = $title;
		$this->url = $url;
	}

	function & getObjectToInsert() {

		// Params
		$arguments		= func_get_arg(0);
		$i = 0;

		$projectId		= $arguments[$i++];
		$linkType		= 0;//$arguments[$i++];
		$linkTypeName		= 0;//$arguments[$i++];
		$title			= "";//$arguments[$i++];
		
		
		$factory = & ProjectsFactory::GetFactory();

		$newLink = & $factory->newProjectLink(
			array(
				"id"			=> "insertLink_".$projectId."_".$linkType,
				"title"			=> $title,
				"projectId"		=> $projectId,
				"linkType"		=> $linkType,
				"linkTypeName"		=> $linkTypeName,
				"url"			=> ""
			)
		);

		return $newLink;
	}
	
	
	
}
?>
