<?php

define("XP_TEAM_AFFECTED",		0);
define("XP_TEAM_NOT_AFFECTED",	1);

class TeamFactory extends ModelFactory {
	// STATIC
	function & GetFactory() {
		
		
		return $GLOBALS["gTeamFactory"];
	}

	function & newTeam($arguments) {

		$item = & new Team($arguments);

		new TeamControler($item);

		return $item;
	}
	
	function & newMember($arguments) {

		$item = & new Member($arguments);

		new MemberControler($item);

		return $item;
	}

	function & newProfilesList($arguments = array()) {

		$item = & new ProfilesList($arguments);

		new ProfilesListControler($item);

		return $item;
	}

	function & newProfile($arguments) {
		
		$item = & new Profile($arguments);

		new ProfileControler($item);

		return $item;
	}

	function & newProfileRight($arguments) {
		
		$item = & new ProfileRight($arguments);

		new ProfileRightControler($item);

		return $item;
	}
	
}

$GLOBALS["gTeamFactory"] = & new TeamFactory();

// *********************************************************************
// Class Team
// *********************************************************************
class Team extends XPItemMultiList {

	var $currentProjectName;
	var $notToInclude;
	
	function Team($arguments) {

		while (list($key, $val) = each($arguments)) {
			${$key} = $val;
		}

		XPItemMultiList::XPItemMultiList(array($projectId));

		$this->id = 0;

		$this->currentProjectName = $currentProjectName;
		
	}
		
	function addMember($member, $bAffected) {

		if ($member->id == "0" || $member->id == XP_IMPOSSIBLE_ID) {
			//handles filtering out Impossible id and Nobody account
			return;
		}

		parent::addMultiIncluded($bAffected ? XP_TEAM_AFFECTED : XP_TEAM_NOT_AFFECTED, $member);
	
	}
		
}


// *********************************************************************
// Class Member
// *********************************************************************
class Member extends XPItem {
	var $title;
	var $description;
	var $profileId;
	var $affectations;
	
	function Member($arguments) {
		

		while (list($key, $val) = each($arguments)) {
			${$key} = $val;
		}

		// Project id has no meaning here => 0
		$this->XPItem(array("member", $id, 0));

		$this->isCurrent = $GLOBALS["gConnectedUser"]->isMyself($this);
		
		$this->title = $title;
		$this->profileId = $profileId;
		$this->description = $description;
		
		$this->affectations = array();
	}

	function & getObjectToInsert() {
		global $gDevelopperProfileId;

		$arguments	= func_get_arg(0);

		$i = 0;
		$inProject	= $arguments[$i++];

		$factory = & TeamFactory::GetFactory();
					
		$newMember = & $factory->newMember(
			array(
				"id"			=> "insert_$inProject",
				"profileId"		=> $gDevelopperProfileId,
				"title"			=> getLang("someone"),
				"description"	=> getLang("someone_nice") /*,
				getLang("someone_mail") */
			)
		);

		return $newMember;

	}
		
	/**
	* Functions giving rights to modify the team member
	*/
	
	function isAllowedToDelete() {
		
		// Notice the "!"
		if (!@($GLOBALS["gConnectedUser"]->rights["member"]["delete"]["all"] || ($GLOBALS["gConnectedUser"]->rights["member"]["delete"]["me"] && $GLOBALS["gConnectedUser"]->isMyself($this)))) {
			
			// Notice the "!"
			if (!@($GLOBALS["gConnectedUser"]->rights["member"]["delete"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOfAnyOf($this->affectations))) {
				 return false;
			}
		}
		return true;
	}
	
	function isAllowedToUpdate() {
		
		// Notice the "!"
		if (!@($GLOBALS["gConnectedUser"]->rights["member"]["modify"]["all"] || ($GLOBALS["gConnectedUser"]->rights["member"]["modify"]["me"] && $GLOBALS["gConnectedUser"]->isMyself($this)))) {
			
			// Notice the "!"
			if (!@($GLOBALS["gConnectedUser"]->rights["member"]["modify"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOfAnyOf($this->affectations))) {
				return false;
			}
		}
		
		return true;
	}
	
	function isAllowedToInsert($memberProjectId = 0) {
		
		// Has to be allowed to insert but also to 
		return @($GLOBALS["gConnectedUser"]->rights["member"]["insert"]["all"] && (($memberProjectId == 0) || $this->isAllowedToOrDeAffectFromToProject($memberProjectId)));
	}
	
	function isAllowedToOrDeAffectFromToProject($memberProjectId) {
		
		
		return @($GLOBALS["gConnectedUser"]->rights["member"]["affect"]["all"] || ($GLOBALS["gConnectedUser"]->rights["member"]["affect"]["me"] && $GLOBALS["gConnectedUser"]->isMyself($this)) || ($GLOBALS["gConnectedUser"]->rights["member"]["affect"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOf($memberProjectId)) || ($GLOBALS["gConnectedUser"]->rights["member"]["affect"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOf(-$memberProjectId)));
	}
	
	function isAllowedToChangePassword() {
		
		
		return @($GLOBALS["gConnectedUser"]->rights["member"]["changePass"]["all"] || ($GLOBALS["gConnectedUser"]->rights["member"]["changePass"]["me"] && $GLOBALS["gConnectedUser"]->isMyself($this)) || ($GLOBALS["gConnectedUser"]->rights["member"]["changePass"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOfAnyOf($this->affectations)));
	}
	
	function isAllowedToChangeProfile() {
		
		
		$isAllowed = @($GLOBALS["gConnectedUser"]->rights["member"]["changeProfile"]["all"] || ($GLOBALS["gConnectedUser"]->rights["member"]["changeProfile"]["me"] && $GLOBALS["gConnectedUser"]->isMyself($this)) || ($GLOBALS["gConnectedUser"]->rights["member"]["changeProfile"]["proj"] && $GLOBALS["gConnectedUser"]->isMemberOfAnyOf($this->affectations)));
		
		/**
		* If the user is trying to insert a member but
		* doesn't have rights to change profile, the profile of the new
		* team member has to be fixed to the logged in user's one
		*/
		if (!$isAllowed && ereg("insert", $this->id)) {
			$this->profileId = $GLOBALS["gConnectedUser"]->profileId;
		}
		
		return $isAllowed;
	}
	
}


// *********************************************************************
// Class ProfilesList
// *********************************************************************
class ProfilesList extends XPItemList {

	function ProfilesList() {
		XPItemList::XPItemList(array(0, "profile"));

		$this->id = 1;
	}

	/**
	* This method is surcharged to change default profiles names 
	* in global list into their translated names...
	*/
	function addIncluded($profile) {
		global $gExistingProfiles;

		parent::addIncluded($profile);

		if ($profile->isDefault) {
			$gExistingProfiles[$profile->id] = getLang("profile_name_".$profile->title);
		}		
	}
}


// *********************************************************************
// Class Profile
// *********************************************************************
class Profile extends XPItemWithIncluded {
	var $title;
	var $description;
	
	var $isDefault;
	var $isAdmin;

	function Profile($arguments) {

		while (list($key, $val) = each($arguments)) {
			${$key} = $val;
		}
		
		// Here project id has no meaning: 0
		XPItemWithIncluded::XPItemWithIncluded(array("profile", $id, 0));

		$this->title = $title;
		$this->description = $description;
		
		$this->isDefault = ($this->title == "Admin" || $this->title == "Anonymous" || $this->title == "Developper" || $this->title == "Coach");

		$this->isAdmin = ($this->title == "Admin");
	}

	function & getObjectToInsert() {

		$arguments	= func_get_arg(0);

		$i = 0;

		$factory = & TeamFactory::GetFactory();
				
		$newProfile = & $factory->newProfile(
			array(
				"id"			=> "insert",
				"title"			=> getLang("new_profile"),
				"description"	=> getLang("new_profile_desc")
			)
		);

		return $newProfile;

	}
		
	function isAllowedToDelete() {
		
		return @($GLOBALS["gConnectedUser"]->rights[$this->type]["delete"]["all"] && !$this->isDefault);
	}
	
	function isAllowedToUpdateDescription() {
		
		return @($GLOBALS["gConnectedUser"]->rights[$this->type]["modify"]["all"] && !$this->isDefault);
	}
	
	function isAllowedToUpdateRights() {
		
		return @($GLOBALS["gConnectedUser"]->rights[$this->type]["modify"]["all"] && !$this->isAdmin);
	}
	
	function isAllowedToInsert() {
		
		return @($GLOBALS["gConnectedUser"]->rights[$this->type]["insert"]["all"]);
	}

	function isAllowedToUpdate() {
		return $this->isAllowedToUpdateDescription();
	}
	
}



// *********************************************************************
// Class ProfileRight
// *********************************************************************
class ProfileRight extends XPItem {
	
	function ProfileRight($arguments) {
		
		// Here project id has no meaning: 0
		$this->XPItem(array("profileRight", $arguments["id"], 0));

	}
}
?>
