<?php
require "Model/Planning_model.inc.php";


class CalendarFactory extends PlanningFactory {

	function & newTask($arguments) {

		// Overrides Planning function to return a XPCalTask object
		// instead of a Task one...

		$item = & new XPCalTask($arguments);

		new XPCalTaskControler($item);

		CalendarFactory::initItem($item);

		return $item;
	}
	
	function & newStory($arguments) {

		// Overrides Planning function to return a XPCalStory object
		// instead of a Story one...
		$item = & new XPCalStory($arguments);

		// @note : will replace the basic StoryControler of this item
		new XPCalStoryControler($item);

		CalendarFactory::initItem($item);

		return $item;
	}
	
	function & newIteration($arguments) {

		// Overrides Planning function to return a XPCalIteration object
		// instead of a Iteration one...
		$item = & new XPCalIteration($arguments);
		
		ModelFactory::init($item);

		// @note : will replace the basic IterationControler of this item
		new XPCalIterationControler($item);

		CalendarFactory::initItem($item);

		return $item;
	}

	function & newDay($arguments) {

		// Overrides Planning function to return a XPCalIteration object
		// instead of a Iteration one...
		$item = & new XPCalDay($arguments);

		new XPCalDayControler($item);

		return $item;
	}

	/// For compatibility with Planning factory...
	function & newPlanning($arguments) {
		
		return $this->newXPCalendar($arguments);
		
	}


	function & newXPCalendar($arguments) {
		
		$item = & new XPCalendar($arguments);
		
		// @note : will replace the basic PlanningControler of this item
		new XPCalendarControler($item);
		
		return $item;
	}



	// Specific to calendar... reused in CalendarView...
	function initItem(& $item) {

		if (isset($this->caller) && isset($this->caller->item)) {
            	
			$item->calStart	= $this->caller->item->calStart;
			$item->calEnd	= $this->caller->item->calEnd;

		}

	}
	
}

$GLOBALS["gPlanningFactory"] = & new CalendarFactory();

// *********************************************************************
// Class XPCalendarItem
// *********************************************************************
class XPCalendarItem {

	var $calStart;
	var $calEnd;

	function setCalStart($calStart) {
		$this->calStart = $calStart;
	}

	function getCalStart() {
		return $this->calStart;
	}

	function setCalEnd($calEnd) {
		$this->calEnd = $calEnd;
	}

	function getCalEnd() {
		return $this->calEnd;
	}
}

// *********************************************************************
// Class XPCalendar
// *********************************************************************
class XPCalendar extends Planning {
	var $theTime;
	var $calStart = "9999-99-99";
	var $calEnd = "0000-00-00";
	var $periods = array();
	var $type;
	var $iterationId = 0;
	var $sortedIterations = array();
	var $calendarType;

	function setDefaultOptions() {

		Planning::setDefaultOptions();

		$values[XP_CALENDAR_CAL_TYPE] = CALENDAR_MONTH;

		$this->setOptions($values, false);
	}

	function setOptions($values, $bDoReset = false) {

		// Override Planning options
		$values[XP_PLANNING_POP_ITERATION_CURRENT]	= true;
	 	$values[XP_PLANNING_POP_ITERATION_OTHER]	= true;
	 	$values[XP_PLANNING_POP_STORY]				= true;
	 	$values[XP_PLANNING_POP_TASK]				= true;
		$values[XP_PLANNING_POP_STORIES_ORDERBY]	= "";

		// Affect planning options
		$result = Planning::setOptions($values, $bDoReset);

		// Calendar specific
		if (isSet($values[XP_CALENDAR_CAL_TYPE])) {
			$this->calendarType = $values[XP_CALENDAR_CAL_TYPE];

			$this->periods = array();
		
			// Time to change from calendar type
			$result = false;

			// Today by default
			$theDate = isSet($values[XP_CALENDAR_CAL_DATE])?@dateFromHtmlToSql($values[XP_CALENDAR_CAL_DATE]):date("Y-m-d");
	        

			//echo "zoubi".$theDate;

			switch ($this->calendarType) {

				default:
        		case CALENDAR_MONTH:
        			$result = $this->setMonth($theDate);
        			break;

        		case CALENDAR_WEEK:
        			$result = $this->setWeek($theDate);
        			break;

        		case CALENDAR_ITERATION:
        			// Current iteration by default
					$iterationId = isSet($values[XP_CALENDAR_CAL_ITE_ID])?$values[XP_CALENDAR_CAL_ITE_ID]:FAKEID_CURRENT_ITERATION;
        			$result = $this->setIteration($iterationId);
        			break;

        		case CALENDAR_DATES:
        			// Today by default
					$calStart = isset($values[XP_CALENDAR_CAL_START])?dateFromHtmlToSql($values[XP_CALENDAR_CAL_START]):$theDate;
	        		
        			// Today + one week by default
					$calEnd = isset($values[XP_CALENDAR_CAL_END])?dateFromHtmlToSql($values[XP_CALENDAR_CAL_END]):
						date("Y-m-d", strtotime($theDate) + ONEDAY * 7);
        			
        			$this->setSpecificDates($calStart, $calEnd);
        			$result = true; 
        			break;

			}
		}
		return $result;
	}

	function XPCalendar($arguments) {

		parent::Planning($arguments);
		$this->calEnd = date("Y-m-d", time());
		
		$this->sortedIterations = array();
		$this->periods			= array();
	}
	
	function sortTasks() {

		$unsortedTasks = $this->multiIncludedItems["task"];
		$this->multiIncludedItems["task"] = array();

		for($i = 0; $i < sizeof($unsortedTasks); $i++) {
			$task = & $unsortedTasks[$i];
			//$task->calculateCompletion();
			$this->addTask($task);
		}

		// Time to keep only iterations
		// in the period and sort them
		/*
		for($i = 0; $i < sizeof($this->multiIncludedItems["iteration"]); $i++) {
            $this->addIteration($this->multiIncludedItems["iteration"][$i]);
		}
		*/
		
	}
	
	function addTask(& $task) {

		// To avoid crazy situations
		if ($task->endTime < $task->startTime) {
			displayErrorMsg("Wrong task", "Task ".$task->id." has its end date before its start date!<br/>Ignored.");
		} else {
		
			$size = sizeof($this->multiIncludedItems["task"]);

			for($i = 0; $i < $size; $i++) {

				$sortedTask = & $this->multiIncludedItems["task"][$i];

				$diffStart = $sortedTask->startTime - $task->startTime;

				if ($diffStart > 0) {
					// task starts earlier
					// ==> WILL INSERT BEFORE
					break;

				} elseif ($diffStart == 0) {

					// Both tasks start at same time
					if ($task->endTime <= $sortedTask->endTime) {
						// Ends earlier
						// ==> WILL INSERT BEFORE
						break;
					}
				}
			}
			// => time to insert task
			echoDebug("addTask $i => add ".$task->id." from user ".$task->memberId);

			array_splice($this->multiIncludedItems["task"], $i, 0, array($task));
		}
	}
		
	function addPeriod($start, $end) {
		if ($start < $end) {
			if ($start < XPCalendarItem::getCalStart()) {
				XPCalendarItem::setCalStart($start);
			}
			if ($end > XPCalendarItem::getCalEnd()) {
				XPCalendarItem::setCalEnd($end);
			}
			
			$startTime = strtotime($start);
			$endTime = strtotime($end);
			$days = array();


			echoDebug("Adding period: $startTime => $endTime");

			for ($dayTime = $startTime; $dayTime <= $endTime; $dayTime += ONEDAY) {
				// for each day between $start and $end

				$factory = & CalendarFactory::GetFactory();

    			$day = & $factory->newDay(
					array(
						"date" => date("Y-m-d", $dayTime)
					)
				);

    			echoDebug("new day : ".$day->date);

    			$days[] = $day;
			}
			$this->periods[] = $days;
		}
	}
	
	function setSpecificDates($startDate, $endDate) {
		$this->setFullPeriod($startDate, $endDate);
		$this->type = CALENDAR_DATES;
	}
	
	function setWeek($theDate) {
		global $XP_WEEK_FORMAT;
		$this->type = CALENDAR_WEEK; 
		$theTime = strtotime($theDate);
		$weekDay = date("D", $theTime);
    	
    	// Get the real day id
		$weekDay = array_search($weekDay, $XP_WEEK_FORMAT);
    	
    	$weekDay = $weekDay * ONEDAY;
    	
		// Begining of week
		$start = date("Y-m-d", $theTime - $weekDay);
		
		// End of week
		$end = date("Y-m-d", $theTime - $weekDay + 6 * ONEDAY);
		
		$this->addPeriod($start, $end);
		
		$this->theTime = strtotime($start);
		XPCalendarItem::setCalStart($start);
		XPCalendarItem::setCalEnd($end);
		
		return true;
	}
	
	function setFullPeriod($startDate, $endDate) {
		global $XP_WEEK_FORMAT;
		$startTime = strtotime($startDate);
		$endTime = strtotime($endDate);
		// Get the real day id
		$weekStartDay = array_search(date("D", $startTime), $XP_WEEK_FORMAT);
		// Back to first day of week...
		$startWeek = date("Y-m-d", $startTime - $weekStartDay * ONEDAY);
		
		// Get the real day id
		$weekEndDay = array_search(date("D", $endTime), $XP_WEEK_FORMAT);
		
		// To last day of week...
		$endWeek = date("Y-m-d", $endTime + (6 - $weekEndDay) * ONEDAY);
		
		
		$aWeek = 7 * ONEDAY;
		for ($date = $startWeek; $date < $endWeek;
			$date = date("Y-m-d", strtotime($date) + $aWeek)) {
			$this->setWeek($date);
		}
		XPCalendarItem::setCalStart($startDate);
		XPCalendarItem::setCalEnd($endDate);
	}
	
	function setMonth($theDate) {
		global $XP_WEEK_FORMAT;

		$theTime = strtotime($theDate);
		
		// First day of month
		$startMonth = date("Y-m-d", mktime(0, 0, 0, date("m", $theTime), 1, date("Y", $theTime)));
		
		// Last day of month
		$endMonth = date("Y-m-d", mktime(0, 0, 0, date("m", $theTime) + 1, 0, date("Y", $theTime)));
		
		$this->setFullPeriod($startMonth, $endMonth);
		
		// At the end because we called setWeek earlier...
		$this->type = CALENDAR_MONTH;
		$this->theTime = strtotime($startMonth);
		
		return true;
	}
	
	function setIteration($iterationId = FAKEID_CURRENT_ITERATION) {

		$this->controler->dbPopulateIterations();

		// Get current iteration date
		for ($j = 0; $j < sizeOf($this->multiIncludedItems["iteration"]); $j ++) {

			$iteration = & $this->multiIncludedItems["iteration"][$j];

			if (
				($iteration->id == $iterationId) // the one we're looking for
				||
				($iterationId == FAKEID_CURRENT_ITERATION && $iteration->isCurrent) // the current iteration
				||
				$iterationId == FAKEID_LATEST_ITERATION // the first one
				) {

				$this->iterationId = $iteration->id;

				$endDate = $iteration->deadlineDate;
				
				$periodOk = false;
				
				// If exists, previous date is date from previous iteration
				while (!$periodOk && $j < sizeOf($this->multiIncludedItems["iteration"]) - 1) {
					// Is not first iteration...
					// Start date = end date of previous iteration...
					$startDate = date("Y-m-d",
						strtotime($this->multiIncludedItems["iteration"][++$j]->deadlineDate));
					// If is same date, we won't see anything!
					if ($startDate != $endDate) {
						// + ONE DAY (iteration starting the day after the previous
						// one ended)
						$startDate = date("Y-m-d",
    						strtotime($startDate)
    						+ (ONEDAY));
						$periodOk = true;
					}
				}
				if (!$periodOk) {
					// else : start date = two weeks before...
					$startDate = date("Y-m-d",
						strtotime($endDate)
						- (ONEDAY * 14));
				}
        		
        		$this->setFullPeriod($startDate, $endDate);
				
        		$this->type = CALENDAR_ITERATION;
        		$this->theTime = strtotime($startDate);
        		unset($this->multiIncludedItems["iteration"]);
				return true;
			}
		}
		
		// At this point we didn't find any current iteration
		// The latest iteration will be taken, if any
		
		if (sizeOf($this->multiIncludedItems["iteration"]) > 0) {
			return $this->setIteration(FAKEID_LATEST_ITERATION);
		}
		
		// The iteration wasn't found
		return false;
	}

	function getComplexStatus($day, $status) {
		$isFirstDay = $status == ON_DAY_WORKING_FIRST
			|| $status == ON_DAY_WORKING_ONEDAY;
		$isBegining = ($day->date <= $this->calStart)
			|| $isFirstDay;
		$isLastDay = $status == ON_DAY_WORKING_LAST
			|| $status == ON_DAY_WORKING_ONEDAY;
		$isFinished = ($day->date >= $this->calEnd)
			|| $isLastDay;
		return array(
			$isFirstDay,
			$isBegining,
			$isLastDay,
			$isFinished
		);
	}
		
	function organizeDays() {

		if (!isset($this->multiIncludedItems["task"])) {
			$this->multiIncludedItems["task"] = array();
		} else {
			reset($this->multiIncludedItems["task"]);
		}

		for($iTask = 0; $iTask < sizeof($this->multiIncludedItems["task"]); $iTask ++) {
			$task = & $this->multiIncludedItems["task"][$iTask];

			$taskDone = false;
			echoDebug("TASK ".$task->id);
			reset($this->periods);

			for ($i = 0; $i < sizeof($this->periods) && !$taskDone; $i ++) {
    			$days = & $this->periods[$i];

				//echoDebug("Period");

    			if ($task->startTime > $days[sizeof($days)-1]->time) {
    				// the task starts in next period
    				continue;
    			}

				echoDebug(sizeof($days)." days");

    			for ($j = 0; $j < sizeof($days); $j ++) {
    				$day = & $days[$j];

					echoDebug("Day[$j] : ".$day->time);

    				$status = $task->dayStatus($day);

    				list(
						$isFirstDay,
						$isBegining,
						$isLastDay,
						$isFinished) = $this->getComplexStatus($day, $status);
    				
    				if ($day->date >= $this->calStart
    					&& $day->date >= $task->startDate) {

    					if ($isBegining) {

							//echoDebug("is begining of task $task->id");
							
    						$taskIndex = $day->getNewIndex();
    					}

    					if ($day->date <= $this->calEnd) {
        					//echo "set : ".$day->date."[$taskIndex] = ".$task->id."<br/>";
							if (!isset($taskIndex)) {

								echo "ERROR : day ".$day->date." task ".$task->id."<br/>";

							} else {

	        					$day->setAtIndex($taskIndex, $task);

							}
    					}

    					if ($task->endDate == $day->date
    						|| $day->date == $this->calEnd) {
							
							echoDebug("BREAK ".$task->endDate." == ".$day->date);

    						$taskDone = true;
    						break;
    					}
    				}
    			}
    		}
		}
		
		if (!isset($this->multiIncludedItems["iteration"])) {
			$this->multiIncludedItems["iteration"] = array();
		} else {
			reset($this->multiIncludedItems["iteration"]);
		}

		for($iIte = 0; $iIte < sizeof($this->multiIncludedItems["iteration"]); $iIte ++) {
			$iteration = & $this->multiIncludedItems["iteration"][$iIte];

			$iteDone = false;
			reset($this->periods);
			for ($i = 0; $i < sizeof($this->periods) && !$iteDone; $i ++) {
    			$days = & $this->periods[$i];
    			if ($iteration->deadlineTime > $days[sizeof($days)-1]->time) {
    				// the iteration ends in next period
    				continue;
    			}
    			for ($j = 0; $j < sizeof($days); $j ++) {
    				$day = & $days[$j];
    				if ($day->date >= $this->calStart
    					&& $day->date <= $this->calEnd
    					&& $day->date == $iteration->deadlineDate) {
    					$iteIndex = $day->getDoubleNewIndex();
    					//echo "setIte : ".$day->date."[$iteIndex] = ".$iteration->id."<br/>";
    					$day->setAtIndex($iteIndex, $iteration);
    					
        				// ***** BECAUSE ROWSPAN = 2 ************
        				// => so the object will be from neither type task or iteration
						//$iteIndex = $day->getNewIndex();
						$zoub = "zoub";
    					$day->setAtIndex($iteIndex + 1, $zoub);
    					$iteDone = true;
    					break;
    				}
    			}
    		}
		}
    }
}

// *********************************************************************
// Class XPCalIteration
// *********************************************************************
class XPCalIteration extends Iteration {
	var $calStart;
	var $calEnd;
	
}

// *********************************************************************
// Class XPCalStory
// *********************************************************************
class XPCalStory extends Story {
	var $calStart;
	var $calEnd;
		
	
}

// *********************************************************************
// Class XPCalTask
// *********************************************************************
class XPCalTask extends Task {
	var $calStart = "9999-99-99";
	var $calEnd = "0000-00-00";
	
	function dayStatus(& $day) {

		$time = $day->time;

        if ($time >= $this->startTime &&
			$time <= $this->endTime) {
			
			// Is on this day
			$weekDay = date("D",$time);

			if ($this->workingDays[$weekDay]
				&& !$day->isHoliday) {

				if ($time == $this->startTime && $time == $this->endTime) {
					//One day task
					return ON_DAY_WORKING_ONEDAY;

				} elseif ($time == $this->startTime) {
					// First day of the task
					return ON_DAY_WORKING_FIRST;

				} elseif ($time == $this->endTime) {
					// Last day of the task
					return ON_DAY_WORKING_LAST;

				} else {
					// A day in the task
					return ON_DAY_WORKING;
				}

			} else {
				// A day in the task, but not working
				return ON_DAY_NOT_WORKING;
			}
		} else {
			// Not a day for this task
			return NOT_ON_DAY;
		}
	}
}

// *********************************************************************
// Class XPCalDay
// *********************************************************************
class XPCalDay {
	var $date;
	var $time;
	var $weekDay;
	var $indexedObjects = array();
	var $isHoliday = false;
	var $isToday = false;
	
	function XPCalDay($arguments) {
		global $XP_EVERY_YEAR_HOLIDAYS, $XP_SPECIFIC_HOLIDAYS;
		
		$date 		= $arguments["date"];
		
		$this->date = $date;
		$this->time = strtotime($date);
		$this->weekDay = date("D",$this->time);
		if (isSet($XP_EVERY_YEAR_HOLIDAYS[date("m-d", $this->time)])
			|| isSet($XP_SPECIFIC_HOLIDAYS[$this->date])) {
			$this->isHoliday = true;
		}
		
		if ($this->date == date("Y-m-d")) {
			$this->isToday = true;
		}
	}
	
	function getNewIndex() {
		global $gMaxIndex;
		
		for ($i = 0; isset($this->indexedObjects[$i]); $i ++) {
			// Nothing
		}

		$gMaxIndex = @max($gMaxIndex, $i);

		return $i;
	}
	
	function getDoubleNewIndex() {
		global $gMaxIndex;

		for ($i = 0; isset($this->indexedObjects[$i]) || isset($this->indexedObjects[$i+1]); $i ++) {
			// Nothing
		}

		$gMaxIndex = @max($gMaxIndex, $i + 1);

		return $i;
	}
	
	function setAtIndex($index, & $object) {
		$this->indexedObjects[$index] = & $object;
	}
}

