<?php

function getMemberName($memberId) {
	if ($memberId == 0) {
		return getlang("nobody");
	}
	
	return $GLOBALS["gBasicMembers"][$memberId]->title;
}

class PlanningFactoryView extends PlanningFactory {

	function & newPlanning($arguments) {
		
		// Create the object
		$object = & parent::newPlanning($arguments);

		// Create the associated view
		new PlanningView($object);

		return $object;
	}

	function & newTask($arguments) {

		// Create the object
		$object = & parent::newTask($arguments);

		// Create the associated view
		new TaskView($object);

		return $object;
	}
	
	function & newStory($arguments) {

		// Create the object
		$object = & parent::newStory($arguments);

		// Create the associated view
		new StoryView($object);

		return $object;
	}
	
	function & newIteration($arguments) {
		
		// Create the object
		$object = & parent::newIteration($arguments);

		// Create the associated view
		new IterationView($object);

		return $object;
	}
	
}

$GLOBALS["gPlanningFactory"] = & new PlanningFactoryView();

// *********************************************************************
// Class PlanningCardView
// *********************************************************************
class PlanningCardView extends XPItemView {

	function getMiniVisibility() {

		if (XP_DISPLAY_LIGHT_PLANNING
			|| ($this->item->actualCompletion == 100
			&& !XPItem::class_itemWasModified($this->item->type, $this->item->id)
			&& (!isset($this->item->containsModified) || !$this->item->containsModified))) {
			
			return XP_VISIBLE;

		}

		return XP_INVISIBLE;
	}

	function miniDisplayStart() {
		parent::miniDisplayStart(PlanningCardView::getMiniVisibility());
	}

	function miniDisplayEnd() {
		parent::miniDisplayEnd(PlanningCardView::getMiniVisibility());
	}

	function displayMiniCustom() {

		if ($this->item->actualCompletion >= 100) {
			$bgcolor = "#eeeeee";
		} else {
			$bgcolor = getColorFromCompletion($this->item->theoryCompletion, $this->item->actualCompletion);
		}
		
		$this->displayMiniRight("background-color: $bgcolor; width: 100px;");

	}

	function getTheoryCompletionText() {
		global $XP_COMPLETION_DO_PASS_HUNDRED;

		$theoryLimitCompletion = min(100, $this->item->theoryCompletion);
	    
		// Display either exact completion or limited to 100 depending on config
		if ($XP_COMPLETION_DO_PASS_HUNDRED) {
    		$completionText = $this->item->theoryCompletion;
		} else {
    		$completionText = $theoryLimitCompletion;
		}

		return $completionText;

	}


	function displayMiniRightCustom() {
		
		$theoryDisplay = "<span class=\"smallComp\"> [".PlanningCardView::getTheoryCompletionText()."".htmlspecialchars("%")."]</span>";
		
		if ($this->item->actualCompletion >= 100) {

			$completionText = getLang("done").$theoryDisplay;

		} elseif ($this->item->theoryCompletion == 0 && $this->item->actualCompletion == 0) {

			$completionText = "<span class=\"smallComp\">".getLang("not_started")."</span>";

		} else {

			$completionText = $this->item->actualCompletion." ".htmlspecialchars("%").$theoryDisplay;

		}
		
		if ($this->item->actualCompletion >= 100) {
			$color = getColorFromCompletion($this->item->theoryCompletion, $this->item->actualCompletion);
			$completionText = "<span style=\"color: $color;\">".$completionText."</span>";
		}
        ?><span class="miniCompletion">
			<?=$completionText?>
		</span>
		<?php
	}

	function displayChooseDate($weekDay, $dateDay, $dateMonth, $dateYear, $endDate = false) {
		
		$dateType = $endDate?"end":"start";
		$dateName = $dateType."Date";
		$updateDate = "'doNothing();'";
		$extraAttributs = "";
		$afterMod = "";
		$id1 = "";
		$id2 = "";
	
		$formName = $this->getFormName();

		if ($this->item->type == "task") {
			if ($endDate) {
				// If user changes end date, reset "to update" button
				$updateDate = "'resEDToUpd(\'$formName\');'";
				$id1 = "id=\"$formName"."_min\"";
				$id2 = "id=\"$formName"."_plus\"";
			} else {
				// This is start date from a task => might change end date
				$afterMod = "sigEDToUpd('$formName');";
				$updateDate = "'sigEDToUpd(\'$formName\');'";
			}
		}
		
		$value = displayDate($weekDay, $dateDay, $dateMonth, $dateYear);
		
		?><button class="less" type="button" <?=$id1?> title="&lt;" onclick="<?=$dateType?>Dec(this);<?=$afterMod?>">&nbsp;</button>
		<input type="text" name="<?=$dateName?>" value="<?=$value?>" size="15" readonly="readonly" onclick="popupCalendar(this, <?=$updateDate?>);" <?=$extraAttributs?>/>
		<button class="more" type="button" <?=$id2?> title="&gt;" onclick="<?=$dateType?>Inc(this);<?=$afterMod?>">&nbsp;</button><?php
	}


	function displayCompletion() {
	
		displayDivStart("completion");
		
		$bgcolor = getColorFromCompletion($this->item->theoryCompletion, $this->item->actualCompletion);
	    
		$completionText = PlanningCardView::getTheoryCompletionText();
	    
		// For array size
		$theoryCompletion = min(100, $this->item->theoryCompletion);
		$theoryCompletion = max($theoryCompletion, 0);
	    	    
		if ($this->item->actualCompletion == 0 && $theoryCompletion == 0) {

    		echoDiv("compNotStarted", getLang("not_started"));

		} else {

			if ($theoryCompletion >= $this->item->actualCompletion) {
				// Theory >>>> actual

    			$firstDiv	= "actualComp";
				$firstWidth	= $this->item->actualCompletion;
				$firstText	= $this->item->actualCompletion."&nbsp;".htmlspecialchars("%");
				$firstStyle = "border-color: $bgcolor; background-color: $bgcolor;";

				$secondDiv		= "theoryComp";
				$secondWidth	= $theoryCompletion - $this->item->actualCompletion;
				$secondText		= $completionText."&nbsp;".htmlspecialchars("%");
				$secondStyle	= "";

			} else {
				// Actual >>>> theory

				$firstDiv	= "theoryComp";
				$firstWidth	= $theoryCompletion;
				$firstText	= $completionText."&nbsp;".htmlspecialchars("%");
				$firstStyle = "background-color: ".PlanningCardView::brightenColor($bgcolor).";";
				$firstStyle .= "color: ".PlanningCardView::darkenColor($bgcolor)."; border-color: $bgcolor;";

				$secondDiv		= "actualComp";
				$secondWidth	= $this->item->actualCompletion - $theoryCompletion;
				$secondText		= $this->item->actualCompletion."&nbsp;".htmlspecialchars("%");
				$secondStyle	= "border-color: $bgcolor; background-color: $bgcolor;";

			}

			$endWitdh = 0;

			if ($firstWidth == 100) {

				$secondWidth = 0;

				$firstWidth -= 3;

				$firstText .= "&nbsp;<span class=\"smallComp finishedComp\">[$secondText]</span>";

			} else {

				if($firstWidth <= 5) {

					$zoub = 5 - $firstWidth;

					$firstWidth += $zoub;

					$secondWidth	-= $zoub + 2;
					
				} elseif ($secondWidth == (100 - $firstWidth)) {

					$secondWidth	-= 4;

				}

				if ($secondWidth <= 5) {

					$zoub = 5 - $secondWidth;

					$secondWidth += $zoub;

					if ($firstWidth > 5) {
						$firstWidth -= $zoub;
					}
				}


				$endWitdh = 100 - $firstWidth - $secondWidth - 3;
			}

			?>
			<div class="firstComp <?=$firstDiv?>"	style="width: <?=$firstWidth?>%; <?=$firstStyle?>"><?=$firstText?></div><?php
			if ($secondWidth > 0) {
				?><div class="secondComp <?=$secondDiv?>"	style="width: <?=$secondWidth?>%; <?=$secondStyle?>"><?=$secondText?></div><?php
			}
			if ($endWitdh >= 5) {
				?><div class="endComp" style="width: <?=$endWitdh?>%; border-left-width: 0;">&nbsp;</div><?php
			}
		}
		displayDivEnd();
	}

	function darkenColor($color) {

		$increase = 20;
		$red =		max(0, HexDec(substr($color,1,2)) - $increase);
		$green =	max(0, HexDec(substr($color,3,2)) - $increase);
		$blue = 	max(0, HexDec(substr($color,5,2)) - $increase);

		return "#".($red<10?"0":"").DecHex($red).
			($green<10?"0":"").DecHex($green).
			($blue<10?"0":"").DecHex($blue);

	}

	function brightenColor($color) {

		$increase = 35;
		$red =		min(255, HexDec(substr($color,1,2)) + $increase + 15);
		$green =	min(255, HexDec(substr($color,3,2)) + $increase);
		$blue = 	min(255, HexDec(substr($color,5,2)) + $increase + 15);

		return "#".($red<10?"0":"").DecHex($red).
			($green<10?"0":"").DecHex($green).
			($blue<10?"0":"").DecHex($blue);

	}

	function displayModifyWorkingDays($comment = "") {
		global $XP_WEEK_FORMAT;

		displayDivStart("workingDays");

		if(!empty($comment)) {
			echoSpan("comment", $comment);
		}
		
		// "mod" like "modification"
		// the modification div that contains all the input fields...
		$formName = $this->getFormName();

		for ($i=0; $i < sizeOf($XP_WEEK_FORMAT); $i ++) {
			$dayToCheck = $XP_WEEK_FORMAT[$i];
			$tdName = $dayToCheck."_".$this->item->type."_".$this->item->id;
			
			$value = $this->item->workingDays[$dayToCheck];
			
			if($value == 1) {
				$class = "wd";
			} else {
				$class = "nWd";
			}
			
			if ($this->item->type == "task") {
				$script = "tskwd(this, '$formName', '$dayToCheck');";
			} else {
				$script = "wd(this, '$formName', '$dayToCheck');";
			}
			$aId = $this->item->type."_".$this->item->id."_".$dayToCheck;
			?>
			<a id="<?=$aId?>" class="<?=$class?>" onclick="<?=$script?> return false;" href="javascript:doNothing();"><?=getLang($dayToCheck)?></a>
			<?php
		}
		
		displayDivEnd();
		
		PlanningCardView::displayHiddenWorkingDays();
	}
	
	function displayHiddenWorkingDays() {
		global $XP_WEEK_FORMAT;
		
		$workingPace = "";

		for ($i=0; $i < sizeOf($XP_WEEK_FORMAT); $i ++) {
			$dayToCheck = $XP_WEEK_FORMAT[$i];
			
			if($this->item->workingDays[$dayToCheck] == 1) {
				$class = "wd";
				$workingPace .= "|$dayToCheck|";
			}
			
		}
		
		// Single input field holding working days...
		displaySimpleInputHidden("wp", $workingPace);
	}

	function displayWorkingDays($text) {
		global $XP_WEEK_FORMAT;

		displayDivStart("workingDays");
		
		if ($text != "") {
			echo forceNbSpaces($text);
		}

		for ($i = 0; $i < sizeOf($XP_WEEK_FORMAT); $i ++) {

			$dayToCheck = & $XP_WEEK_FORMAT[$i];
			$value = & $this->item->workingDays[$dayToCheck];
			
			if($value == 1) {
				$class = "wd";
			} else {
				$class = "nWd";
			}

			echoSpan($class, getLang($dayToCheck));
		}
		
		displayDivEnd();
		
	}




}

// *********************************************************************
// Class TaskView
// *********************************************************************
class TaskView extends PlanningCardView {

	function displayMiniTitle($title, $titleClass) {

		$this->getWarningText($title);

		parent::displayMiniTitle($title, $titleClass);
	}

	function getWarningText(& $text) {

		if ($this->item->outOfTime > 0) {
			// Is out of time

			if ($this->item->outOfTime == 1) {
				$days = getLangNbsp("one_day");
			} else {
				$days = getLangNbsp("some_days", $this->item->outOfTime);
			}
			
			$title = getLang("ending")." ".$days." ".getLang("after_deadl");

			$text = "<span class=\"notOnTime\" title=\"$title\">$text</span>";

		}

		return $text;
	}


	function displayFields($fieldsVisibility = XP_VISIBLE) {
		
		$this->displayContentStart($fieldsVisibility);
		
		displayDivStart("leftHalf");
		
			displayDivStart("title");
			echo getHtmlDbText($this->item->title);
			displayDivEnd();

			displayDivStart("description");
			displayDescription($this->item->description);
			displayDivEnd();
	
		displayDivEnd();

		displayDivStart("rightHalf");

			if (!$this->item->isDefaultWorkingDays) {
				PlanningCardView::displayWorkingDays("");
			} else {
				displayDefaultWorkingDays();
			}

			displayDivStart("dateTask");

			displayDivStart("leftHalf");
			
			echo displayDate(
				$this->item->startWeekDay,
				$this->item->startDateDay,
				$this->item->startDateMonth,
				$this->item->startDateYear);

			displayDivEnd();

			displayDivStart("rightHalf");
							
			if ($this->item->weight != 0 || !$this->item->autoUpdate) {

				if (!$this->item->autoUpdate) {
					displaySpanStart("custom");
				}

				echo displayDate(
					$this->item->endWeekDay,
					$this->item->endDateDay,
					$this->item->endDateMonth,
					$this->item->endDateYear);

				if (!$this->item->autoUpdate) {
					displaySpanEnd();
				}

			} else {
				echoSpan("low", "?");
			}

			displayDivEnd();

			displayDivEnd();

			displayDivStart("weight");

				if ($this->item->weight == 0) {
					echoLang("not_eval");
				} elseif ($this->item->weight == 1) {
					echoLang("eval_one");
				} elseif ($this->item->weight == -1) {
					echoLang("eval_less_one");
				} else {
					echoLang("eval_more_one", "<span class=\"value\">".round($this->item->weight, 2)."</span>");
				}

			displayDivEnd();

		displayDivEnd();

		

		displayDivStart("responsible");

		displayDivStart("leftHalf");
		echo getLang("resp")." ";
		if ($this->item->memberId == 0) {
			echoSpan("error","[".getLang("no_resp")."]");
		} else {
			echo getMemberName($this->item->memberId);
		}
		displayDivEnd();

		displayDivStart("rightHalf");
		if ($this->item->pairId != 0) {
			echo getLang("pair_with")." ";
			echo getMemberName($this->item->pairId);
		}
		displayDivEnd();

		displayDivEnd();
		
		PlanningCardView::displayCompletion();
		
		if ($this->item->outOfTime > 0) {
			if ($this->item->outOfTime == 1) {
				$days = getLangNbsp("one_day");
			} else {
				$days = getLangNbsp("some_days", $this->item->outOfTime);
			}
			displayDivStart("notOnTime");
			echoSpan("bigError", getLang("warn"));
			echo " ".getLang("ending")." ";
			echoSpan("bigError", $days);
			echo " ".getLang("after_deadl");
			displayDivEnd();
		}
		
		$this->displayContentEnd($fieldsVisibility);
	}
	
	function displayModifyContent() {
		
		$formName = $this->getFormName();
		
		displayInputHidden("projectId", $this->item->projectId);
		displayInputHidden("weight", $this->item->weight);
		displayInputHidden("loadFactor", $this->item->loadFactor);
		displayInputHidden("theoryEndCompletion", $this->item->theoryEndCompletion);
		displayInputText("title", "title", $this->item->title, getLang("title"));
		$this->displayModifyDescription();
		
		?>
		
		
		<?=modifyLineStart()?>

		<?=getLang("resp_memb")?>

		<?=modifyLineMiddle()?>

			<table width="100%">
			<tr>

			<td>

			<?php
			if (($this->item->toInsert && $this->item->isAllowedToInsertAll()) || $this->item->isAllowedToUpdateAll()){
				// If allowed to modify responsible member
				?>
				<select onchange="changeUserTasks(this, 'pairId');" name="memberId" class="responsible">
					<option value="<?=$this->item->memberId?>" selected="selected">&nbsp;</option>
				</select>
				<?php
			} else {
				// If not allowed to modify responsible member
				displayForbiddenSelect("memberId", $this->item->memberId, getMemberName($this->item->memberId));
			}
			?>

			</td>
			<td>

			<?=getLang("pair_with")?>

			</td>
			<td>

			<select onchange="checkSelect(this);" name="pairId" class="responsible">
				<option value="<?=$this->item->pairId?>" selected="selected">&nbsp;</option>
			</select>

			</td>

			</tr>
			</table>

		<?=modifyLineEnd()?>
		<?=modifyLineStart()?>

				<?=getLang("work_on")?>
				<?=linkToHelp("Planning", "workingPace", getLang("work_on"))?>

		<?=modifyLineMiddle()?>

				<?php
				$defaultName = "wtD".$this->item->id;
				$text = "<input type=\"radio\" id=\"$defaultName\" name=\"wkType\" value=\"default\"";

				if ($this->item->isDefaultWorkingDays) {
					$text .= " checked=\"checked\"";
				}

				$text .= " onclick=\"dfWd(this);\"/>";
				$text .= "<label for=\"$defaultName\">".getLang("default")."</label><br />";
				
				$customName = "wtC".$this->item->id;
				$text .= "<input type=\"radio\" id=\"$customName\" name=\"wkType\" value=\"custom\"";

				if (!$this->item->isDefaultWorkingDays) {
					$text .= " checked=\"checked\"";
				}
				$text .= " onclick=\"csWd(this, ".$this->item->iterationId.");\"/>";
				$text .= "<label for=\"$customName\">".getLang("custom")."</label>";
				
				PlanningCardView::displayModifyWorkingDays($text);
				?>
			
		<?=modifyLineEnd()?>
		
		<?=modifyLineStart()?>

				<?=getLang("progress")?>

				<?=linkToHelp("Planning", "doneToDo", getLang("progress"))?>

		<?=modifyLineMiddle()?>
				
				<table class="completion" border="0" width="100%" cellpadding="3" cellspacing="0">
				<tr>
					<td width="50%">
						<?php
						echoLang("work_starts");
						echo "<br/>";
						displayDateFormat();
						?>
					</td>
					<td width="50%">
        				<?php

        				// Starting date...
        				PlanningCardView::displayChooseDate(
							$this->item->startWeekDay,
							$this->item->startDateDay,
							$this->item->startDateMonth,
							$this->item->startDateYear);
        				?>
					</td>
				</tr>
				<tr>
					<td colspan="2">
    					<table class="taskHighlight" border="0" width="100%" cellpadding="3" cellspacing="0">
							<tr>
            				<?php
            				$this->displayChooseDoneTodo("done");
            				?>
							<td>
								<button title="&lt;=" type="button" class="moveDay" onclick="mvToDone(this);">&nbsp;</button>
							</td>
							<?php
            				$this->displayChooseDoneTodo("todo");

                			$realValue = $this->item->todo * $this->item->loadFactor;
                			?>
							</tr>
							<tr>
								<td colspan="7" align="right">
                    			<?php displaySpanStart("modifyDate"); ?>
                    			(<span id="<?=$formName?>_realTodo"><?=$realValue?></span>&nbsp;<?=getLang("real_days")?>)
                    			<?php displaySpanEnd(); ?>
                    			</td>
							</tr>
    					</table>
					</td>
				</tr>
				<tr>
					<td>
						<?php
						echoLang("work_ends");
						$calcId = $formName."_auto";
						?>
						<input onclick="clickAuto('<?=$formName?>');"
							type="checkbox" <?=htmlChecked($this->item->autoUpdate)?> id="<?=$calcId?>" name="auto" value="1"/><label for="<?=$calcId?>" class="modifyDate"><?=getLang("date_auto_update")?></label>
						<br/>
						<?php
						displayDateFormat();
						?>
					</td>
					<td valign="middle">
        				<?php
        				// End date
        				PlanningCardView::displayChooseDate($this->item->endWeekDay, $this->item->endDateDay, $this->item->endDateMonth, $this->item->endDateYear, true);
        				?>&nbsp;<button onclick="clickCalculateEndDate('<?=$formName?>');"
							id="<?=$formName?>_calcEnd"
							type="button"
							class="autoEnd"
							title="<?=getLang("click_calculated_value")?>">&nbsp;</button>
					</td>
				</tr>
				</table>
			
		<?=modifyLineEnd()?>
		<?php
		if ($this->item->toInsert) {
			// insertion => fixed position...
			displayInputHidden("iterationStoryId", $this->item->iterationId."_".$this->item->storyId);
		} else {
			?>
			
			<?=modifyLineStart()?>
	
				<?=getLang("move_to")?>
	
			<?=modifyLineMiddle()?>
	
				<?php
				displayIterationStorySelect($this->item->iterationId, $this->item->storyId);
				?>
	
			<?=modifyLineEnd()?>
			
			<?php
		}
		
	}
	
	function getShowJSScript() {
		$jsIsDefault = ($this->item->isDefaultWorkingDays)?"true":"false";
		$customScript = "focTask('".$this->getFormName()."', $jsIsDefault);";
		return $customScript;
	}

	function getCancelJSScript() {
		return "onModifyTask = false;";
	}
	
	function displayChooseDoneTodo($doneOrTodo) {
		$inputId = $doneOrTodo."_".$this->item->id;
		if ($doneOrTodo == "done") {
			$days = getLang("real_days");
		} elseif ($doneOrTodo == "todo") {
			$days = getLang("ideal_days");
		}
		?>
			<td width="90" valign="middle">
			<?php
			echoLang("task_".$doneOrTodo);
			echoDiv("modifyDate","$days");
			?>
			</td>
			<td valign="middle" align="right">
				<input onblur="bl(this);" type="text" id="<?=$inputId?>" name="<?=$doneOrTodo?>" value="<?=$this->item->$doneOrTodo?>" size="4" />
			</td>
			<td valign="middle" align="left" width="15"><button type="button" title="+" class="inc" onclick="inc(this, '<?=$doneOrTodo?>');">&nbsp;</button><br/>
<button title="-" type="button" class="dec" onclick="dec(this, '<?=$doneOrTodo?>');">&nbsp;</button></td>
		<?php
	}
}
// *********************************************************************
// Class TaskContainerView
// *********************************************************************
class TaskContainerView extends XPItemWithIncludedView  {

	function displayMiniTitle($title, $titleClass) {

		$this->getWarningText($title);

		parent::displayMiniTitle($title, $titleClass);
	}

	function getWarningText(& $text) {
		return $this->getWarningTextParam($text, $this->item->outOfDateTasks);
	}

	function getWarningTextParam(& $text, $outOfDate) {

		if ($outOfDate > 0) {
			// Contains out of date tasks

			if ($outOfDate == 1) {
				$days = getLang("one_day");
			} else {
				$days = getLang("some_days", $outOfDate);
			}

			$title = getLang("contains_task_end")." ".$days." ".getLang("after_deadl2");

			$text = "<span class=\"notOnTime\" title=\"$title\">$text</span>";

		}

		return $text;
	}

	function getIncludedComment() {
		$nbTasks = sizeOf($this->item->multiIncludedItems["task"]);
		
		$dispTasks = "";
		
		if ($nbTasks > 1) {
			$dispTasks = getLangNbsp("nb_tasks",$nbTasks);
		} else if ($nbTasks == 1) {
			$dispTasks = getLangNbsp("one_task");
		}

		$this->getWarningTextParam($dispTasks, $this->item->outOfDateTasks);

		if (!$this->item->isAllTasks()) {
			return "";
		}

		return $dispTasks;
	}

	function getFromMemberLegend() {
		
		if (!$this->item->isAllTasks()) {
			return "[".getUsersTasksDisplayName($this->item->nTasksResponsibleId, $this->item->nTasksPairId, true)."]";
		}
		return "";
		
    }
	
	function displayIntro($type) {
		if ($type == "task") {
			
			if (!$this->item->isAllTasks()) {

				echoDiv("includedSection", getUsersTasksDisplayName($this->item->nTasksResponsibleId, $this->item->nTasksPairId));

			} else {

				parent::displayIntro($type);

			}

		} else {
			parent::displayIntro($type);
		}
	}

	
	function isShowModifyDeleteLinks() {
		return parent::isShowModifyDeleteLinks() && $this->item->isAllTasks();
	}
	

	function isOpen() {
		return parent::isOpen() || !$this->item->isAllTasks();
	}

	function allowedToDisplayInType($item, $type) {

		if ($type == "task") {

			if (!$this->item->isAllResponsible()) {
			
				return $item->isFromMember($this->item->nTasksResponsibleId);
			}

			if (!$this->item->isAllPair()) {
			
				return $item->isFromPair($this->item->nTasksPairId);
			}
			
		}
		
		return parent::allowedToDisplayInType($item, $type);

	}

}

// *********************************************************************
// Class TaskAndStoryContainerView
// *********************************************************************
class TaskAndStoryContainerView extends TaskContainerView {
	var $bIsInitDisplayed = false;
	
	function displayOpenCloseAll($incType) {
		switch ($incType) {
			case "iteration":
			case "story":
				return true;
				break;
			default:
				return false;
				break;
		}
	}
	
	function getWarningText(& $text) {
		return $this->getWarningTextParam($text, max($this->item->outOfDateStories, $this->item->outOfDateTasks));
	}

	function displayAddIncluded($type) {
		
		if ($type == "task" && !$this->bIsInitDisplayed) {
			// This is a little bit dirty
			// Form inputs for inserting a task => to update the working days
			// and load factor of the task to insert
			$iteInfoFormName = $this->getFormName()."_info";
			
			displaySimpleFormStart($iteInfoFormName);
			PlanningCardView::displayHiddenWorkingDays();
			displaySimpleInputHidden("loadFactor", $this->item->loadFactor);
			displaySimpleFormEnd($iteInfoFormName);
			
			$this->bIsInitDisplayed = true;
		}
		
		if ($type == "task" || $this->item->isAllTasks()) {
			// You can always insert a task, 
			parent::displayAddIncluded($type);
		}
	}

	function getIncludedComment() {

		$dispTasks = parent::getIncludedComment();

		$nbStories = sizeOf($this->item->multiIncludedItems["story"]);
		
		$dispStories = "";

		if (!$this->item->isAllTasks()) {
			return "";
		}
		
		if ($nbStories > 1) {
			$dispStories = getLangNbsp("nb_stories",$nbStories)."<br/>";
		} else if ($nbStories == 1) {
			$dispStories = getLangNbsp("one_story")."<br/>";
		}

		$this->getWarningTextParam($dispStories, $this->item->outOfDateStories);

		$comment = $dispStories.$dispTasks;

		return $comment;
	}

	function displayIntro($type) {

		if ($type == "task") {
			
			parent::displayIntro($type);

		} elseif ($type == "story") {

			if ($this->item->isAllTasks()) {

				echoDiv("includedSection", ucfirst(getLang("stories")));

			}
			
		}
	}

	function allowedToDisplayInType($item, $type) {

		if ($type == "story" && !$this->item->isAllTasks()) {
			
			return $item->containsTaskWithRespAndPair($this->item->nTasksResponsibleId, $this->item->nTasksPairId);
			
		}
		
		return parent::allowedToDisplayInType($item, $type);

	}

}

// *********************************************************************
// Class StoryView
// *********************************************************************
class StoryView extends TaskContainerView {

	function getShowJSScript() {
		$customScript = "focStory('".$this->getFormName()."');";
		return $customScript;
	}

	function doDisplayAddIncluded() {
		displayInsert(array(
			"task",
			$this->item->iterationId,
			$this->item->id,
			$this->item->nTasksResponsibleId,
			$this->item->nTasksPairId));
	}

	function displayFields($fieldsVisibility = XP_VISIBLE) {

		$this->displayContentStart($fieldsVisibility);

		if ($this->item->description == "") {
			$description = getLang("no_desc");
		} else {
			$description = $this->item->description;
		}
		
		
		$nbTasks = sizeOf($this->item->includedItems);
		
		displayDivStart("leftHalf");
			displayDivStart("title");
			
			echo getHtmlDbText($this->item->title);

			displayDivEnd();    
		displayDivEnd();

		displayDivStart("rightHalf riskPriority");
			echoSpan("title", getLang("priority"));
			echoSpan("value", $this->item->priority);
			
			echoSpan("title", getLang("risk"));
			echoSpan("value", $this->item->risk);

			echoSpan("title", getLang("weight"));
			echoSpan("value", $this->item->weight);
		displayDivEnd();
		

		if ($this->item->isAllTasks()) {

			displayDivStart("leftHalf");
				if ($this->item->validationCriteria != "") {
					displayDivStart("descriptionTitle");
					echoLang("description");
					displayDivEnd();
				}
				displayDivStart("description");
				displayDescription($description);
				displayDivEnd();		
			displayDivEnd();
			
			
			displayDivStart("rightHalf");
				if ($this->item->validationCriteria != "") {
					displayDivStart("descriptionTitle validation");
					echoLang("valid_criteria");
					displayDivEnd();
					
					displayDivStart("description validation");
					echo displayDescription($this->item->validationCriteria);
					displayDivEnd();
					
				}
			displayDivEnd();
		}
		
		PlanningCardView::displayCompletion();
				
		$this->displayContentEnd($fieldsVisibility);
	}

	function displayModifyContent() {
		displayInputHidden("projectId", $this->item->projectId);
		
		displayInputText("title", "title", $this->item->title, getLang("title"));

		$this->displayModifyDescription();

		$this->displayModifyDescription("validationCriteria", "valid_criteria");
		
		?>
		<tr>
			<td colspan="2">
			<?php
			displayDivStart("riskPriority");
				echoSpan("title", getLang("priority"));
				?>
				<select name="priority" class="value">
				<option value="<?=$this->item->priority?>" selected="selected">&nbsp;</option>
				</select>
				<?php

				echoSpan("title", getLang("risk"));
				?>
				<select name="risk" class="value">
				<option value="<?=$this->item->risk?>" selected="selected">&nbsp;</option>
				</select>
				<?php

				echoSpan("title", getLang("weight"));
				?>
				<select name="weight" class="value">
				<option value="<?=$this->item->weight?>" selected="selected">&nbsp;</option>
				</select>
				<?php
			displayDivEnd();
			?>
		 </td>
		</tr>
		<?php
		if ($this->item->toInsert) {
		// insertion
			displayInputHidden("iterationId", $this->item->iterationId);
		} else {
		?>
		<tr>
			<td align="left">
				<?=getLang("move_to")?>
			</td>
			<td align="left">
		<?php
		
		// Iteration id...
		displayIterationSelect($this->item->iterationId);
		?>
			</td>
		</tr>
		
		<?php
		}
	}
	
	function displayDeleteConfirm() {
		if (sizeOf($this->item->includedItems) == 0) {
			// No choice if there is no task included...
			?>
			<input type="hidden" name="whatTasks" value="delete" />
			<?php
			$this->displayAreYouSure();
		} else {
			?>
			<input id="move" type="radio" name="whatTasks" value="move" checked="checked" />
			<label for="move"><?php
			echoLang("move_task");
			?>
			</label>
			<?php
			displayIterationStorySelect($this->item->iterationId, (-$this->item->id), /*true,*/ "this.form.whatTasks[0].checked = true;");
			?>
			<br />
			<input id="delete" type="radio" name="whatTasks" value="delete" />
			<label for="delete">
			<?php
			echoLang("del_task");
			?>
			</label>
		
			<?=XP_JAVASCRIPT_START?>
			focDelStory("<?=$this->getFormName()?>");
			<?=XP_JAVASCRIPT_END?>
			
			<?php
		}
	}
	
	

	/////////////////////////////////////////////////////////////////
	// The following needs to be copy / pasted from PlanningCard
	/////////////////////////////////////////////////////////////////
	function displayMiniRightCustom() {
		PlanningCardView::displayMiniRightCustom();
	}

	function miniDisplayStart() {
		PlanningCardView::miniDisplayStart();
	}

	function miniDisplayEnd() {
		PlanningCardView::miniDisplayEnd();
	}

	function displayMiniCustom() {
		PlanningCardView::displayMiniCustom();
	}
	/////////////////////////////////////////////////////////////////

	function displayNone($type) {
		displayNone(getLang("no_task_story"));
	}
}

// *********************************************************************
// Class IterationView
// *********************************************************************
class IterationView extends TaskAndStoryContainerView {

	function displayIntro($type) {
		if ($type == "task" && $this->item->isAllTasks()) {
			
			echoDiv("includedSection", ucfirst(getLang("tasks")));

		} else {

			parent::displayIntro($type);

		}
	}
	
	function doDisplayAddIncluded($type) {

		$arguments = array(
			$type,
			$this->item->id);

		switch ($type) {
			case "story":
				break;
			case "task":
				$arguments[] = 0;
				$arguments[] = $this->item->nTasksResponsibleId;
				$arguments[] = $this->item->nTasksPairId;
				break;
		}

		displayInsert($arguments);
	}

	function displayNone($type) {
		switch ($type) {
			case "story":
				displayNone(getLang("no_stories_iteration"));
				break;
			case "task":
				displayNone(getLang("no_isotask_iteration"));
				break;
		}
	}

	/// In wich order should the items be displayed...
	/// By default, the order in wich they appear in the array..
	function getIncludedDisplayOrder() {
		return array("story", "task");
	}

	/////////////////////////////////////////////////////////////////
	// The following needs to be copy / pasted from PlanningCard
	/////////////////////////////////////////////////////////////////
	function displayMiniRightCustom() {
		PlanningCardView::displayMiniRightCustom();
	}

	function miniDisplayStart() {
		PlanningCardView::miniDisplayStart();
	}

	function miniDisplayEnd() {
		PlanningCardView::miniDisplayEnd();
	}

	function displayMiniCustom() {
		PlanningCardView::displayMiniCustom();
	}
	/////////////////////////////////////////////////////////////////
	
	function displayFields() {
		
		$this->displayContentStart(XP_VISIBLE, "");
				
		displayDivStart("leftHalf");
			displayDivStart("title");
			echo getHtmlDbText($this->item->title);
			displayDivEnd();
			if ($this->item->isCurrent) {
				displayDivStart("subTitle");
				echoLang("curr_iteration");
				displayDivEnd();
			}

			if ($this->item->isAllTasks()) {
				/*displayDivStart("description");
				displayDescription($this->item->description);
				displayDivEnd();
				*/
				$this->displayDescription();
			}

		displayDivEnd();
		
		
		displayDivStart("rightHalf");

		PlanningCardView::displayWorkingDays(getLangNbsp("default_pace"));
        		
		if ($this->item->isAllTasks()) {
			displayDivStart("date");
			
			echo getLang("deadline")."&nbsp;";
			$date = displayDate($this->item->deadlineWeekDay, $this->item->deadlineDay, $this->item->deadlineMonth, $this->item->deadlineYear);
			
			$this->getWarningText($date);

			echo $date."<br/>";
			
			// necessary for display
			// To make sure they're initialized...
			$this->item->totalStoryWeightDone	+= 0;
			$this->item->totalStoryWeight		+= 0;
			$this->item->totalTaskWeight		= round($this->item->totalTaskWeight,2);
			
			echoSpan("loadFactor","x".$this->item->loadFactor);
			
			displaySpanStart("iterationWeight");
			echo "<br/>";
			echo getLang("story_points_1", $this->item->totalStoryWeightDone." / ".$this->item->totalStoryWeight);
			echo "<br/>";
			echo getLang("story_points_2", "".$this->item->totalTaskWeight);
			displaySpanEnd();

			displayDivEnd();

		}
		displayDivEnd();

		PlanningCardView::displayCompletion();
				
		$this->displayContentEnd();
	}
	
	function displayDeleteConfirm() {
		if (sizeOf($this->item->multiIncludedItems["story"]) == 0
			&& sizeOf($this->item->multiIncludedItems["task"]) == 0) {
			// No choice if there is no task or story included...
			?>
			<input type="hidden" name="whatStories" value="delete" />
			<?php
			$this->displayAreYouSure();
			return false;
		} else {
			?>
			<input type="radio" name="whatStories" value="move" checked="checked" id="deleteIte1"/>
			<label for="deleteIte1"> <?=getLang("move_stories")?></label>
			<?php
			displayIterationSelect(-$this->item->id, "iterationId", "smallInput", "this.form.whatStories[0].checked = true;");
			?>
			<br />
			<input type="radio" name="whatStories" value="delete" id="deleteIte2"/>
			<label for="deleteIte2"> <?=getLang("del_stories")?></label>
			
		
			<?=XP_JAVASCRIPT_START?>
			focDelIte("<?=$this->getFormName()?>");
			<?=XP_JAVASCRIPT_END?>
			
			<?php
			return true;
		}
	}

	function displayModifyContent() {
		
		displayInputText("title", "title", $this->item->title, getLang("title"));
		$this->displayModifyDescription();
		displayInputHidden("projectId", $this->item->projectId);
		?>
		<?=modifyLineStart()?>

				<?=getLang("deadline")?>
			
		<?=modifyLineMiddle()?>

				<table width="100%" cellpadding="0" cellspacing="0" border="0">
				<tr>
				<td>
					<?php
					// Date...
					displayDateFormat();
					echo "<br/>";
					PlanningCardView::displayChooseDate($this->item->deadlineWeekDay, $this->item->deadlineDay, $this->item->deadlineMonth, $this->item->deadlineYear);
					?>
				</td>
				<td align="right" class="member">
				<?=getLang("load_fact_expl_1")." "?><input size="3" type="text" name="loadFactor" class="loadFactor" value="<?=$this->item->loadFactor?>" onblur="blLF(this);"/><?=" ".getLang("load_fact_expl_2")?>
				<input type="image" onclick="javascript:setAutoLF('<?=$this->getFormName()?>', '<?=$GLOBALS["gCurrentStatistics"]->idealLF?>');"
					src="<?=$GLOBALS["gCurrentPath"]?>View/images/CalendarLink.gif"
					alt="<?=getLang("click_calculated_value")?>"
					title="<?=getLang("click_calculated_value")?>"/>
				</td>
				</tr>
				</table>
			
		<?=modifyLineEnd()?>

		<?=modifyLineStart()?>

				<?=getLang("work_on")?>
				<?=linkToHelp("Planning", "workingPace", getLang("work_on"))?>

		<?=modifyLineMiddle()?>

			<?php
			PlanningCardView::displayModifyWorkingDays();
			?>

		<?=modifyLineEnd()?>
		
		<?php
	}
	
	
}

// *********************************************************************
// Class PlanningView
// *********************************************************************
class PlanningView extends TaskAndStoryContainerView  {
	var $oldIterationsAreHidden;
	
	function displayNone($type) {
		switch ($type) {
			case "task":
				displayNone(getLang("no_isolated_task"));
				break;
			case "story":
				displayNone(getLang("no_isolated_story"));
				break;
			case "iteration":
				displayNone("");
				break;
		}
	}

	function getClassFromKey($key) {
		switch ($key) {
			case XP_PLANNING_POP_ITERATION_CURRENT:
			case XP_PLANNING_POP_ITERATION_OTHER:
				return "iteration";
			default:
				return parent::getClassFromKey($key);
		}
	}

	function displayAddIncluded($type) {
		if ($type == XP_PLANNING_POP_ITERATION_CURRENT
			|| $type == XP_PLANNING_POP_ITERATION_OTHER) {
			$this->displayAddIncluded("iteration");
		} else {
			parent::displayAddIncluded($type);
		}
	}

	function doDisplayAddIncluded($type) {
		$arguments = array($type);

		switch ($type) {
			case "iteration":
				break;
			case "story":
				$arguments[] = 0;
				break;
			case "task":
				$arguments[] = 0;
				$arguments[] = 0;
				$arguments[] = $this->item->nTasksResponsibleId;
				$arguments[] = $this->item->nTasksPairId;
				break;
		}

		displayInsert($arguments);
	}

	function displayIntro($type) {

		switch ($type) {
			case "task":
				$this->_newSection(
					XP_PLANNING_POP_TASK,
					getLang("isolated_tasks"),
					$this->item->populate[XP_PLANNING_POP_TASK]);

				if ($this->item->populate[XP_PLANNING_POP_TASK]) {

					displayDivStart("intro");
					PlanningCardView::displayWorkingDays(
						getLang("default_pace"));
					echo "&nbsp;".getLang("load_fact_expl_1");
					echoSpan("bold",XP_DEFAULT_LOAD_FACTOR);
					echo "&nbsp;";
					echo getLang("load_fact_expl_2")."<br/>";
					displayDivEnd();

					$this->hiddenDefaultValues();

					parent::displayIntro($type);
				}
				break;
			case "story":
				$this->_newSection(
					XP_PLANNING_POP_STORY,
					getLang("isolated_stories"),
					$this->item->populate[XP_PLANNING_POP_STORY]);
				break;
			case XP_PLANNING_POP_ITERATION_CURRENT:
				$this->_newSection(
					XP_PLANNING_POP_ITERATION_CURRENT,
					getLang("curr_iteration"),
					$this->item->populate[XP_PLANNING_POP_ITERATION_CURRENT]);
				break;
			case XP_PLANNING_POP_ITERATION_OTHER:
				$this->_newSection(
					XP_PLANNING_POP_ITERATION_OTHER,
					getLang("prev_iterations"),
					$this->item->populate[XP_PLANNING_POP_ITERATION_OTHER]);
				break;
		}
	}

	function displayInsertInit() {
		displayInsertItemInit(array("iteration"));
		displayInsertItemInit(array("story", 0));
		displayInsertItemInit(array("task", 0, 0, $this->item->nTasksResponsibleId, $this->item->nTasksPairId));
	}

	function displayOrderOptions($readOnly = false) {

		// Order by available fields are user-defined
		$orderFields = array();

		preg_match_all("/(\|\b[\w]+\b\|)/", ORDERBY_STORIES_FIELDS, $matches);

		foreach($matches[1] as $fieldName) {
			switch($fieldName) {
				case ORDERBY_ID:
					$orderFields["id"] = "id";
					break;
				case ORDERBY_TITLE:
					$orderFields["name"] = getLang("title");
					break;
				case ORDERBY_RISK:
					$orderFields["risk"] = preg_replace("/:/", "", getLang("risk"));
					break;
				case ORDERBY_WEIGHT:
					$orderFields["weight"] = preg_replace("/:/", "", getLang("weight"));
					break;
				case ORDERBY_PRIORITY:
					$orderFields["priority"] = preg_replace("/:/", "", getLang("priority"));
					break;
			}
		}
		
		$currentOrderBy = "";

		foreach($this->item->storiesOrderBy as $field => $ascOrDesc) {
			$currentOrderBy .= "|$field=$ascOrDesc|";
		}

		displaySimpleInputHidden(XP_PLANNING_POP_STORIES_ORDERBY, $currentOrderBy);

		if (!empty($orderFields)) {

			$i = 0;
			foreach($orderFields as $key => $title) {
				displaySimpleInputHidden("storyOrderBy_$i", $key);
				$i ++;
			}
		?>
		<table class="orderBy" align="center" cellspacing="0" cellpadding="0">
			<tr>
			<td>
				<?=getLangNbsp("order_something_by", getLang("stories"))?>
			</td>
				<?php
				$i = 0;
				foreach($orderFields as $key => $title) {
					$class = "orderNone";
					if (isset($this->item->storiesOrderBy[$key])) {
						if ($this->item->storiesOrderBy[$key] == "ASC") {
							$class = "orderAsc";
						} elseif ($this->item->storiesOrderBy[$key] == "DESC") {
							$class = "orderDesc";
						}
					}

					if ($readOnly) {
						$class .= " readOnly";
					}

					// We don't display unused fields if in readonly mode
					if (!$readOnly || isset($this->item->storiesOrderBy[$key])) {
						?>
						<td id="storyOrderBy<?=$i?>" class="<?=$class?>" <?php if(!$readOnly) {?> onclick="checkOrder(this);" <?}?>>
							<?=$title?>
						</td>
						<?php
					}

					$i++;	

				}
				?>
			<td>
				&nbsp;
			</td>
			</tr>
		</table>
		<?php
		}
	}
	
	function display() {

		if (!$this->item->isAllTasks()) {
			displayDivStart("somebodysTasks");
		}

		$this->displayInsertInit();

		if ($this->isPrint) {
			$this->displayPrintHeader();
		}
	
		// Current iteration
		$this->displayIncludedType(XP_PLANNING_POP_ITERATION_CURRENT, XP_PLANNING_POP_ITERATION_CURRENT);
		
		// Isolated stories
		$this->displayIncludedType("story", XP_PLANNING_POP_STORY);
				
		// Isolated tasks
		$this->displayIncludedType("task", XP_PLANNING_POP_TASK);
		
		// Previous iterations
		$this->displayIncludedType(XP_PLANNING_POP_ITERATION_OTHER, XP_PLANNING_POP_ITERATION_OTHER);

		if (!$this->item->isAllTasks()) {
			displayDivEnd();
		}

	}

	function displayIncludedType($type, $popType) {

		if ($this->item->populate[$popType]) {
			parent::displayIncludedType($type);
		} else {
			//echoDebug("$popType are not popuplated => only intro");
			$this->displayIntro($type);
		}

	}

	function allowedToDisplayInType($item, $type) {
		// Select only current iteration in "current" section,
		// other iterations in "others" section...

		switch ($type) {
			case XP_PLANNING_POP_ITERATION_CURRENT:
				$bAllowed = $item->isCurrent;
				break;
			case XP_PLANNING_POP_ITERATION_OTHER:
				$bAllowed = !$item->isCurrent;
				break;
			default:
				$bAllowed = parent::allowedToDisplayInType($item, $type);
				break;
		}

		if ($type != "task" && !$this->item->isAllTasks()) {

			$bAllowed &= $item->containsTaskWithRespAndPair($this->item->nTasksResponsibleId, $this->item->nTasksPairId);
		}

		return $bAllowed;
	}
	
	function hiddenDefaultValues() {
		displayVisibleInvisibleSectionStart(XP_INVISIBLE, "defaultValues");

		// Need to have "iteration_0" form name...
		$iteration = & Iteration::getObjectToInsert();
		$iteration->toInsert = false;
		$iteration->id = 0;

		$formInsertName = $iteration->view->getFormName();
		?>
        <form name="<?=$formInsertName?>" action="" method="post">
		<?php
		PlanningCardView::displayModifyWorkingDays($this->item->workingDays);
    	displaySimpleInputText("", "loadFactor", $this->item->loadFactor);
    	?>
        </form>
    	<?php
		displayVisibleInvisibleSectionEnd(XP_INVISIBLE, "defaultValues");
	}

	function _newSection($type, $name, $isOpen) {

		$newValue = $isOpen?"0":"1";

		$href = $_SERVER['PHP_SELF']."?$type=$newValue".($this->isPrint?"&amp;print":"");

		/*
		if ($this->isPrint) {
			$href .= "&print";
		}
		*/

		$href .= "#$type";

		if (!($this->isPrint && !$isOpen)) {
			displayDivStart("section");
			?>
			<a id="<?=$type?>" class="<?=$isOpen?"open":"closed"?>" href="<?=$href?>"><?=$name?></a>
			<?php
			displayDivEnd();
		}
	}

	function displayPrintHeader() {

		$this->displayOrderOptions(true);

		$this->displayMemberHeader();

	}

	function displayMemberHeader() {
		if (!$this->item->isAllTasks()) {

    		echo "<br/>";
    		displayDivStart("printHeader");
    		echo $this->getFromMemberLegend();
    		displayDivEnd();
    		echo "<br/>";

		}
	}

	function displayChooseOptions() {

		if (!$this->isPrint) {
			displayDivStart("planningOptions");
			?>
			<form name="ChooseOptions" action="<?=$GLOBALS["gThisPage"]?>.php" method="get">
			<table class="planningOptions" align="center">
			<tr>
				<td class="planningOptionsSep">
				<?php
				$this->displayBasicOptions();
				?>
				</td>
				<td class="planningOptionsSep">
				<?php
				$this->displaySpecificOptions();
				?>
				</td>
						
				<td class="planningOptions">
					<a class="okOptions" href="javascript:document.forms['ChooseOptions'].submit();"><?=getLang("ok")?></a>
				</td>
			</tr>
			</table>
			</form>
			<?php
			displayDivEnd();
		}
	}

	function displaySpecificOptions() {
		$this->displayOrderOptions();
	}

	function displayBasicOptions() {
		$this->displayListOfUsersTasksOptions();
		echo getLang("pair_with");
		$this->displayListOfUsersTasksOptions(true);
	}

	function displayListOfUsersTasksOptions($isPair = false) {
		;
		

		$options = & new HTMLSelect($isPair?XP_PLANNING_POP_PAIR:XP_PLANNING_POP_RESPONSIBLE,
			"checkSelect(this);".($isPair?"":"changeUserTasks(this, '".XP_PLANNING_POP_PAIR."');"),
			"planningOptionsSelect");

		$currentTaskMember = $isPair?$this->item->nTasksPairId:$this->item->nTasksResponsibleId;
		
		// All tasks
		$options->add(new HTMLSelectOption(
			ALL_TASKS,
			getPairOrUserDisplayName(ALL_TASKS, $isPair),
			$currentTaskMember == ALL_TASKS)
			);

		// Separator
		$options->add(new HTMLSelectOption(
			XP_FORBIDDEN_ID,
			"",
			false,
			true)
			);

		// If I'm member of project, My tasks
		if ($GLOBALS["gConnectedUser"]->isMemberOf(getCurrentProjectId())) {

			$options->add(new HTMLSelectOption(
				$GLOBALS["gConnectedUser"]->id,
				getPairOrUserDisplayName($GLOBALS["gConnectedUser"]->id, $isPair),
				$currentTaskMember == $GLOBALS["gConnectedUser"]->id)
				);

			// Separator
			$options->add(new HTMLSelectOption(
				XP_FORBIDDEN_ID2,
				"",
				false,
				true)
				);
		}
		
		// All members' tasks
		$oneMember = false;

		// To know if there is at least one member (else than me)
		reset($GLOBALS["gBasicMembers"]);
		foreach ($GLOBALS["gBasicMembers"] as $id => $_basicMember) {

			if ($id != $GLOBALS["gConnectedUser"]->id) {
				
				$options->add(new HTMLSelectOption(
					$id,
					getPairOrUserDisplayName($id, $isPair),
					$currentTaskMember == $id)
					);

				$oneMember = true;
			}
		}

		if ($oneMember) {
			// Separator
			$options->add(new HTMLSelectOption(
				XP_FORBIDDEN_ID3,
				"",
				false,
				true)
				);
		}
		
		// Not affected
		$options->add(new HTMLSelectOption(
			0,
			getPairOrUserDisplayName(0, $isPair),
			$currentTaskMember == 0)
			);

		$options->display();
		
	}

	function displayChoosePrintOptions() {
			
		displayPrintParamFormStart();
		
		$this->displayBasicOptions();

		?>
		<br/>
		<br/>
		<?php

		$this->displayOrderOptions();

		?>
		<br/>
		<?php
	
		$check1 = & new HTMLCheckbox(
			XP_PLANNING_POP_ITERATION_CURRENT,
			1,
			getLang("show_curr_iter"),
			$this->item->populate[XP_PLANNING_POP_ITERATION_CURRENT]);

		$check2 = & new HTMLCheckbox(
			XP_PLANNING_POP_STORY,
			1,
			getLang("show_isol_stories"),
			$this->item->populate[XP_PLANNING_POP_STORY]);

		$check3 = & new HTMLCheckbox(
			XP_PLANNING_POP_TASK,
			1,
			getLang("show_isol_tasks"),
			$this->item->populate[XP_PLANNING_POP_TASK]);

		$check4 = & new HTMLCheckbox(
			XP_PLANNING_POP_ITERATION_OTHER,
			1,
			getLang("show_iterations"),
			$this->item->populate[XP_PLANNING_POP_ITERATION_OTHER]);

		$check1->display();
		echo "<br/>";
		$check2->display();
		echo "<br/>";
		$check3->display();
		echo "<br/>";
		$check4->display();

		displayPrintParamFormEnd();

	}

	function displayJSHeader() {
		
		global $gXPLanguage;
		?>
		<!-- import the calendar script -->
		<script type="text/javascript" src="<?=$GLOBALS["gCurrentPath"]?><?=XP_JSCALENDAR_BASEDIR?>/calendar.js"></script>

		<!-- import the DEFAULT language module -->
		<script type="text/javascript" src="<?=$GLOBALS["gCurrentPath"]?><?=XP_JSCALENDAR_BASEDIR?>/lang/calendar-en.js"></script>

		<!-- import the language module -->
		<script type="text/javascript" src="<?=$GLOBALS["gCurrentPath"]?><?=XP_JSCALENDAR_BASEDIR?>/lang/calendar-<?=getLang("_special_calendar_language")?>.js"></script>

		<!-- helper script that uses the calendar -->
		<script type="text/javascript" src="<?=$GLOBALS["gCurrentPath"]?>View/XPWebCalendar.js" charset="<?=getLang("_special_charset")?>"></script>

		<!-- Planning js vars -->
		<?=XP_JAVASCRIPT_START?>
		<?=$this->displayJSVars()?>
		</script>

		<?php

	}

	function displayJSVars() {
		// Variables that we need
		global $XP_WEEK_FORMAT;
		global $XP_SEPERATE_OPT;
		global $XP_DATE_FORMAT;
		global $XP_EVERY_YEAR_HOLIDAYS;
		global $XP_SPECIFIC_HOLIDAYS;
		
			
		//------------------------------------------------------------------
		//      Global javascript variables
		//------------------------------------------------------------------
		
		// *** Prepare list of other projects
		// ***********************
		$myProjectsList = array();
		$myProjectsString = "";

		if(XP_ALLOW_MOVE_OTHER_PROJECTS) {

			if ($GLOBALS["gConnectedUser"]->isAdmin || $GLOBALS["gConnectedUser"]->hasAdminProfile()) {
				// Allowed to move to all projects...
				
				$realFactory = & $GLOBALS["gFactory"];
				
				require_once "Model/Projects_model.inc.php";
				require_once "Controler/Projects_controler.inc.php";
							
				$projectList = $GLOBALS["gProjectsFactory"]->newProjectList();
							
				$myProjectsList = $projectList->controler->dbGetListOfAllProjects();
				
				// Re-set the factory
				$GLOBALS["gFactory"] = & $realFactory;
				
			} else {
				// Is allowed to move to projects he's affected to.
				$myProjectsList = $GLOBALS["gConnectedUser"]->affectations;
				
			}
			
			unset($myProjectsList[getCurrentProjectId()]);
			reset($myProjectsList);
			// If there's something, add at the beginning

			if (!empty($myProjectsList)) {
				$myProjectsString .= ",'".XP_FORBIDDEN_ID4."':\"$XP_SEPERATE_OPT\"";
				$myProjectsString .= ",'".XP_FORBIDDEN_ID5."':\"-- ".ucfirst(getLang("projects")).": --\"";
				
				reset($myProjectsList);
				while (list($id,$name) = each($myProjectsList)) {
					$myProjectsString .= ",'proj_".$id."':\"".getLang("type_project")." $id - ".formatMenuItem($name)."\"";
				}
				reset($myProjectsList);
			}
		}
					
		?>

		gMaxPriority	= <?=XP_STORY_MAX_PRIORITY?>;
		gMaxRisk	= <?=XP_STORY_MAX_RISK?>;
		gMaxWeight	= <?=XP_STORY_MAX_WEIGHT?>;

		<?php
		// *** Holidays per iteration
		// **************************

		$jsNonWorkingDays = "{";

		if (!empty($this->item->basicIterations)){
			reset($this->item->basicIterations);

			// for all iterations
			while (list($_key, $basicIteration) = each($this->item->basicIterations)) {

				$jsNonWorkingDays .= "'$basicIteration->id': '";

				// for all working days
				while (list($day, $isWorking) = each($basicIteration->workingDays)) {
					if (!$isWorking) {
						$jsNonWorkingDays .= "|$day|";
					}
				}

				$jsNonWorkingDays .= "', ";
			}
			
			reset($this->item->basicIterations);
		}

		$jsNonWorkingDays .= "'':''}";
		?>
		
		gIterationsNonWorkingDays = <?=$jsNonWorkingDays?>;

		<?php
		// *** Days of the week
		// ***********************
		$jsAllDays = "{";
		reset($XP_WEEK_FORMAT);
		while (list($key, $value) = each($XP_WEEK_FORMAT)) {
			$jsAllDays .= "'$key': '$value',";
		}
		$jsAllDays .= "'':''}";
		?>
			
		gAllDays = <?=$jsAllDays?>;

		<?php
		// *** Holidays & date format
		// ***********************
		for ($i=0; $i < sizeof($XP_DATE_FORMAT); $i ++) {
    		if ($XP_DATE_FORMAT[$i] == "Day") {
    			$format[$i] = "%d";
    		}
    		if ($XP_DATE_FORMAT[$i] == "Month") {
    			$format[$i] = "%m";
    		}
    		if ($XP_DATE_FORMAT[$i] == "Year") {
    			$format[$i] = "%Y";
    		}
		}
		$format = "%a, ".$format[0]."/".$format[1]."/".$format[2];
		?>

		gFormat = "<?=$format?>";

		<?php
		// What is first day of week?
		// based on weekformat...
		$dayIndexes = array(
			"Sun" => 0,
			"Mon" => 1,
			"Tue" => 2,
			"Wed" => 3,
			"Thu" => 4,
			"Fri" => 5,
			"Sat" => 6
		);
		
		?>

		gFirstDayOfWeek = <?=$dayIndexes[$XP_WEEK_FORMAT[0]]?>;

		<?php
		// *** HOLIDAYS
		// **********************
		$everyYear = "";
		reset($XP_EVERY_YEAR_HOLIDAYS);
		while (list($key, $value) = each($XP_EVERY_YEAR_HOLIDAYS)) {
    		if ($value) {
    			$everyYear .= "|".ereg_replace("0","",$key)."|";
    		}
		}
	    
		$specific = "";
	    
		reset($XP_SPECIFIC_HOLIDAYS);
		while (list($key, $value) = each($XP_SPECIFIC_HOLIDAYS)) {
    		if ($value) {
    			$specific .= "|".ereg_replace("-0","-",$key)."|";
    		}
		}
		?>

		gEveryYear = "<?=$everyYear?>";
		gSpecific = "<?=$specific?>";

		<?php
		// *** Iterations array
		// ***********************
		$allIterations = "{";
	
		if (!empty($this->item->basicIterations)) {
			reset($this->item->basicIterations);

			$isFirst = true;

			while (list($_key, $basicIteration) = each($this->item->basicIterations)) {
				$id	= $basicIteration->id;
				$title	= $basicIteration->title;

				if ($basicIteration->isCurrent) {
					$title .= " [ ".getLang("curr_iteration")." ]";
				}

				if (!$isFirst) {
					$allIterations .= ", ";
				}

				$allIterations .= "'$id':\"".getLang("type_iteration")." $id - ".formatMenuItem($title)."\"";

				if ($isFirst) {
					$isFirst = false;
				}
			}
			
			reset($this->item->basicIterations);

		}

		// Includes projects
		$allIterationsAndProjects = $allIterations;

		if (!empty($this->item->basicIterations)) {
			$allIterationsAndProjects .= ",";
		}
		
		$allIterationsAndProjects .= "'".XP_FORBIDDEN_ID."':\"$XP_SEPERATE_OPT\"";
		$allIterationsAndProjects .= ",'0':\"[ ".decodeHTML(getLangNoHtml("isolated"))." ]\"";

		// Add other projects at the end...
		// (if any)
		$allIterationsAndProjects .= $myProjectsString;
		$allIterationsAndProjects .= "}";

		// Because ends with a comma
		$allIterations .= "}";
		?>
			
		// Selects
		gAllIterationsAndProjects	= <?=$allIterationsAndProjects?>;
		gAllIterations			= <?=$allIterations?>;

		<?php

		// *** Stories of iterations
		// ***********************
		$iterationsStories = "{";

		
		if (!empty($this->item->basicIterations)){
			reset($this->item->basicIterations);

			// For all iterations
			while (list($_key, $basicIteration) = each($this->item->basicIterations)) {
				$id		= $basicIteration->id;
				$title	= $basicIteration->title;

				if ($basicIteration->isCurrent) {
					$title .= " [ ".getLang("curr_iteration")." ]";
				}
				
				// The iteration itself
				$iterationsStories .= "'".$id."_0':\"".getLang("type_iteration")." $id - ".formatMenuItem($title)."\",";

				if (!empty($basicIteration->basicStories)){
					reset($basicIteration->basicStories);

					while (list($_key, $basicStory) = each($basicIteration->basicStories)) {
						$storyId	= $basicStory->id;
						$storyTitle	= $basicStory->title;

						$iterationsStories .= "'".$id."_".$storyId."':\" - ".getLang("type_story")." $storyId - ".formatMenuItem($storyTitle)."\",";
					}
					reset($basicIteration->basicStories);
				}
			}
			$iterationsStories .= "'".XP_FORBIDDEN_ID."':\"$XP_SEPERATE_OPT\",";
			reset($this->item->basicIterations);
		}
	
		if (!empty($this->item->basicStories)){
			reset($this->item->basicStories);

			while (list($_key, $basicStory) = each($this->item->basicStories)) {
				$id		= $basicStory->id;
				$title	= $basicStory->title;

				$iterationsStories .= "'0_".$id."': \"- ".getLang("type_story")." $id - ".formatMenuItem($title)."\",";
			}
			$iterationsStories .= "'".XP_FORBIDDEN_ID."':\"$XP_SEPERATE_OPT\",";
			reset($this->item->basicStories);
		}

		$iterationsStories .= "'0_0':\"[".decodeHTML(getLangNoHtml("isolated"))."]\"";

		// Add other projects at the end...
		// (if any)
		$iterationsStories .= $myProjectsString;
		$iterationsStories .= "}";
		?>
			
		gIterationStories = <?=$iterationsStories?>;

		<?php
		// *** Project members
		// ***********************
		// Warning: here comas are BEFORE entries...
		$jsMembers = "{";
		$jsMembers .= "'0':\"".decodeHTML(getLangNoHtml("nobody"))."\"";

		if (!empty($GLOBALS["gBasicMembers"])){
			reset($GLOBALS["gBasicMembers"]);

			$jsMembers .= ",'".XP_FORBIDDEN_ID."':\"$XP_SEPERATE_OPT\"";

			while (list($_key, $basicMember) = each($GLOBALS["gBasicMembers"])) {
				$id		= $basicMember->id;
				$title	= $basicMember->title;

				$jsMembers .= ",'$id': \"".formatMenuItem($title)."\"";
			}
			reset($GLOBALS["gBasicMembers"]);
		}

		$jsMembers .= "}";
			
		?>

		gMembers = <?=$jsMembers?>;

		<?php
		// *** Forbidden ids
		// ***********************
		// Warning: here comas are BEFORE entries...
		$forbidden = "{'".XP_FORBIDDEN_ID."':\"\",'".XP_FORBIDDEN_ID2."':\"\",'".XP_FORBIDDEN_ID3."':\"\",'".XP_FORBIDDEN_ID4."':\"\",'".XP_FORBIDDEN_ID5."':\"\"}";
		?>

		gForbiddenIds = <?=$forbidden?>;

		<?php
		
	}


}

// ------------------------ Global functions ---------------------------


function displayIterationSelect($iterationId, $name = "iterationId", $class = "smallInput", $onClick = "") {
	?>
	<select onchange="checkSelect(this);" name="<?=$name?>" class="<?=$class?>" <?=($onClick=="")?"":"onclick=\"$onClick\""?>>
    	<option value="<?=$iterationId?>">&nbsp;</option>
	</select>
	<?php
}

function displayIterationStorySelect($itId, $currentStoryId, $onClick = "") {
	
	$name = "iterationStoryId";
	
	?>
	<select onchange="checkSelect(this);" name="<?=$name?>" class="smallInput" <?=($onClick=="")?"":"onclick=\"$onClick\""?>>
    	<?php
	
	$value = $itId."_".$currentStoryId;
	echo "<option value=\"$value\">&nbsp;</option>";
	?>
	</select>
	<?php
	
	/*
	$iterationSelect = & new HTMLSelect("iterationStoryId", "checkSelect(this);", "smallInput");
	
	if (!empty($this->item->basicIterations)){
		reset($this->item->basicIterations);
	
		// For all iterations
		while (list($_key, $basicIteration) = each($this->item->basicIterations)) {
			$id	= $basicIteration->id;
			$title	= $basicIteration->title;
	
			if ($basicIteration->isCurrent) {
				$title .= " [ ".getLang("curr_iteration")." ]";
			}
			
			
			// The iteration itself
			$option = & new HTMLSelectOption(
				$id."_0",
				getLang("type_iteration")." $id - ".formatMenuItem($title));
			
			$iterationSelect->add($option);
				
			if (!empty($basicIteration->basicStories)){
				reset($basicIteration->basicStories);
	
				while (list($_key, $basicStory) = each($basicIteration->basicStories)) {
					$storyId	= $basicStory->id;
					$storyTitle	= $basicStory->title;
					
					$option = & new HTMLSelectOption(
						$id."_".$storyId,
						" - ".getLang("type_story")." $storyId - ".formatMenuItem($storyTitle));
					
					
					if ($storyId == $currentStoryId) {
						$option->selected = true;
					} else if ($storyId == -$currentStoryId) {
						$option->disabled = true;
					}
						
					$iterationSelect->add($option);
				}
				reset($basicIteration->basicStories);
			}
		}
		
		$option = & new HTMLSelectOption(
			XP_FORBIDDEN_ID,
			$XP_SEPERATE_OPT);
		
		$iterationSelect->add($option);
		reset($this->item->basicIterations);
	}
	
	$iterationSelect->display();
	*/
}



function displayDateFormat() {
	global $XP_DATE_FORMAT;
	$text = getLang(strtolower($XP_DATE_FORMAT[0]))
		."/".getLang(strtolower($XP_DATE_FORMAT[1]))
		."/".getLang(strtolower($XP_DATE_FORMAT[2]));
	echoSpan("modifyDate", "$text");
}

function displayDefaultWorkingDays() {
	//echoSpan("defaultWorkingDay","Default pace<br/>");
}

function getDefaultIterationDeadline() {
	$startTime = mktime(0,0,0,date("m"),date("d")+10,date("Y"));
	$startDate = date("Y-m-d", $startTime);
	return $startDate;
}


function getColorFromCompletion($theoryCompletion, $actualCompletion) {
	//if ($theoryCompletion > 100) {
	//	$theoryCompletion = 100;
	//}
	$diffCompletion = $actualCompletion - $theoryCompletion;
	if ($actualCompletion == 0 && $theoryCompletion == 0) {
		// Not started
		$bgcolor = "#eeeeee";
	} elseif ($diffCompletion < -50) {
		$bgcolor = "#dd653d";
		// red
	} elseif($diffCompletion < -10) {
		$bgcolor = "#eed78c";//"#ffcc66";
		// yellow
	} elseif($diffCompletion < 15 ) {
		$bgcolor = "#96c2e1";
		// blue
	} else {
		$bgcolor = "#92cda7";
		// green
	}
	return $bgcolor;
}

function getClassFromCompletion($theoryCompletion, $actualCompletion) {
	$diffCompletion = $actualCompletion - $theoryCompletion;
		
	if ($diffCompletion < -50) {
		$class = "completionVeryLate";
	} elseif($diffCompletion < -10) {
		$class = "completionLate";
		// yellow
	} elseif($diffCompletion < 15 ) {
		$class = "completionOk";
		// blue
	} else {
		$class = "completionAhead";
		// green
	}
	return $class;
}



function getResponsibleTasksDisplayName($memberId, $callMe = true) {
	
	;

	if ($memberId == 0) {
		
		$text = getLangNoHtml("nobodys");
		
	} elseif ($memberId == $GLOBALS["gConnectedUser"]->id && $callMe) {

		$text = getLangNoHtml("my_tasks");

	} elseif ($memberId == ALL_TASKS) {

		$text = getLangNoHtml("full_planning");

	} else {
	
		$text = getLangNoHtml("tasks_from", $GLOBALS["gBasicMembers"][$memberId]->title);

	}

	return $text;
}

function getUsersTasksDisplayName($memberId, $pairId, $forPrint = false) {

	$text = "";

	if ($memberId != ALL_TASKS) {

		$text .= getResponsibleTasksDisplayName($memberId, !$forPrint);

		if ($pairId != ALL_TASKS) {

			$text .= ", ";

		}

	}

	if ($pairId != ALL_TASKS) {

		$text .= getLangNoHtml("pair_with")." ".getPairsTasksDisplayName($pairId, !$forPrint);
		
	}

	return $text;

}

function getPairsTasksDisplayName($memberId, $callMe = true) {
	

	if ($memberId == 0) {
		
		return getLangNoHtml("nobody");
		
	} elseif ($memberId == $GLOBALS["gConnectedUser"]->id && $callMe) {

		return getLangNoHtml("me");

	} elseif ($memberId == ALL_TASKS) {

		return getLangNoHtml("anybody");

	}
	
	return $GLOBALS["gBasicMembers"][$memberId]->title;

}

function getPairOrUserDisplayName($memberId, $isPair = false) {
	if ($isPair) {
		return getPairsTasksDisplayName($memberId);
	}
	return getResponsibleTasksDisplayName($memberId);
}

$extraCss["calendar"] = $GLOBALS["gCurrentPath"].XP_JSCALENDAR_BASEDIR."/calendar-blue.css";


$onLoadText = "planningOptionsInit();";

?>