<?php

class ProjectsFactoryView extends ProjectsFactory {

	function & newProjectList($arguments = array()) {
		
		// Create the object
		$object = & parent::newProjectList($arguments);

		// Create the associated view
		new ProjectListView($object);

		return $object;
	}

	function & newProject($arguments) {
		
		// Create the object
		$object = & parent::newProject($arguments);

		// Create the associated view
		new ProjectView($object);

		return $object;
	}
		
	function & newProjectLink($arguments) {
		
		// Create the object
		$object = & parent::newProjectLink($arguments);

		// Create the associated view
		new ProjectLinkView($object);
		
		return $object;
	}
	
}

$GLOBALS["gProjectsFactory"] = & new ProjectsFactoryView();
$GLOBALS["gFactory"] = & $GLOBALS["gProjectsFactory"];

class ProjectListView extends XPItemListView {
	var $showCurrentProjectOnly;

	function display() {
		if ($this->isPrint && $this->showCurrentProjectOnly) {
			displayDivStart("printHeader");
			echo "[".getLang("current_project")."]";
			displayDivEnd();
		}

		parent::display();
	}

	function displayMiniAllLinks($key, $active) {
		$this->displayMiniAllLinksWithAction($key, $active);
	}

	function displayOpenCloseAll($incType) {
		return true;
	}

	function setCurrentProjectOnly($showCurrentProjectOnly) {
		$this->showCurrentProjectOnly = $showCurrentProjectOnly;
	}
		
	function displayNone($type) {
		displayNone(getLang("no_curr_project"));
	}

	function allowedToDisplayInType($item, $type) {

		return
			(!XP_CANT_SELECT_CANT_SEE_PROJECT
			|| $item->isAllowedToSelect())
			&&
			(!$this->showCurrentProjectOnly || $item->isCurrent);
	}
	
	function doDisplayAddIncluded($type) {
		displayInsert(array("project", $this->item->availableLinkTypes));
	}
}

class ProjectView extends XPItemWithIncludedView {

	function getClassFromKey(& $key) {
		return "projectLink";
	}

	function display() {
		$this->displayWithAction();
	}
	
	function getIncludedComment() {
		return getLang("setup");
	}
			
	function displayFields($fieldsVisibility = XP_VISIBLE) {
	
		$this->displayContentStart($fieldsVisibility);
			
		displayDivStart("title");
		echo $this->item->title;
		displayDivEnd();
		
		if ($this->item->isCurrent) {
			displaySpanStart("subTitle");
			echoLang("current_project");
			displaySpanEnd();
		}
		displayDivStart("description");
		displayDescription($this->item->description);
		displayDivEnd();
		?>
		<br />
		<?php
		
		$this->displayContentEnd($fieldsVisibility);
	}
	
	function displayModifyContent() {
		displayInputText("title", "name", $this->item->title, getLang("name")." ");
		$this->displayModifyDescription();
	}
	
	function displaydeleteconfirm() {
		global $projectIds;
		global $projectNames;
		if (sizeOf($projectIds) <= 1) {
			?>
			<input type="hidden" name="whatEverything" value="delete"/>
			<?php
			echo getLang("del_link_project")." ";
		} else {
			?>
			<input type="radio" name="whatEverything" value="move" checked="checked" id="whatEverything_move"/>
			<label for="whatEverything_move"><?=getLang("move_all_del_project")?></label>
			<select onchange="checkSelect(this);" onclick="this.form.whatEverything[0].checked = true;" name="moveToProject" class="smallInput">
				<?php
				for ($i=0; $i< sizeof($projectIds); $i ++) {
					$id = $projectIds[$i];
					$name = $projectNames[$i];
					if ($id != -$this->item->id) {
						echo "<option value=\"$id\">$name</option>\r\n";
					}
				}
				?>
			</select>
			<br />
			<input type="radio" name="whatEverything" value="delete"  id="whatEverything_delete"/>
			<label for="whatEverything_delete"><?=getLang("del_link_project")?></label>
			<?php
		}
		
	}
	
	function displayNone($type) {
		displayNone(getLang("no_link_for_proj",
			getLangNoHtml("link_type_".$this->item->availableLinkTypes[$type])));
	}
	
	function displayIncludedItems() {

		if ($this->item->isAllowedToDisplayContent()) {
			
			
			parent::displayIncludedItems();

		} else {

			echoSpan("error", getLang("not_allow_setup_proj"));

		}
	}

	function displayIntro($type) {

		$typeName = $this->item->availableLinkTypes[$type];
		$name = getLang("link_type_".$typeName);
		newSectionStart("includedSection");
		echo $name;
		linkToHelp("Projects", $typeName, $name);
        newSectionEnd();
	}
	
	function doDisplayAddIncluded($type) {

		$linkTypeName = $this->item->availableLinkTypes[$type];
		$nbLinks = sizeOf($this->item->multiIncludedItems[$type]);

		if ($linkTypeName != "cvs" || $nbLinks == 0) {
			// Only one CVS link allowed...

			displayInsert(array(
				"projectLink",
				$this->item->id,
				$type,
				$linkTypeName,
				getLang("new_proj_link")
				));

		} else {

			echoSpan("forbidden", getLang("one_cvs_allow"));

		}
	}
	
	function displayActionOnItem() {
		
		if ($this->item->isCurrent) {

			?>
			<div class="currentProject" title="<?=getLang("current_project")?>">&nbsp;</div>
			<?php

		} elseif(!$this->isPrint && $this->item->isAllowedToSelect()) {

			$formName = "chooseProject_".$this->item->id;
			
			$text = getLang("choose_as_cur_proj");
			
			$this->displaySimpleFormStart($formName);
			displaySimpleInputHidden("newProjectName", urlencode($this->item->title));

			?><button type="button" class="chooseProject" onclick="this.form.submit();" title="<?=$text?>">&nbsp;</button><?php
			
			$this->displaySimpleFormEnd($formName, "chooseProject");

		}
	}
}


class ProjectLinkView extends XPItemView {
	
	// _________ Private functions
	
	function displayFields($fieldsVisibility = XP_VISIBLE) {
		
		$this->displayContentStart($fieldsVisibility);
		
		displayDivStart("title");
		echo getHtmlDbText($this->item->title);
		displayDivEnd();
		displayDivStart("description");
		echo $this->item->url;
		displayDivEnd();
		
		$this->displayContentEnd($fieldsVisibility);
	}
	
	function displayModifyContent() {
		displayInputText("title", "comment", $this->item->title, getLang("title"));
		displayInputText("linkUrl", "url", $this->item->url, getLang("url")." ");
		displayInputHidden("type", $this->item->linkType);
		displayInputHidden("projectId", $this->item->projectId);
	}
}

function displayChooseProjectsPrintParams() {
	displayPrintParamFormStart();
	$autoLogId = "curr";
	?>
	<input id="<?=$autoLogId?>" name="showCurrentProjectOnly" type="checkbox" value="1"/>
	<label for="<?=$autoLogId?>"><?=getLang("show_curr_proj_only")?></label>
	<br/>
	<br/>
	<?php
	displayPrintParamFormEnd();
}


?>
