<?php

class TeamFactoryView extends TeamFactory {

	function & newTeam($arguments) {
		
		// Create the object
		$object = & parent::newTeam($arguments);

		// Create the associated view
		$view = & new TeamView($object);

		return $object;
	}

	function & newMember($arguments) {
		
		// Create the object
		$object = & parent::newMember($arguments);

		// Create the associated view
		$view = & new MemberView($object);

		return $object;
	}

	function & newProfilesList($arguments = array()) {
		
		// Create the object
		$object = & parent::newProfilesList($arguments);

		// Create the associated view
		$view = & new ProfilesListView($object);

		return $object;
	}

	function & newProfile($arguments) {
		
		// Create the object
		$object = & parent::newProfile($arguments);

		// Create the associated view
		$view = & new ProfileView($object);

		return $object;
	}

	function & newProfileRight($arguments) {
		
		// Create the object
		$object = & parent::newProfileRight($arguments);

		// Create the associated view
		$view = & new ProfileRightView($object);

		return $object;
	}
	
}

$GLOBALS["gTeamFactory"] = & new TeamFactoryView();

class TeamView extends XPItemMultiListView {

	function displayMiniAllLinks($key, $active) {
		$this->displayMiniAllLinksWithAction($key, $active);
	}

	function displayIntro($type) {

		if (!$this->isPrint) {

			switch ($type) {
				case XP_TEAM_AFFECTED:
					$this->displayInProject();
					break;
				case XP_TEAM_NOT_AFFECTED:
					$this->displayNotInProject();
					break;
			}

		}
	}

	function displayAddIncluded($type){
			
		if (!$this->isPrint) {
			switch ($type) {
				case XP_TEAM_AFFECTED:
					displayInsert(array("member", $this->item->projectId, $this->item->currentProjectName));
					break;
				case XP_TEAM_NOT_AFFECTED:
					displayInsert(array("member", 0, ""));
					break;
			}
		}
	}


	function getClassFromKey(& $key) {
		return "member";
	}

	function displayListOfMembers($type) {

		$listOfMembers = & $this->item->multiIncludedItems[$type];

		if (sizeOf($listOfMembers) == 0 && !$this->isPrint) {

			displayNone(getLang($type==XP_TEAM_AFFECTED?"no_memb_affect":"no_memb"));

		} else {

			$incName = $this->getIncDivName($type);

			displayDivStart("", $incName);

			$this->displayMiniAllLinks($type, (sizeOf($listOfMembers) != 0));

			for ($j = 0; $j < sizeOf($listOfMembers); $j ++) {
				$memberToDisplay = & $listOfMembers[$j];
				// Notice the minus ("-") before $this->item->projectId

				$memberToDisplay->view->projectIdToAffect = 
					$type==XP_TEAM_AFFECTED?$this->item->projectId:-$this->item->projectId;

				$memberToDisplay->view->isCheckable = true;
				
				$memberToDisplay->view->displayWithAction();
				
			}

			displayDivEnd();

		}

		$this->displayAddIncluded($type);
		

	}
	
	function display() {

		if ($this->isPrint) {
			$this->displayPrintIntro();
		}

		if ($this->item->projectId != 0) {

			$this->displayIntro(XP_TEAM_AFFECTED);
			
			$this->displayListOfMembers(XP_TEAM_AFFECTED);
			
			$this->displayIntro(XP_TEAM_NOT_AFFECTED);
					
		}
		
		$this->displayListOfMembers(XP_TEAM_NOT_AFFECTED);

	}
	
	function displayInProject() {
		newSectionStart();
		echoLang("affected_to_proj",$this->item->currentProjectName);
		newSectionEnd();
	}
	
	function displayNotInProject() {
		newSectionStart();
		echoLang("not_affected_to_proj",$this->item->currentProjectName);
		newSectionEnd();
	}
	
	function displayPrintIntro() {
		newSectionStart();
		echoLang("team_memb");
		newSectionEnd();
	}
	
	
}

class MemberView extends XPItemView {

	var $projectIdToAffect;
	
	/**
	 * Overrides because display in modify mode if modification failed for user
	 */
	 /*
	function displayItem() {
		if ($this->item->errorModified) {
			$this->displayModify();
		} else {
			parent::displayItem();
		}
	}
	*/
	
	
	/**
	* Display a link to affect / deaffect member from the project
	*/
	function displayActionOnItem() {
		
		if (!$this->isPrint && $this->projectIdToAffect != 0
			&& $this->item->isAllowedToOrDeAffectFromToProject($this->projectIdToAffect)) {

			$formName = "affectation_".$this->item->id;
			
			if ($this->projectIdToAffect > 0) {
				$text = getLang("remove_memb_proj");
				$class = "removeUser";
			} else {
				$text = getLang("affect_memb_proj");
				$class = "affectUser";
			}
			
			$this->displaySimpleFormStart($formName);
			displaySimpleInputHidden("memberProjectId", $this->projectIdToAffect);
		
			?><button type="submit" class="<?=$class?>" title="<?=$text?>" onclick="this.form.submit();">&nbsp;</button><?php
			
			$this->displaySimpleFormEnd($formName, "affectation");

		}
	}
	
	/**
	* Display a link to add the current member object
	*/
	
	function displayAddThisMember($visibility = XP_INVISIBLE, $inProject = 0, $inProjectName = "") {
		
		$this->item->id = "insert_".$inProject;
		
		if ($visibility == XP_INVISIBLE) {
			$contentVisibility = XP_VISIBLE;
		} else {
			$contentVisibility = XP_INVISIBLE;
		}
		
		if ($inProject != 0) {
			$text = getLang("create_affect_memb_proj", $inProjectName);	
		} else {
			$text = getLang("create_memb", $inProjectName);	
		}
	
		/** User has to be allowed to:
		* create new members
		* and if specific project, to affect to this project or to any project
		*/
		
		if ($this->item->isAllowedToInsert($inProject)) {
			$this->displayContentStart($contentVisibility);
			$this->displayContentEnd($contentVisibility);
			$this->_displayModify($visibility, $text, $inProject);

			$this->displayXPItemEnd();
		} else {
			displayForbidden($text);
		}
			
			
		
	}
	
	// _________ Private functions
	
	function displayFields($fieldsVisibility = XP_VISIBLE) {
		global $gExistingProfiles;

		$extraStyle = "";
		
		$this->displayContentStart($fieldsVisibility);

		displayDivStart("title");
		echo getHtmlDbText($this->item->title);

		echoSpan("memberProfile","&nbsp;&nbsp;-&nbsp;&nbsp;".$gExistingProfiles[$this->item->profileId]);

		if ($this->item->isCurrent) {
			displaySpanStart("subTitle");
			echoLang("memb_logged_in");
			displaySpanEnd();
		}
		displayDivEnd();
		displayDivStart("description");
		displayDescription($this->item->description);
		displayDivEnd();
		displayDivStart("affectation");
		echo getLang("affected_to")." ";
		reset($this->item->affectations);
		if (list($key,$value) = each($this->item->affectations)) {
			echo $value;
			while(list($key,$value) = each($this->item->affectations)) {
				echo ", ".$value;
			}
		} else {
			echoLang("memb_no_proj");
		}
		displayDivEnd();
		
		$this->displayContentEnd($fieldsVisibility);
	}
	
	/*
	function _displayModify($visibility = XP_INVISIBLE, $comment = "modify", $inProject = 0) {
		
		if ($visibility == XP_INVISIBLE) {
			$linkVisibility = XP_VISIBLE;
		} else {
			$linkVisibility = XP_INVISIBLE;
		}
		
		$this->displayModifyStart($visibility);

		$this->displayModifyContent($inProject);
		
		$this->displayModifyEnd();
		
		if ($comment == "modify") {
			$allowedModify = $this->item->isAllowedToUpdate();
			$allowedDelete = $this->item->isAllowedToDelete();
		} else {
			$allowedModify = true;
			$allowedDelete = false;
		}
		
		$this->displayLinksModifyDeleteNew($comment, $linkVisibility, $allowedModify, $allowedDelete);
	}
	*/

	function displayModifyContent() {
		global $MIN_PASSWORD_LENGTH;
		global $extraValue;
		global $gExistingProfiles;

		if (func_num_args() == 1 && is_array(func_get_arg(0))) {
			// Called with an array as a list of parameters
			$arguments = func_get_arg(0);
		} else {
			// Called with a list of parameters
			$arguments = func_get_args();
		}

		$i = 0;
		$inProject	= @$arguments[$i++];

		displayInputText("title", "name", $this->item->title, getLang("name")." ");
		$this->displayModifyDescription();
		if ($this->item->toInsert) {
			displayInputHidden("memberProjectId", $inProject);
		}
		
		?>
		<?=modifyLineStart()?>

			<?=displaySpanStart("member")?>
			<?=getLang("memb_profile")?>
			<?=linkToHelp("Team", "profiles", getLang("memb_profile"))?>
			<?=displaySpanEnd()?>

		<?=modifyLineMiddle()?>

			<?php
			if ($this->item->isAllowedToChangeProfile()) {
				displaySelectProfile($this->item->profileId);
			} else {
				displayForbiddenSelect("profileId", $this->item->profileId, $gExistingProfiles[$this->item->profileId]);
			}
			?>
		
		<?=modifyLineEnd()?>
		<?php
		
		if ($this->item->toInsert || $this->item->isAllowedToChangePassword()) {
			if ($this->item->toInsert) {
			// Insertion
				$passVisibility = XP_VISIBLE;
			} else {
				$passVisibility = XP_INVISIBLE;
			}
			$passSection = "newPass".$this->getFormName();
			
			// Used by "old visibility mode"
			$isOpen = "false";
			
			if (isset($extraValue)) {
				$isOpen = $extraValue;
			} elseif (isset($_POST["extraValue"])) {
				$isOpen = $_POST["extraValue"];
			}

			?>
			<?=modifyLineStart()?>

				<?php
				if (!$this->item->toInsert) {
					// If item is to insert, change password is necessary.
					$chgPassId = "chP".$this->item->id;
					?>
					<input id="<?=$chgPassId?>" type="checkbox" name="passCheck<?=$this->getFormName()?>" onclick="showHidePassChange(this, '<?=$passSection?>');setItemSize('<?=$this->item->type?>_<?=$this->item->id?>');" <?=($isOpen == "true")?" checked=\"checked\"":""?>/>
					<label for="<?=$chgPassId?>"><?=getLang("memb_change_pass")?></label>
					<?php
				}
				?>
				
			<?=modifyLineMiddle()?>

				<?php
				$style = "style=\"height: 100px;\"";
				displayVisibleInvisibleSectionStart($passVisibility, $passSection);
				echo "<table>";
				$passComment = getLang("memb_new_pass_1")."<br/><span class=\"smallComment\">(".getLang("memb_new_pass_2", $MIN_PASSWORD_LENGTH).")</span>";
				displayInputPassword("member", "password", "", $passComment);
				displayInputPassword("member", "passwordAgain", "", getLang("memb_repeat_pass")." ");
				echo "</table>";
				displayVisibleInvisibleSectionEnd($passVisibility, $passSection);
				?>
				
			<?=modifyLineEnd()?>
			<?php
			
		}
		
		// Deactivated for now
		/*
		if ($this->item->toInsert) {
		// Insertion
			displayInputText("", "question", "", getLang("memb_hint")." ");
			displayInputText("", "answer", "", getLang("memb_anwser"));
		}
		*/

	}
}


class ProfilesListView extends XPItemListView {
		
	function displayOpenCloseAll($incType) {
		$incType = $incType;
		return true;
	}

	function displayIntro() {
		newSectionStart();
		echoLang("profiles");
		newSectionEnd();
	}
}

class ProfileView extends XPItemWithIncludedView {

	function displayInsertInit() {
		// No "insert" for rights...
	}

	function getIncludedComment() {
		return getLang("prof_rights");
	}

	function displayIncludedItems() {
		global $gProtectedSections;
		global $gRightsNames;
		
		// Project id has no meaning here => 0
		$rightsItem = XPItemView::newXPItem(array($this->item->type."_rights", $this->item->id, 0));
		
		$rightsItem->view->displayFormStart(false);
		
		displayCommentInFormStart();

		displayDivStart("leftHalf");

		$previousName1 = "";
		$numSections = sizeOf($this->item->multiIncludedItems);
		$i = 0;
		$secondColumn = false;

		reset($this->item->multiIncludedItems);
		
		$even = !$this->oddEven;
		
		//echoDebug("size: ".sizeof($this->item->multiIncludedItems));

		while (list($sectionId, $rightArray)
			= each($this->item->multiIncludedItems)) {
			// For each available right
			
			$right = $rightArray[0];

			reset($gProtectedSections[$sectionId][2]);
			
			$name1 = $gProtectedSections[$sectionId][0];
			
			//echoDebug("name1: $name1");

			if ($name1 != $previousName1) {
				// Section has changed
										
				if (!$secondColumn && ($i >= ($numSections/2)-1)) {
					
					$secondColumn = true;
					?>
					</fieldset>
					<?php
					displayDivEnd();
					displayDivStart("rightHalf");

				} else {
					
  					if ($previousName1 != "") {
  						?>
  						</fieldset>
  						<?php
  					}
		  			
  				}
				
				$previousName1 = $name1;
				$displayName1 = $name1;
				
				$extraStyle = $even?" even":"";
				$even = !$even;
				
				?>
				<fieldset class="rights<?=$extraStyle?>">
				<legend>
				<?=getLang("protected_section_name1_".$displayName1)?>
				</legend>
				<?php
					
			} else {

				$displayName1 = "";

			}
			
			displayDivStart("leftHalf");

			?>
			<?=getLang("protected_section_name2_".$gProtectedSections[$sectionId][1])?>
			
			<input type="hidden" name="section_<?=$i?>_id" value="<?=$sectionId?>" />
			<?php
			
			displayDivEnd();
			displayDivStart("rightHalf");

			if ($this->item->isAllowedToUpdateRights() && !$this->isPrint) {
				?>
				<select name="right_<?=$sectionId?>">
				<?
				reset($gProtectedSections[$sectionId][2]);

				while(list(, $value) = each($gProtectedSections[$sectionId][2])) {
					$selected = ($value == $right->id)?" selected=\"selected\"":"";
					?>
					<option value="<?=$value?>"<?=$selected?>><?=getLang("right_".$gRightsNames[$value])?></option>
					<?php
				}
				?>
				</select>
				<?php
			} else {
				echoLang("right_".$gRightsNames[$right->id]);
			}

			displayDivEnd();

			$i++;
		}
		?>
		</fieldset>
		<input type="hidden" name="numRights" value="<?=$i?>" />
		<?php
		
		// right half
		displayDivEnd();

		displayCommentInFormEnd();
		
		if ($this->item->isAllowedToUpdateRights() && !$this->isPrint) {

			$rightsItem->view->displayOkCancel(getLang("modify_rights"), false);

		}
	    	$rightsItem->view->displayFormEnd();
	}

	function displayFields($fieldsVisibility = XP_VISIBLE) {
		$this->displayContentStart($fieldsVisibility);
		displayDivStart("title");
		if ($this->item->isDefault) {
			echoLang("profile_name_".$this->item->title);
		} else {
			echo getHtmlDbText($this->item->title);
		}
		displayDivEnd();
		?>
		<br />
		<?php
		displayDivStart("description");
		if ($this->item->isDefault) {
			displayDescription(getLang("profile_description_".$this->item->title));
		} else {
			displayDescription($this->item->description);
		}
		displayDivEnd();
		
		$this->displayContentEnd($fieldsVisibility);
	}
	
	/*
	function _displayModify($visibility = XP_INVISIBLE, $comment = "modify") {
		if ($visibility == XP_INVISIBLE) {
			$linkVisibility = XP_VISIBLE;
		} else {
			$linkVisibility = XP_INVISIBLE;
		}
		
		$this->displayModifyStart($visibility);

		$this->displayModifyContent();
				
		$this->displayModifyEnd();
		
		if ($comment == "modify") {
			$allowedModify = $this->item->isAllowedToUpdateDescription();
			$allowedDelete = $this->item->isAllowedToDelete();
		} else {
			$allowedModify = true;
			$allowedDelete = false;
		}
		$this->displayLinksModifyDeleteNew($comment, $linkVisibility, $allowedModify, $allowedDelete);
		
	}
	*/

	function displayModifyContent() {
		displayInputText("title", "name", $this->item->title, getLang("name")." ");
		$this->displayModifyDescription();
	}
	
	function displayDeleteConfirm() {
		echo getLang("conf_del_profile")." ";
		displaySelectProfile($this->item->id);
	}

	/// Profile is open if modified
	function isOpen() {
		return parent::isOpen() || $this->item->itemWasModified();
	}
}


class ProfileRightView extends XPItemView {
}
function displaySelectProfile($currentProfileId) {
	global $gExistingProfiles;
	global $gDefaultProfileId;
	reset($gExistingProfiles);
	?> 
	<select onchange="checkSelect(this);" name="profileId" class="smallInput">
    <?
        while(list($id,$name) = each($gExistingProfiles)) {
            $value = $id;
			$extra = "";
			if ($id != -$currentProfileId) {
				if (($id == $currentProfileId) || ($currentProfileId < 0 && $id == $gDefaultProfileId)) {
					$extra = " selected=\"selected\"";
				}
				echo "<option value=\"".$value."\"".$extra.">$name</option>\r\n";
			}
		}
    ?>
	</select>
	<?php
}

function displayChooseTeamPrintParams() {
	
	displayPrintParamFormStart();
	$showMembersId = "sMemb";
	?>
	<input id="<?=$showMembersId?>" name="showMembers" type="checkbox" value="1" checked="checked"/>
	<label for="<?=$showMembersId?>"><?=getLang("show_members")?></label>
	<br/>
	<?php
	if ($GLOBALS["gConnectedUser"]->isAllowedToDisplaySection("UserProfiles")) {
		$showProfilesId = "sProf";
		?>
		<input id="<?=$showProfilesId?>" name="showProfiles" type="checkbox" value="1" checked="checked"/>
		<label for="<?=$showProfilesId?>"><?=getLang("show_profiles")?></label>
		<br/>
		<?php
	} else {
		?>
		<input name="showProfiles" type="hidden" value="0"/>
		<?=getLang("forbid_show_profiles")?>
		<br/>
		<?php
	}
	displayPrintParamFormEnd();
}
?>