<?php
class PlanningFactoryXML extends PlanningFactory {

	function & newPlanning($arguments) {

		// Create the object
		$object = & parent::newPlanning($arguments);

		// Create the associated view
		$view = & new PlanningXML($object);

		return $object;
	}

	function & newTask($arguments) {

		// Create the object
		$object = & parent::newTask($arguments);

		// Create the associated view
		$view = & new TaskXML($object);

		return $object;
	}
	
	function & newStory($arguments) {

		// Create the object
		$object = & parent::newStory($arguments);

		// Create the associated view
		$view = & new StoryXML($object);

		return $object;
	}
	
	function & newIteration($arguments) {
		
		// Create the object
		$object = & parent::newIteration($arguments);

		// Create the associated view
		$view = & new IterationXML($object);

		return $object;
	}

	
}

$GLOBALS["gPlanningFactory"] = & new PlanningFactoryXML();

// *********************************************************************
// Class PlanningCardXML
// *********************************************************************
class PlanningCardXML extends XPItemXML {
	
	function buildCustomProperties() {

		parent::buildCustomProperties();
			
		$this->addXMLTextProperty("title");
		$this->addStrippedDescription();
		
	}

	function & buildWorkingDays() {

		$workingElement = & $this->addElement("workingDays");

		while (list($day, $isWorking) = each($this->item->workingDays)) {
			
			$xmlDay = & getXmlDoc()->createElement("day");
			$xmlDay = $workingElement->appendChild($xmlDay);

			$xmlDay->setAttribute("name", $day);
			$xmlDay->setAttribute("working", $isWorking);
		}

		return $workingElement;

	}
	
}
// *********************************************************************
// Class TaskXML
// *********************************************************************
class TaskXML extends PlanningCardXML {

	function buildCustomProperties() {

		parent::buildCustomProperties();
				
		$this->addXMLAttributeProperty("actualCompletion");
		$this->addXMLAttributeProperty("theoryCompletion");
		$this->addXMLAttributeProperty("todo");
		$this->addXMLAttributeProperty("done");
		$this->addXMLAttributeProperty("startTime");
		$this->addXMLAttributeProperty("endTime");
		$this->addXMLAttributeProperty("weight");
		$this->addXMLAttributeProperty("memberId");
		$this->addXMLAttributeProperty("autoUpdate");
		
		if (!$this->item->isDefaultWorkingDays) {

			PlanningCardXML::buildWorkingDays();

		}
	}
	
}



// *********************************************************************
// Class TaskContainerXML
// *********************************************************************
class TaskContainerXML extends XPItemWithIncludedXML  {

}
// *********************************************************************
// Class StoryXML
// *********************************************************************
class StoryXML extends XPItemWithIncludedXML {

	function buildCustomProperties() {

		parent::buildCustomProperties();

		PlanningCardXML::buildCustomProperties();

		$this->addStrippedDescription("validationCriteria");
				
		$this->addXMLAttributeProperty("weight");
		$this->addXMLAttributeProperty("priority");
		$this->addXMLAttributeProperty("risk");
		
	}
	
}

// *********************************************************************
// Class TaskAndStoryContainerXML
// *********************************************************************
class TaskAndStoryContainerXML extends TaskContainerXML {	
}
// *********************************************************************
// Class IterationXML
// *********************************************************************
class IterationXML extends TaskAndStoryContainerXML {
	
	function buildCustomProperties() {

		parent::buildCustomProperties();

		PlanningCardXML::buildCustomProperties();
				
		$this->addXMLAttributeProperty("loadFactor");
		$this->addXMLAttributeProperty("deadlineTime");
		
		if ($this->item->isCurrent) {
			$this->addXMLAttribute("isCurrent", 1);
		}

		PlanningCardXML::buildWorkingDays();
		
	}
}

// *********************************************************************
// Class PlanningXML
// *********************************************************************
class PlanningXML extends TaskAndStoryContainerXML {
	
	function buildCustomProperties() {
			
		$this->addXMLAttribute("type", "planning");
		$this->addXMLAttributeProperty("projectId");
		
	}

	function buildIncludedType($type) {
		if ($type != XP_PLANNING_POP_ITERATION_OTHER && $type != XP_PLANNING_POP_ITERATION_CURRENT) {
			return parent::buildIncludedType($type);
		}
		return null;
	}

}

?>