<?php

class TeamFactoryXML extends TeamFactory {

	function & newTeam($arguments) {
		
		// Create the object
		$object = & parent::newTeam($arguments);

		// Create the associated XML
		$xml = & new TeamXML($object);

		return $object;
	}

	function & newMember($arguments) {
		
		// Create the object
		$object = & parent::newMember($arguments);

		// Create the associated XML
		$xml = & new MemberXML($object);

		return $object;
	}

	function & newProfilesList($arguments = array()) {
		
		// Create the object
		$object = & parent::newProfilesList($arguments);

		// Create the associated XML
		$xml = & new ProfilesListXML($object);

		return $object;
	}

	function & newProfile($arguments) {
		
		// Create the object
		$object = & parent::newProfile($arguments);

		// Create the associated XML
		$xml = & new ProfileXML($object);

		return $object;
	}

	function & newProfileRight($arguments) {
		
		// Create the object
		$object = & parent::newProfileRight($arguments);

		// Create the associated XML
		$xml = & new ProfileRightXML($object);

		return $object;
	}
	
}

$GLOBALS["gTeamFactory"] = & new TeamFactoryXML();

class TeamXML extends XPItemMultiListXML {

}

class MemberXML extends XPItemXML {
	
	function buildCustomProperties() {

		parent::buildCustomProperties();

		$this->addXMLAttributeProperty("profileId");
			
		$this->addXMLTextProperty("title");
		$this->addStrippedDescription();
		
		// Affectations

		$tagName = "affectations";

		$affectations = & $this->addElement($tagName);
		
		reset($this->item->affectations);

		while(list($key,$value) = each($this->item->affectations)) {
			
			$tagName = "affectedTo";
			$xmlAffectedTo = & getXmlDoc()->createElement($tagName);
			$xmlAffectedTo->setAttribute("projectId", $key);
			$xmlAffectedTo = $affectations->appendChild($xmlAffectedTo);

		}
		
	}
	
}


class ProfilesListXML extends XPItemListXML {
	
	
}

class ProfileXML extends XPItemWithIncludedXML {

	function buildCustomProperties() {

		parent::buildCustomProperties();
			
		$this->addXMLTextProperty("title");
		$this->addStrippedDescription();
		
	}

	function buildIncludedType($type) {
		global $gRightsNames;
		global $gProtectedSections;

		$name1 = $gProtectedSections[$type][0];

		$includedSections = $this->xmlElement->getElementsByTagName("section");

		$section = null;

		for ($i = 0; $i < $includedSections->length; $i ++) {
			
			$existingSection = $includedSections->item($i);
			
			if ($existingSection->getAttribute("name") == $name1) {
				// Found the right section...
				$section = & $existingSection;
				break;
			}

		}

		if ($section == null) {
			// Section wasn't found => need to insert

			$section = & getXmlDoc()->createElement("section");
			$section->setAttribute("name", $name1);

			$this->insertIncludedItem($section);
		}

		$name2 = $gProtectedSections[$type][1];

		$right = & $this->item->multiIncludedItems[$type][0];

		$right->view->name = $name2;
		
		$right->view->value = $gRightsNames[$right->id];

		$right->view->buildXML();
			
		$section->appendChild($right->view->xmlElement);
		
	}
	
}


class ProfileRightXML extends XPItemXML {
	var $name;
	var $value;

	function buildCustomProperties() {

		$this->addXMLAttribute("name", $this->name);
		$this->addXMLAttribute("value", $this->value);

	}
}
?>
